@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -214,6 +214,14 @@ void NCompactSearcher::runSearch(long maxDepth) {
     if (! started) {
         // Search initialisation.
         started = true;
+
+        // Do we in fact have no permutation at all to choose?
+        if (maxDepth == 0 || pairing->dest(0, 0).isBoundary(nTets)) {
+            use_(this, useArgs_);
+            use_(0, useArgs_);
+            return;
+        }
+
         orderElt = 0;
         orientation[0] = 1;
     }
@@ -241,16 +249,15 @@ void NCompactSearcher::runSearch(long maxDepth) {
 
         // Move to the next permutation.
 
-        // Be sure to preserve the orientation of the permutation if
-        // necessary.
-        if ((! orientableOnly_) || pairing->dest(face).face == 0)
+        // Be sure to preserve the orientation of the permutation if necessary.
+        if ((! orientableOnly_) || adj.face == 0)
             permIndex(face)++;
         else
             permIndex(face) += 2;
 
         // Are we out of ideas for this face?
         if (permIndex(face) >= 6) {
-            // Head back down to the previous face.
+            // Yep.  Head back down to the previous face.
             permIndex(face) = -1;
             permIndex(adj) = -1;
             orderElt--;
@@ -265,7 +272,7 @@ void NCompactSearcher::runSearch(long maxDepth) {
         }
 
         // We are sitting on a new permutation to try.
-        permIndex(adj) = NPerm::invS3[permIndex(face)];
+        permIndex(adj) = NPerm4::invS3[permIndex(face)];
 
         // Merge edge links and run corresponding tests.
         if (mergeEdgeClasses()) {
@@ -342,7 +349,7 @@ void NCompactSearcher::runSearch(long maxDepth) {
                 permIndex(face) = -1;
                 orderElt--;
 
-                // Pull apart vertex links at the previous level.
+                // Pull apart vertex and edge links at the previous level.
                 if (orderElt >= minOrder) {
                     splitVertexClasses();
                     splitEdgeClasses();
@@ -387,12 +394,10 @@ void NCompactSearcher::runSearch(long maxDepth) {
                     << std::endl;
             if (vertexState[i].bdryTwist[0])
                 std::cerr << "ERROR: vertexState[" << i << "].bdryTwist == "
-                    << static_cast<int>(vertexState[i].bdryTwist[0])
-                    << " at end of search!" << std::endl;
+                    "true at end of search!" << std::endl;
             if (vertexState[i].bdryTwist[1])
                 std::cerr << "ERROR: vertexState[" << i << "].bdryTwist == "
-                    << static_cast<int>(vertexState[i].bdryTwist[1])
-                    << " at end of search!" << std::endl;
+                    "true at end of search!" << std::endl;
         }
         for (unsigned i = 0; i < nTets * 8; i++)
             if (vertexStateChanged[i] != -1)
@@ -417,8 +422,7 @@ void NCompactSearcher::runSearch(long maxDepth) {
                     << edgeState[i].size << " at end of search!" << std::endl;
             if (! edgeState[i].bounded)
                 std::cerr << "ERROR: edgeState[" << i << "].bounded == "
-                    << edgeState[i].bounded << " at end of search!"
-                    << std::endl;
+                    "false at end of search!" << std::endl;
             if (edgeState[i].hadEqualRank)
                 std::cerr << "ERROR: edgeState[" << i << "].hadEqualRank == "
                     "true at end of search!" << std::endl;
@@ -533,7 +537,7 @@ int NCompactSearcher::mergeVertexClasses() {
     int vRep, wRep;
     int vNext[2], wNext[2];
     char vTwist[2], wTwist[2];
-    NPerm p = gluingPerm(face);
+    NPerm4 p = gluingPerm(face);
     char parentTwists, hasTwist, tmpTwist;
     for (v = 0; v < 4; v++) {
         if (v == face.face)
@@ -782,7 +786,7 @@ void NCompactSearcher::splitVertexClasses() {
     int vIdx, wIdx;
     unsigned orderIdx;
     int rep, subRep;
-    NPerm p = gluingPerm(face);
+    NPerm4 p = gluingPerm(face);
     // Do everything in reverse.  This includes the loop over vertices.
     for (v = 3; v >= 0; v--) {
         if (v == face.face)
@@ -871,7 +875,7 @@ bool NCompactSearcher::mergeEdgeClasses() {
 
     bool retVal = false;
 
-    NPerm p = gluingPerm(face);
+    NPerm4 p = gluingPerm(face);
     int v1, w1, v2, w2;
     int e, f;
     int orderIdx;
