@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,9 +26,9 @@
 
 /* end stub */
 
-/*! \file ncensus.h
- *  \brief Deals with forming a census of all triangulations of a given
- *  size.
+/*! \file census/ncensus.h
+ *  \brief Deals with forming a census of all 3-manifold triangulations
+ *  of a given size.
  */
 
 #ifndef __NCENSUS_H
@@ -36,6 +36,7 @@
 #define __NCENSUS_H
 #endif
 
+#include "regina-core.h"
 #include "census/nfacepairing.h"
 #include "utilities/nbooleans.h"
 
@@ -55,29 +56,21 @@ class NTriangulation;
  */
 
 /**
- * A routine used to determine whether a particular triangulation should be
- * included in a census.  Routines of this type are used by
- * NCensus::formCensus().
+ * A legacy typedef that is identical to NCensus::AcceptTriangulation.
+ * See NCensus::AcceptTriangulation for further details.
  *
- * The first parameter passed should be a triangulation currently under
- * consideration.
- * The second parameter may contain arbitrary data as passed to
- * NCensus::formCensus().
- *
- * The return value should be \c true if the triangulation passed should
- * be included in the census, or \c false otherwise.
+ * \deprecated This global typedef is now deprecated.  Please use the
+ * identical class typedef NCensus::AcceptTriangulation instead.
  */
 typedef bool (*AcceptTriangulation)(NTriangulation*, void*);
 
 /**
  * A utility class used to form a complete census of 3-manifold
- * triangulations satisfying certain constraints.  Other tasks (such as
- * finding all completions of a triangulation with boundary) are also
- * offered.
+ * triangulations satisfying certain constraints.
  *
  * \testpart
  */
-class NCensus {
+class REGINA_API NCensus {
     public:
         static const int PURGE_NON_MINIMAL;
             /**< Indicates that non-minimal triangulations may be ignored. */
@@ -97,6 +90,21 @@ class NCensus {
             /**< Indicates that any triangulation containing an embedded
                  two-sided projective plane may be ignored. */
 
+        /**
+         * A routine used to determine whether a particular triangulation
+         * should be included in a census.  Routines of this type are used by
+         * NCensus::formCensus().
+         *
+         * The first parameter passed should be a triangulation currently
+         * under consideration.
+         * The second parameter may contain arbitrary data as passed to
+         * NCensus::formCensus().
+         *
+         * The return value should be \c true if the triangulation passed
+         * should be included in the census, or \c false otherwise.
+         */
+        typedef bool (*AcceptTriangulation)(NTriangulation*, void*);
+
     private:
         NPacket* parent;
             /**< The argument passed to formCensus(). */
@@ -366,8 +374,6 @@ class NCensus {
          * All parameters not explained are taken directly from
          * formCensus().
          *
-         * \pre \a nTetrahedra is at least 1.
-         *
          * @param newProgress the object with which to report progress
          * for the census generation, or 0 if progress reporting is not
          * required.
