@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nfacepairing.h
+/*! \file census/nfacepairing.h
  *  \brief Deals with pairing off tetrahedron faces in a triangulation
  *  census.
  */
@@ -38,6 +38,7 @@
 
 #include <iostream>
 #include <list>
+#include "regina-core.h"
 #include "utilities/nbooleans.h"
 #include "utilities/nthread.h"
 #include "triangulation/nisomorphism.h"
@@ -97,7 +98,7 @@ typedef void (*UseFacePairing)(const NFacePairing*, const NFacePairingIsoList*,
  *
  * \testpart
  */
-class NFacePairing : public NThread {
+class REGINA_API NFacePairing : public NThread {
     private:
         unsigned nTetrahedra;
             /**< The number of tetrahedra under consideration. */
