@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -151,6 +151,8 @@ void NGluingPermSearcher::runSearch(long maxDepth) {
         return;
     }
 
+    // ---------- Selecting the individual gluing permutations ----------
+
     int minOrder = orderElt;
     int maxOrder = orderElt + maxDepth;
 
@@ -162,16 +164,17 @@ void NGluingPermSearcher::runSearch(long maxDepth) {
 
         // TODO: Check for cancellation.
 
-        // When moving to the next permutation, be sure to preserve the
-        // orientation of the permutation if necessary.
+        // Move to the next permutation.
+
+        // Be sure to preserve the orientation of the permutation if necessary.
         if ((! orientableOnly_) || adj.face == 0)
             permIndex(face)++;
         else
             permIndex(face) += 2;
 
+        // Are we out of ideas for this face?
         if (permIndex(face) >= 6) {
-            // Out of ideas for this face.
-            // Head back down to the previous face.
+            // Yep.  Head back down to the previous face.
             permIndex(face) = -1;
             permIndex(adj) = -1;
             orderElt--;
@@ -179,7 +182,7 @@ void NGluingPermSearcher::runSearch(long maxDepth) {
         }
 
         // We are sitting on a new permutation to try.
-        permIndex(adj) = NPerm::invS3[permIndex(face)];
+        permIndex(adj) = NPerm4::invS3[permIndex(face)];
 
         // Is this going to lead to an unwanted triangulation?
         if (mayPurge(face))
@@ -189,7 +192,7 @@ void NGluingPermSearcher::runSearch(long maxDepth) {
                 continue;
 
         // Fix the orientation if appropriate.
-        if (adj.face == 0) {
+        if (adj.face == 0 && orientableOnly_) {
             // It's the first time we've hit this tetrahedron.
             if ((permIndex(face) + (face.face == 3 ? 0 : 1) +
                     (adj.face == 3 ? 0 : 1)) % 2 == 0)
@@ -218,6 +221,7 @@ void NGluingPermSearcher::runSearch(long maxDepth) {
             // Be sure to get the orientation right.
             face = order[orderElt];
             if (orientableOnly_ && pairing->dest(face).face > 0) {
+                // permIndex(face) will be set to -1 or -2 as appropriate.
                 adj = (*pairing)[face];
                 if (orientation[face.tet] == orientation[adj.tet])
                     permIndex(face) = 1;
@@ -279,8 +283,6 @@ NGluingPermSearcher* NGluingPermSearcher::readTaggedData(std::istream& in,
 }
 
 void NGluingPermSearcher::dumpData(std::ostream& out) const {
-    // Assuming nTets < 100, estimated worst case (35 * nTets + 12) bytes total.
-    // Don't quote me on this.
     NGluingPerms::dumpData(out);
 
     out << (orientableOnly_ ? 'o' : '.');
@@ -415,11 +417,11 @@ bool NGluingPermSearcher::badEdgeLink(const NTetFace& face) const {
     // Run around all three edges bounding the face.
     NTetFace adj;
     unsigned tet;
-    NPerm current;
-    NPerm start(face.face, 3);
+    NPerm4 current;
+    NPerm4 start(face.face, 3);
     bool started, incomplete;
     for (unsigned permIdx = 0; permIdx < 3; permIdx++) {
-        start = start * NPerm(1, 2, 0, 3);
+        start = start * NPerm4(1, 2, 0, 3);
 
         // start maps (0,1,2) to the three vertices of face, with
         // (0,1) mapped to the edge that we wish to examine.
@@ -445,7 +447,7 @@ bool NGluingPermSearcher::badEdgeLink(const NTetFace& face) const {
 
             // Push through the current tetrahedron.
             started = true;
-            current = current * NPerm(2, 3);
+            current = current * NPerm4(2, 3);
 
             // Push across a face.
             if (pairing->isUnmatched(tet, current[3])) {
@@ -480,12 +482,12 @@ bool NGluingPermSearcher::lowDegreeEdge(const NTetFace& face,
     // Run around all three edges bounding the face.
     NTetFace adj;
     unsigned tet;
-    NPerm current;
-    NPerm start(face.face, 3);
+    NPerm4 current;
+    NPerm4 start(face.face, 3);
     bool started, incomplete;
     unsigned size;
     for (unsigned permIdx = 0; permIdx < 3; permIdx++) {
-        start = start * NPerm(1, 2, 0, 3);
+        start = start * NPerm4(1, 2, 0, 3);
 
         // start maps (0,1,2) to the three vertices of face, with
         // (0,1) mapped to the edge that we wish to examine.
@@ -514,7 +516,7 @@ bool NGluingPermSearcher::lowDegreeEdge(const NTetFace& face,
             }
 
             // Push through the current tetrahedron.
-            current = current * NPerm(2, 3);
+            current = current * NPerm4(2, 3);
 
             // Push across a face.
             if (pairing->isUnmatched(tet, current[3])) {
