@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,9 +26,9 @@
 
 /* end stub */
 
-/*! \file ngluingpermsearcher.h
- *  \brief Supports searching through all possible gluing permutations sets
- *  for a given face pairing.
+/*! \file census/ngluingpermsearcher.h
+ *  \brief Supports searching through all possible sets of tetrahedron
+ *  gluing permutations for a given tetrahedron face pairing.
  */
 
 #ifndef __NGLUINGPERMSEARCHER_H
@@ -36,6 +36,7 @@
 #define __NGLUINGPERMSEARCHER_H
 #endif
 
+#include "regina-core.h"
 #include "census/ngluingperms.h"
 
 /**
@@ -71,8 +72,8 @@ class NGluingPermSearcher;
 
 /**
  * A routine used to do arbitrary processing upon a particular set of
- * gluing permutations.  Such routines are used to process permutation
- * sets found when running NGluingPermSearcher::findAllPerms().
+ * tetrahedron gluing permutations.  Such routines are used to process
+ * permutation sets found when running NGluingPermSearcher::findAllPerms().
  *
  * The first parameter passed will be a set of gluing permutations
  * (in fact it will be of the subclass NGluingPermSearcher in order to
@@ -89,7 +90,7 @@ typedef void (*UseGluingPerms)(const NGluingPermSearcher*, void*);
 
 /**
  * A utility class for searching through all possible gluing permutation
- * sets that correspond to a given face pairing.  Subclasses of
+ * sets that correspond to a given tetrahedron face pairing.  Subclasses of
  * NGluingPermSearcher correspond to specialised (and heavily optimised)
  * search algorithms that may be used in sufficiently constrained scenarios.
  * The main class NGluingPermSearcher offers a default (but slower) search
@@ -111,7 +112,7 @@ typedef void (*UseGluingPerms)(const NGluingPermSearcher*, void*);
  *
  * \ifacespython Not present.
  */
-class NGluingPermSearcher : public NGluingPerms {
+class REGINA_API NGluingPermSearcher : public NGluingPerms {
     public:
         static const char dataTag_;
             /**< A character used to identify this class when reading
@@ -439,7 +440,7 @@ class NGluingPermSearcher : public NGluingPerms {
          * and it is the responsibility of the caller of this routine to
          * destroy it after use.
          *
-         * The arguments \a use and \a useArgs are the smae as for the
+         * The arguments \a use and \a useArgs are the same as for the
          * NGluingPermSearcher constructor.
          *
          * \warning The data format is liable to change between Regina
@@ -487,7 +488,7 @@ class NGluingPermSearcher : public NGluingPerms {
          * will be based.
          * @return \c true if the permutations under construction will
          * lead to an edge identified with itself in reverse, or \c false
-         * if no such edge is identified.
+         * if no such edge is found.
          */
         bool badEdgeLink(const NTetFace& face) const;
 
@@ -514,7 +515,7 @@ class NGluingPermSearcher : public NGluingPerms {
          * @return \c true if the permutations under construction will
          * lead to a low-degree edge as specified by parameters
          * \a testDegree12 and \a testDegree3, or \c false if no such
-         * edge is identified.
+         * edge is found.
          */
         bool lowDegreeEdge(const NTetFace& face, bool testDegree12,
             bool testDegree3) const;
@@ -568,7 +569,10 @@ class NGluingPermSearcher : public NGluingPerms {
  * guaranteed to lead to bad edge or vertex links.  For details see
  * "Enumeration of non-orientable 3-manifolds using face-pairing graphs and
  * union-find", Benjamin A. Burton, Discrete Comput. Geom. 38 (2007), no. 3,
- * 527--571.
+ * 527--571; and "Detecting genus in vertex links for the fast enumeration
+ * of 3-manifold triangulations", Benjamin A. Burton, in "ISSAC 2011:
+ * Proceedings of the 36th International Symposium on Symbolic and
+ * Algebraic Computation", ACM, 2011, pp. 59-66.
  *
  * No additional unwanted triangulations will be produced by this search
  * (in contrast to other search classes, such as NClosedPrimeMinSearcher).
@@ -576,7 +580,7 @@ class NGluingPermSearcher : public NGluingPerms {
  *
  * \ifacespython Not present.
  */
-class NCompactSearcher : public NGluingPermSearcher {
+class REGINA_API NCompactSearcher : public NGluingPermSearcher {
     protected:
         static const char VLINK_CLOSED;
             /**< Signifies that a vertex link has been closed off (i.e.,
@@ -609,7 +613,7 @@ class NCompactSearcher : public NGluingPermSearcher {
     protected:
         /**
          * A structure used to track equivalence classes of tetrahedron
-         * vertices as the gluing permutation is constructed.  Two
+         * vertices as the gluing permutation set is constructed.  Two
          * vertices are considered equivalent if they are identified
          * within the triangulation.
          *
@@ -780,7 +784,7 @@ class NCompactSearcher : public NGluingPermSearcher {
 
         /**
          * A structure used to track equivalence classes of tetrahedron
-         * edges as the gluing permutation is constructed.  Two edges
+         * edges as the gluing permutation set is constructed.  Two edges
          * are considered equivalent if they are identified within the
          * triangulation.
          *
@@ -1013,7 +1017,7 @@ class NCompactSearcher : public NGluingPermSearcher {
          * union-find tree, i.e., the representative of the equivalence
          * class.
          */
-        int findEdgeClass(int edgeID);
+        int findEdgeClass(int edgeID) const;
 
         /**
          * Returns the representative of the equivalence class containing
@@ -1044,7 +1048,7 @@ class NCompactSearcher : public NGluingPermSearcher {
          * union-find tree, i.e., the representative of the equivalence
          * class.
          */
-        int findEdgeClass(int edgeID, char& twisted);
+        int findEdgeClass(int edgeID, char& twisted) const;
 
         /**
          * Merge the classes of tetrahedron vertices as required by the
@@ -1266,7 +1270,7 @@ class NCompactSearcher : public NGluingPermSearcher {
          * @param vertexID2 the second tetrahedron vertex to examine; this
          * must be between 0 and 4n-1 inclusive, where \a n is the number of
          * tetrahedra.
-         * @return \c true if a one-edge boundary component is formed as
+         * @return \c true if a two-edge boundary component is formed as
          * described above, or \c false otherwise.
          */
         bool vtxBdryLength2(int vertexID1, int vertexID2);
@@ -1316,7 +1320,7 @@ class NCompactSearcher : public NGluingPermSearcher {
  *
  * \ifacespython Not present.
  */
-class NClosedPrimeMinSearcher : public NCompactSearcher {
+class REGINA_API NClosedPrimeMinSearcher : public NCompactSearcher {
     private:
         static const unsigned EDGE_CHAIN_END;
             /**< Represents the end of a one-ended chain in a face
@@ -1580,14 +1584,14 @@ inline char NCompactSearcher::dataTag() const {
     return NCompactSearcher::dataTag_;
 }
 
-inline int NCompactSearcher::findEdgeClass(int edgeID) {
+inline int NCompactSearcher::findEdgeClass(int edgeID) const {
     while (edgeState[edgeID].parent >= 0)
         edgeID = edgeState[edgeID].parent;
 
     return edgeID;
 }
 
-inline int NCompactSearcher::findEdgeClass(int edgeID, char& twisted) {
+inline int NCompactSearcher::findEdgeClass(int edgeID, char& twisted) const {
     for ( ; edgeState[edgeID].parent >= 0; edgeID = edgeState[edgeID].parent)
         twisted ^= edgeState[edgeID].twistUp;
 
