@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nenumconstraint.h
+/*! \file enumerate/nenumconstraint.h
  *  \brief Deals with validity constraints in polytope vertex enumeration.
  */
 
@@ -37,6 +37,7 @@
 
 #include <set>
 #include <vector>
+#include "regina-core.h"
 
 namespace regina {
 
@@ -81,7 +82,7 @@ namespace regina {
  *
  * \ifacespython Not present.
  */
-class NEnumConstraintList : public std::vector<std::set<unsigned> > {
+class REGINA_API NEnumConstraintList : public std::vector<std::set<unsigned> > {
     public:
         /**
          * Creates an empty list of constraints.
