@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nfileinfo.h
+/*! \file file/nfileinfo.h
  *  \brief Deals with determining information about Regina data files.
  */
 
@@ -35,6 +35,7 @@
 #define __NFILEINFO_H
 #endif
 
+#include "regina-core.h"
 #include "shareableobject.h"
 
 namespace regina {
@@ -51,7 +52,7 @@ namespace regina {
  * Routine identify() can be used to determine this information for a
  * given file.
  */
-class NFileInfo : public ShareableObject {
+class REGINA_API NFileInfo : public ShareableObject {
     public:
         static const int TYPE_BINARY;
             /**< Represents an old-style binary data file. */
