@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -50,5 +50,9 @@ std::string NGlobalDirs::engineDocs() {
     return home() + "/engine-docs";
 }
 
+std::string NGlobalDirs::data() {
+    return home() + "/internal/data";
+}
+
 } // namespace regina
 
