@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nglobaldirs.h
+/*! \file file/nglobaldirs.h
  *  \brief Gives information about system installation directories.
  */
 
@@ -36,6 +36,7 @@
 #endif
 
 #include <string>
+#include "regina-core.h"
 
 namespace regina {
 
@@ -59,7 +60,7 @@ namespace regina {
  * support more interesting scenarios, such as running out of the source
  * tree or moving ready-made "application bundles" around the system.
  */
-class NGlobalDirs {
+class REGINA_API NGlobalDirs {
     public:
         /**
          * Returns Regina's primary home directory on the system.  This
@@ -122,6 +123,18 @@ class NGlobalDirs {
          * @return Regina's calculation engine documentation directory.
          */
         static std::string engineDocs();
+
+        /**
+         * Returns the directory containing internal data files for Regina's
+         * calculation engine.
+         *
+         * \warning Currently this routine only returns useful results
+         * if Regina has been properly installed on the system.  See the
+         * NGlobalDirs class notes for further details.
+         *
+         * @return Regina's calculation engine data directory.
+         */
+        static std::string data();
 };
 
 /*@}*/
