@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file dehydration.h
+/*! \file foreign/dehydration.h
  *  \brief Allows reading lists of dehydrated triangulations.
  */
 
@@ -35,6 +35,8 @@
 #define __DEHYDRATION_H
 #endif
 
+#include "regina-core.h"
+
 namespace regina {
 
 class NContainer;
@@ -87,7 +89,7 @@ class NContainer;
  * @return a new container as described above, or 0 if an I/O error occurred
  * whilst reading the given file.
  */
-NContainer* readDehydrationList(const char *filename,
+REGINA_API NContainer* readDehydrationList(const char *filename,
     unsigned colDehydrations = 0, int colLabels = -1,
     unsigned long ignoreLines = 0);
 
