@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file approx.h
+/*! \file maths/approx.h
  *  \brief Provides facilities for working around rounding errors when
  *  dealing with real numbers.
  */
@@ -36,6 +36,8 @@
 #define __APPROX_H
 #endif
 
+#include "regina-core.h"
+
 namespace regina {
 
 /**
