@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -31,13 +31,14 @@
 #define __NMPI_H
 #endif
 
-/*! \file nlargeinteger.h
+/*! \file maths/nlargeinteger.h
  *  \brief Deals with arbitrary precision integers.
  */
 
 #include <gmp.h>
 #include <iostream>
 #include <string>
+#include "regina-core.h"
 
 /**
  * \hideinitializer
@@ -75,9 +76,12 @@ namespace regina {
  * Infinity is catered for.  For comparison operations, infinity is
  * considered larger than any other integer but equal to itself.
  *
+ * All routines in this class, including random number generation, are
+ * thread-safe.
+ *
  * \testpart
  */
-class NLargeInteger {
+class REGINA_API NLargeInteger {
     public:
         static const NLargeInteger zero;
             /**< Globally available zero. */
@@ -94,6 +98,13 @@ class NLargeInteger {
         bool infinite;
             /**< Does this NLargeInteger represent infinity? */
 
+        static gmp_randstate_t randState;
+            /**< Used for generation of random NLargeIntegers.
+                 This is protected by a mutex defined in nlargeinteger.cpp. */
+        static bool randInitialised;
+            /**< Have we yet initialised randState?
+                 This is protected by a mutex defined in nlargeinteger.cpp. */
+
     public:
         /**
          * Initialises this integer to zero.
@@ -711,6 +722,33 @@ class NLargeInteger {
          */
         int legendre(const NLargeInteger& p) const;
 
+        /**
+         * Generate a pseudo-random NLargeInteger that is uniformly
+         * distributed in the interval [0,*this)
+         *
+         * @return a pseudo-random NLargeInteger.
+         */
+        NLargeInteger randomBoundedByThis();
+
+        /**
+         * Generate a pseudo-random NLargeInteger that is uniformly
+         * distributed in the interval [0,2^n).
+         *
+         * @param n the maximum number of bits in the pseudo-random integer.
+         * @return a pseudo-random NLargeInteger.
+         */
+        static NLargeInteger randomBinary(unsigned long n);
+
+        /**
+         * Generate a pseudo-random NLargeInteger that is distributed in the
+         * interval [0,2^n), with a tendency to have long strings of 0s
+         * and 1s in its binary expansion.
+         *
+         * @param n the maximum number of bits in the pseudo-random integer.
+         * @return a pseudo-random NLargeInteger.
+         */
+        static NLargeInteger randomCornerBinary(unsigned long n);
+
     private:
         /**
          * Initialises this integer to infinity.
@@ -733,7 +771,8 @@ class NLargeInteger {
  * @param large the integer to write.
  * @return a reference to \a out.
  */
-std::ostream& operator << (std::ostream& out, const NLargeInteger& large);
+REGINA_API std::ostream& operator << (std::ostream& out,
+    const NLargeInteger& large);
 
 /*@}*/
 
@@ -799,9 +838,8 @@ inline NLargeInteger& NLargeInteger::operator =(const std::string& value) {
     return *this;
 }
 inline void NLargeInteger::swap(NLargeInteger& other) {
-    NLargeInteger tmp(other);
-    other = *this;
-    *this = tmp;
+    std::swap(infinite, other.infinite);
+    mpz_swap(data, other.data);
 }
 
 inline bool NLargeInteger::operator ==(const NLargeInteger& compareTo) const {
