@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nmatrix.h
+/*! \file maths/nmatrix.h
  *  \brief Deals with matrices of elements of various types.
  */
 
@@ -37,6 +37,7 @@
 
 #include <iostream>
 #include <memory>
+#include "regina-core.h"
 
 namespace regina {
 
@@ -309,6 +310,9 @@ class NMatrix {
 /**
  * Represents a matrix of elements from a given ring T.
  *
+ * Note that many important functions (such as entry()) are inherited
+ * from the parent class NMatrix, and are not documented again here.
+ *
  * \pre Type T has a default constructor and overloads the assignment
  * (<tt>=</tt>) operator.
  * \pre An element <i>t</i> of type T can be written to an output stream
