@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nscript.h
+/*! \file packet/nscript.h
  *  \brief Contains a packet representing a script.
  */
 
@@ -38,6 +38,7 @@
 #include <map>
 #include <utility>
 #include <vector>
+#include "regina-core.h"
 #include "file/nfilepropertyreader.h"
 #include "packet/npacket.h"
 
@@ -54,7 +55,7 @@ class NXMLPacketReader;
  * A packet representing a script that can be run.
  * Accessor methods for a script work a line at a time.
  */
-class NScript : public NPacket, public NFilePropertyReader {
+class REGINA_API NScript : public NPacket, public NFilePropertyReader {
     private:
         std::vector<std::string> lines;
             /**< An array storing the lines of this script; none of
@@ -231,30 +232,30 @@ inline const std::string& NScript::getLine(unsigned long index) const {
 }
 
 inline void NScript::addFirst(const std::string& line) {
+    ChangeEventSpan span(this);
     lines.insert(lines.begin(), line);
-    fireChangedEvent();
 }
 inline void NScript::addLast(const std::string& line) {
+    ChangeEventSpan span(this);
     lines.push_back(line);
-    fireChangedEvent();
 }
 inline void NScript::insertAtPosition(const std::string& line,
         unsigned long index) {
+    ChangeEventSpan span(this);
     lines.insert(lines.begin() + index, line);
-    fireChangedEvent();
 }
 inline void NScript::replaceAtPosition(const std::string& line,
         unsigned long index) {
+    ChangeEventSpan span(this);
     lines[index] = line;
-    fireChangedEvent();
 }
 inline void NScript::removeLineAt(unsigned long index) {
+    ChangeEventSpan span(this);
     lines.erase(lines.begin() + index);
-    fireChangedEvent();
 }
 inline void NScript::removeAllLines() {
+    ChangeEventSpan span(this);
     lines.clear();
-    fireChangedEvent();
 }
 
 inline unsigned long NScript::getNumberOfVariables() const {
@@ -262,17 +263,17 @@ inline unsigned long NScript::getNumberOfVariables() const {
 }
 inline bool NScript::addVariable(const std::string& name,
         const std::string& value) {
+    ChangeEventSpan span(this);
     bool ans = variables.insert(std::make_pair(name, value)).second;
-    fireChangedEvent();
     return ans;
 }
 inline void NScript::removeVariable(const std::string& name) {
+    ChangeEventSpan span(this);
     variables.erase(name);
-    fireChangedEvent();
 }
 inline void NScript::removeAllVariables() {
+    ChangeEventSpan span(this);
     variables.clear();
-    fireChangedEvent();
 }
 
 inline void NScript::writeTextShort(std::ostream& o) const {
