@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file ntext.h
+/*! \file packet/ntext.h
  *  \brief Contains a packet representing a text string.
  */
 
@@ -35,6 +35,7 @@
 #define __NTEXT_H
 #endif
 
+#include "regina-core.h"
 #include "packet/npacket.h"
 
 namespace regina {
@@ -49,7 +50,7 @@ class NXMLPacketReader;
 /**
  * A packet representing a text string.
  */
-class NText : public NPacket {
+class REGINA_API NText : public NPacket {
     private:
         std::string text;
             /**< The text string stored in this packet. */
@@ -129,13 +130,13 @@ inline const std::string& NText::getText() const {
 }
 
 inline void NText::setText(const std::string& newText) {
+    ChangeEventSpan span(this);
     text = newText;
-    fireChangedEvent();
 }
 
 inline void NText::setText(const char* newText) {
+    ChangeEventSpan span(this);
     text = newText;
-    fireChangedEvent();
 }
 
 inline void NText::writeTextShort(std::ostream& o) const {
