@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nsnappeatriangulation.h
+/*! \file snappea/nsnappeatriangulation.h
  *  \brief Allows Regina triangulations to interact with the SnapPea kernel.
  */
 
@@ -35,6 +35,7 @@
 #define __NSNAPPEATRIANGULATION_H
 #endif
 
+#include "regina-core.h"
 #include "shareableobject.h"
 
 // Forward declaration of SnapPea structures.
@@ -64,16 +65,18 @@ class NTriangulation;
  * calculation engine and the SnapPea kernel.  Regina code should not
  * interact with the SnapPea kernel other than through this class.
  *
- * Portions of the SnapPea kernel have been built into Regina as of
- * version 4.2 (July 2005).  SnapPea is copyright (c) 1991-2005 by Jeff Weeks,
- * and is distributed under the terms of the GNU General Public License.
+ * SnapPea was written by Jeff Weeks, and is now maintained by both
+ * Jeff Weeks and Nathan Dunfield at http://www.math.uic.edu/t3m/.
+ * The SnapPeaKernel is distributed under the terms of the GNU General
+ * Public License, version 2 or any later version, as published by the
+ * Free Software Foundation.
  *
- * See http://www.geometrygames.org/SnapPea/ for further information on
- * SnapPea.
+ * See http://www.math.uic.edu/t3m/SnapPy/ for further information on
+ * SnapPea and its successor SnapPy.
  *
  * \testpart
  */
-class NSnapPeaTriangulation : public ShareableObject {
+class REGINA_API NSnapPeaTriangulation : public ShareableObject {
     public:
         /**
          * Describes the different types of solution that can be found when
@@ -275,8 +278,10 @@ class NSnapPeaTriangulation : public ShareableObject {
          * Returns whether or not the SnapPea kernel writes diagnostic
          * messages to standard output.
          *
-         * By default such diagnostic messages are enabled.  To disable
-         * them, call disableKernelMessages().
+         * By default such diagnostic messages are disabled.  To enable
+         * them, call enableKernelMessages().
+         *
+         * This routine (which interacts with static data) is thread-safe.
          *
          * @return \c true if and only if diagonstic messages are enabled.
          */
@@ -286,7 +291,9 @@ class NSnapPeaTriangulation : public ShareableObject {
          * Configures whether or not the SnapPea kernel should write
          * diagnostic messages to standard output.
          *
-         * By default such diagnostic messages are enabled.
+         * By default such diagnostic messages are disabled.
+         *
+         * This routine (which interacts with static data) is thread-safe.
          *
          * @param enabled \c true if diagnostic messages should be
          * enabled, or \c false otherwise.
@@ -300,7 +307,9 @@ class NSnapPeaTriangulation : public ShareableObject {
          * Calling this routine is equivalent to calling
          * enableKernelMessages(false).
          *
-         * Note that diagnostic messages are enabled by default.
+         * Note that diagnostic messages are already disabled by default.
+         *
+         * This routine (which interacts with static data) is thread-safe.
          */
         static void disableKernelMessages();
 
@@ -348,18 +357,6 @@ inline void NSnapPeaTriangulation::dump() const {
     saveAsSnapPea("");
 }
 
-inline bool NSnapPeaTriangulation::kernelMessagesEnabled() {
-    return kernelMessages;
-}
-
-inline void NSnapPeaTriangulation::enableKernelMessages(bool enabled) {
-    kernelMessages = enabled;
-}
-
-inline void NSnapPeaTriangulation::disableKernelMessages() {
-    kernelMessages = false;
-}
-
 } // namespace regina
 
 #endif
