@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nsigcensus.h
+/*! \file split/nsigcensus.h
  *  \brief Deals with forming a census of splitting surface signatures.
  */
 
@@ -36,6 +36,7 @@
 #endif
 
 #include <list>
+#include "regina-core.h"
 #include "split/nsignature.h"
 #include "split/nsigisomorphism.h"
 
@@ -102,7 +103,7 @@ typedef void (*UseSignature)(const NSignature&, const NSigIsoList&, void *);
  * function \a use which will be called upon each signature found.
  * @return the total number of non-equivalent signatures that were found.
  */
-unsigned long formSigCensus(unsigned order, UseSignature use,
+REGINA_API unsigned long formSigCensus(unsigned order, UseSignature use,
     void* useArgs = 0);
 
 /**
@@ -112,7 +113,7 @@ unsigned long formSigCensus(unsigned order, UseSignature use,
  *
  * \ifacespython Not present.
  */
-class NSigCensus {
+class REGINA_API NSigCensus {
     private:
         NSignature sig;
             /**< The signature being constructed. */
