@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nsigisomorphism.h
+/*! \file split/nsigisomorphism.h
  *  \brief Deals with full and partial isomorphisms of splitting surface
  *  signatures.
  */
@@ -36,12 +36,11 @@
 #define __NSIGISOMORPHISM_H
 #endif
 
+#include "regina-core.h"
 #include "split/nsignature.h"
 
 namespace regina {
 
-class regina::NSigCensus;
-
 /**
  * \weakgroup split
  * @{
@@ -70,7 +69,7 @@ class regina::NSigCensus;
  *
  * \ifacespython Not present.
  */
-class NSigPartialIsomorphism {
+class REGINA_API NSigPartialIsomorphism {
     private:
         unsigned nLabels;
             /**< The number of symbols whose images are defined. */
