@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nlayeredchainpair.h
+/*! \file subcomplex/nlayeredchainpair.h
  *  \brief Deals with layered chain pair components of a triangulation.
  */
 
@@ -35,6 +35,7 @@
 #define __NLAYEREDCHAINPAIR_H
 #endif
 
+#include "regina-core.h"
 #include "subcomplex/nlayeredchain.h"
 
 namespace regina {
@@ -74,7 +75,7 @@ class NComponent;
  * All optional NStandardTriangulation routines are implemented for this
  * class.
  */
-class NLayeredChainPair : public NStandardTriangulation {
+class REGINA_API NLayeredChainPair : public NStandardTriangulation {
     private:
         NLayeredChain* chain[2];
             /**< The two layered chains that make up this pair. */
