@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file ntrivialtri.h
+/*! \file subcomplex/ntrivialtri.h
  *  \brief Deals with a few specific trivial triangulations.
  */
 
@@ -35,6 +35,7 @@
 #define __NTRIVIALTRI_H
 #endif
 
+#include "regina-core.h"
 #include "subcomplex/nstandardtri.h"
 
 namespace regina {
@@ -53,7 +54,7 @@ namespace regina {
  *
  * \testpart
  */
-class NTrivialTri : public NStandardTriangulation {
+class REGINA_API NTrivialTri : public NStandardTriangulation {
     public:
         /**
          * Represents the two-tetrahedron four-vertex triangulation of
