@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file ntxicore.h
+/*! \file subcomplex/ntxicore.h
  *  \brief Provides various triangulations of the product of the torus
  *  and the interval.
  */
@@ -36,6 +36,7 @@
 #define __NTXICORE_H
 #endif
 
+#include "regina-core.h"
 #include "maths/nmatrix2.h"
 #include "triangulation/ntriangulation.h"
 
@@ -72,7 +73,7 @@ namespace regina {
  * class without reason).  This triangulation can be accessed through the
  * core() routine.
  */
-class NTxICore : public ShareableObject {
+class REGINA_API NTxICore : public ShareableObject {
     protected:
         NTriangulation core_;
             /**< A full copy of the <tt>T x I</tt> triangulation that is
@@ -80,7 +81,7 @@ class NTxICore : public ShareableObject {
         unsigned bdryTet_[2][2];
             /**< The tetrahedra that provide the upper and lower
                  boundary faces.  See bdryTet() for details. */
-        NPerm bdryRoles_[2][2];
+        NPerm4 bdryRoles_[2][2];
             /**< Describes which tetrahedron vertices play which roles
                  in the upper and lower boundary faces.  See bdryRoles()
                  for details. */
@@ -172,7 +173,7 @@ class NTxICore : public ShareableObject {
          * @return the permutation mapping roles 0, 1 and 2 in the
          * diagram above to real tetrahedron vertex numbers.
          */
-        NPerm bdryRoles(unsigned whichBdry, unsigned whichFace) const;
+        NPerm4 bdryRoles(unsigned whichBdry, unsigned whichFace) const;
         /**
          * Returns a 2-by-2 matrix describing the \a alpha and \a beta curves
          * on a torus boundary in terms of specific tetrahedron edges.
@@ -341,7 +342,7 @@ class NTxICore : public ShareableObject {
  *
  * \image html diag92.png
  */
-class NTxIDiagonalCore : public NTxICore {
+class REGINA_API NTxIDiagonalCore : public NTxICore {
     private:
         unsigned long size_;
             /**< The number of tetrahedra in this <tt>T x I</tt>
@@ -411,7 +412,7 @@ class NTxIDiagonalCore : public NTxICore {
  *
  * \image html parallel.png
  */
-class NTxIParallelCore : public NTxICore {
+class REGINA_API NTxIParallelCore : public NTxICore {
     public:
         /**
          * Creates a new copy of this <tt>T x I</tt> triangulation.
@@ -438,7 +439,8 @@ inline unsigned NTxICore::bdryTet(unsigned whichBdry, unsigned whichFace)
     return bdryTet_[whichBdry][whichFace];
 }
 
-inline NPerm NTxICore::bdryRoles(unsigned whichBdry, unsigned whichFace) const {
+inline NPerm4 NTxICore::bdryRoles(unsigned whichBdry, unsigned whichFace)
+        const {
     return bdryRoles_[whichBdry][whichFace];
 }
 
