@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -143,7 +143,7 @@ namespace {
                      towards vertex i), then link_[i] is the inner
                      tetrahedron containing this face.  Otherwise, link_[i]
                      is null. */
-            NPerm linkVertices_[4];
+            NPerm4 linkVertices_[4];
                 /**< If link_[i] is non-zero, then linkVertices_[i] is
                      a mapping from vertices of the inner tetrahedron
                      \a link_[i] to vertices of the outer tetrahedron
@@ -175,12 +175,6 @@ namespace {
              */
             void joinTo(int face, Block* other);
 
-            /**
-             * Inserts all of the inner tetrahedra for this block into
-             * the given triangulation.
-             */
-            void insertInto(NTriangulation* tri);
-
             /**
              * Creates a new inner tetrahedron within this block.
              * It is assumed that this tetrahedron is to be used for
@@ -191,6 +185,11 @@ namespace {
              * This routine assumes that the innerTet_ array has enough
              * space for a new tetrahedron (which should be true if the
              * correct arguments were passed to the Block constructor).
+             *
+             * The new tetrahedron will be automatically added to the
+             * same triangulation as the previous tetrahedra in this block.
+             *
+             * \pre This block already contains at least one inner tetrahedron.
              */
             NTetrahedron* layeringTetrahedron();
 
@@ -211,9 +210,13 @@ namespace {
              * This constructor creates \a initialNumTet inner tetrahedra,
              * but also leaves enough extra room in the inner tetrahedron
              * array for up to \a maxLayerings layerings on the boundaries.
+             *
+             * All new inner tetrahedra created now and in subsequent
+             * layerings will be automatically inserted into the given
+             * triangulation.
              */
             Block(NTetrahedron* outerTet, unsigned initialNumTet,
-                unsigned maxLayerings);
+                unsigned maxLayerings, NTriangulation* insertInto);
     };
 
     /**
@@ -239,8 +242,12 @@ namespace {
              *
              * Equivalently, the block type describes which vertex of
              * the outer tetrahedron this triangular prism surrounds.
+             *
+             * All new inner tetrahedra will be automatically
+             * inserted into the given triangulation.
              */
-            TriPrism(NTetrahedron *outerTet, int type);
+            TriPrism(NTetrahedron *outerTet, int type,
+                NTriangulation* insertInto);
     };
 
     /**
@@ -263,8 +270,12 @@ namespace {
              * The given block type is an integer between 0 and 2
              * inclusive, describing which quadrilateral type in the
              * outer tetrahedron supplies the two ends of the prism.
+             *
+             * All new inner tetrahedra will be automatically
+             * inserted into the given triangulation.
              */
-            QuadPrism(NTetrahedron *outerTet, int type);
+            QuadPrism(NTetrahedron *outerTet, int type,
+                NTriangulation* insertInto);
     };
 
     /**
@@ -287,8 +298,12 @@ namespace {
              * The given block type is an integer between 0 and 5
              * inclusive, describing which edge of the outer tetrahedron
              * this half-tetrahedron does not meet at all.
+             *
+             * All new inner tetrahedra will be automatically
+             * inserted into the given triangulation.
              */
-            TruncHalfTet(NTetrahedron *outerTet, int type);
+            TruncHalfTet(NTetrahedron *outerTet, int type,
+                NTriangulation* insertInto);
     };
 
     /**
@@ -305,8 +320,11 @@ namespace {
             /**
              * Creates a new truncated tetrahedron within the given outer
              * tetrahedron.
+             *
+             * All new inner tetrahedra will be automatically
+             * inserted into the given triangulation.
              */
-            TruncTet(NTetrahedron *outerTet);
+            TruncTet(NTetrahedron *outerTet, NTriangulation* insertInto);
     };
 
     /**
@@ -330,7 +348,7 @@ namespace {
         protected:
             Block* block_;
                 /**< The block whose boundary this is a piece of. */
-            NPerm outerVertices_;
+            NPerm4 outerVertices_;
                 /**< A mapping from the outer vertex numbers 0, 1 and 2
                      to the corresponding vertex numbers in the
                      outer tetrahedron (block_->outerTet). */
@@ -358,7 +376,7 @@ namespace {
              * given mapping from outer vertex numbers to vertices of
              * the outer tetrahedron.
              */
-            Bdry(Block* block, NPerm outerVertices);
+            Bdry(Block* block, NPerm4 outerVertices);
     };
 
     /**
@@ -373,7 +391,7 @@ namespace {
             NTetrahedron* innerTet_[2];
                 /**< The two inner tetrahedra of the block that supply the
                      two inner boundary faces for this quadrilateral. */
-            NPerm innerVertices_[2];
+            NPerm4 innerVertices_[2];
                 /**< For the ith inner boundary face, the permutation
                      innerVertices_[i] maps the inner vertex numbers
                      0, 1 and 2 to the corresponding vertex numbers in
@@ -391,7 +409,7 @@ namespace {
              * given mapping from outer vertex numbers to vertices of
              * the outer tetrahedron.
              */
-            BdryQuad(Block* block, NPerm outerVertices);
+            BdryQuad(Block* block, NPerm4 outerVertices);
 
             /**
              * Layers a new tetrahedron upon the quadrilateral boundary, so
@@ -420,7 +438,7 @@ namespace {
             NTetrahedron* innerTet_[4];
                 /**< The four inner tetrahedra of the block that supply the
                      four inner boundary faces for this quadrilateral. */
-            NPerm innerVertices_[4];
+            NPerm4 innerVertices_[4];
                 /**< For the ith inner boundary face, the permutation
                      innerVertices_[i] maps the inner vertex numbers
                      0, 1 and 2 to the corresponding vertex numbers in
@@ -438,7 +456,7 @@ namespace {
              * given mapping from outer vertex numbers to vertices of
              * the outer tetrahedron.
              */
-            BdryHex(Block* block, NPerm outerVertices);
+            BdryHex(Block* block, NPerm4 outerVertices);
 
             /**
              * Layers four new tetrahedra upon the hexagon boundary, so
@@ -541,8 +559,14 @@ namespace {
              * This contructor also creates the four small tetrahedra in
              * the vertex neighbourhoods, and glues them to the four
              * blocks closest to the outer tetrahedron vertices.
+             *
+             * All new inner tetrahedra (that is, the inner tetrahedra
+             * from the triangulated blocks and also the small
+             * tetrahedra in the vertex neighbourhoods) will be automatically
+             * inserted into the given triangulation.
              */
-            TetBlockSet(const NNormalSurface* s, unsigned long tetIndex);
+            TetBlockSet(const NNormalSurface* s, unsigned long tetIndex,
+                NTriangulation* insertInto);
 
             /**
              * Destroys all block and boundary structures within this outer
@@ -598,19 +622,6 @@ namespace {
              * See the data member \a vertexNbd_ for further details.
              */
             NTetrahedron* vertexNbd(int vertex);
-
-            /**
-             * Insert all of the triangulated blocks within this outer
-             * tetrahedron into the new triangulation.
-             *
-             * Specifically, for all blocks within this outer tetrahedron,
-             * all of the corresponding \e inner tetrahedra are inserted
-             * into the given triangulation.
-             *
-             * The small tetrahedra that make up the extra vertex
-             * neighbourhoods are inserted also.
-             */
-            void insertInto(NTriangulation* tri);
     };
 
     inline Block::~Block() {
@@ -626,13 +637,9 @@ namespace {
         bdry_[face]->joinTo(other->bdry_[outerTet_->adjacentFace(face)]);
     }
 
-    inline void Block::insertInto(NTriangulation* tri) {
-        for (unsigned i = 0; i < nInnerTet_; ++i)
-            tri->addTetrahedron(innerTet_[i]);
-    }
-
     inline NTetrahedron* Block::layeringTetrahedron() {
-        return (innerTet_[nInnerTet_++] = new NTetrahedron());
+        return (innerTet_[nInnerTet_++] =
+            innerTet_[0]->getTriangulation()->newTetrahedron());
     }
 
     inline void Block::attachVertexNbd(NTetrahedron* nbd, int vertex) {
@@ -641,60 +648,62 @@ namespace {
     }
 
     inline Block::Block(NTetrahedron *outerTet, unsigned initialNumTet,
-            unsigned maxLayerings) :
+            unsigned maxLayerings, NTriangulation* insertInto) :
             outerTet_(outerTet),
             innerTet_(new NTetrahedron*[initialNumTet + maxLayerings]),
             nInnerTet_(initialNumTet) {
         unsigned i;
         for (i = 0; i < nInnerTet_; ++i)
-            innerTet_[i] = new NTetrahedron();
+            innerTet_[i] = insertInto->newTetrahedron();
         std::fill(link_, link_ + 4, static_cast<NTetrahedron*>(0));
     }
 
-    TriPrism::TriPrism(NTetrahedron *outerTet, int type) :
-            Block(outerTet, 3, 3) {
-        innerTet_[1]->joinTo(1, innerTet_[0], NPerm());
-        innerTet_[1]->joinTo(3, innerTet_[2], NPerm());
+    TriPrism::TriPrism(NTetrahedron *outerTet, int type,
+            NTriangulation* insertInto) :
+            Block(outerTet, 3, 3, insertInto) {
+        innerTet_[1]->joinTo(1, innerTet_[0], NPerm4());
+        innerTet_[1]->joinTo(3, innerTet_[2], NPerm4());
 
-        NPerm vertices = NPerm(0, type);
+        NPerm4 vertices = NPerm4(0, type);
 
         BdryQuad* q;
 
         bdry_[vertices[0]] = 0;
 
-        q = new BdryQuad(this, vertices * NPerm(0, 2, 3, 1));
+        q = new BdryQuad(this, vertices * NPerm4(0, 2, 3, 1));
         q->innerTet_[0] = innerTet_[1];
         q->innerTet_[1] = innerTet_[2];
-        q->innerVertices_[0] = NPerm(2, 3, 1, 0);
-        q->innerVertices_[1] = NPerm(1, 3, 2, 0);
+        q->innerVertices_[0] = NPerm4(2, 3, 1, 0);
+        q->innerVertices_[1] = NPerm4(1, 3, 2, 0);
         bdry_[vertices[1]] = q;
 
-        q = new BdryQuad(this, vertices * NPerm(2, 3));
+        q = new BdryQuad(this, vertices * NPerm4(2, 3));
         q->innerTet_[0] = innerTet_[0];
         q->innerTet_[1] = innerTet_[2];
-        q->innerVertices_[0] = NPerm(2, 1, 0, 3);
-        q->innerVertices_[1] = NPerm(0, 3, 2, 1);
+        q->innerVertices_[0] = NPerm4(2, 1, 0, 3);
+        q->innerVertices_[1] = NPerm4(0, 3, 2, 1);
         bdry_[vertices[2]] = q;
 
         q = new BdryQuad(this, vertices);
         q->innerTet_[0] = innerTet_[0];
         q->innerTet_[1] = innerTet_[1];
-        q->innerVertices_[0] = NPerm(3, 1, 0, 2);
-        q->innerVertices_[1] = NPerm(0, 1, 3, 2);
+        q->innerVertices_[0] = NPerm4(3, 1, 0, 2);
+        q->innerVertices_[1] = NPerm4(0, 1, 3, 2);
         bdry_[vertices[3]] = q;
 
         link_[vertices[0]] = innerTet_[0];
-        linkVertices_[vertices[0]] = vertices * NPerm(0, 1, 3, 2);
+        linkVertices_[vertices[0]] = vertices * NPerm4(0, 1, 3, 2);
     }
 
-    QuadPrism::QuadPrism(NTetrahedron *outerTet, int type) :
-            Block(outerTet, 5, 4) {
-        innerTet_[4]->joinTo(2, innerTet_[0], NPerm());
-        innerTet_[4]->joinTo(3, innerTet_[1], NPerm());
-        innerTet_[4]->joinTo(0, innerTet_[2], NPerm());
-        innerTet_[4]->joinTo(1, innerTet_[3], NPerm());
+    QuadPrism::QuadPrism(NTetrahedron *outerTet, int type,
+            NTriangulation* insertInto) :
+            Block(outerTet, 5, 4, insertInto) {
+        innerTet_[4]->joinTo(2, innerTet_[0], NPerm4());
+        innerTet_[4]->joinTo(3, innerTet_[1], NPerm4());
+        innerTet_[4]->joinTo(0, innerTet_[2], NPerm4());
+        innerTet_[4]->joinTo(1, innerTet_[3], NPerm4());
 
-        NPerm vertices(
+        NPerm4 vertices(
             regina::vertexSplitDefn[type][0],
             regina::vertexSplitDefn[type][2],
             regina::vertexSplitDefn[type][1],
@@ -702,47 +711,48 @@ namespace {
 
         BdryQuad* q;
 
-        q = new BdryQuad(this, vertices * NPerm(2, 3, 1, 0));
+        q = new BdryQuad(this, vertices * NPerm4(2, 3, 1, 0));
         q->innerTet_[0] = innerTet_[2];
         q->innerTet_[1] = innerTet_[1];
-        q->innerVertices_[0] = NPerm(1, 0, 2, 3);
-        q->innerVertices_[1] = NPerm(2, 3, 1, 0);
+        q->innerVertices_[0] = NPerm4(1, 0, 2, 3);
+        q->innerVertices_[1] = NPerm4(2, 3, 1, 0);
         bdry_[vertices[0]] = q;
 
-        q = new BdryQuad(this, vertices * NPerm(3, 0, 2, 1));
+        q = new BdryQuad(this, vertices * NPerm4(3, 0, 2, 1));
         q->innerTet_[0] = innerTet_[3];
         q->innerTet_[1] = innerTet_[2];
-        q->innerVertices_[0] = NPerm(2, 1, 3, 0);
-        q->innerVertices_[1] = NPerm(3, 0, 2, 1);
+        q->innerVertices_[0] = NPerm4(2, 1, 3, 0);
+        q->innerVertices_[1] = NPerm4(3, 0, 2, 1);
         bdry_[vertices[1]] = q;
 
-        q = new BdryQuad(this, vertices * NPerm(0, 1, 3, 2));
+        q = new BdryQuad(this, vertices * NPerm4(0, 1, 3, 2));
         q->innerTet_[0] = innerTet_[0];
         q->innerTet_[1] = innerTet_[3];
-        q->innerVertices_[0] = NPerm(3, 2, 0, 1);
-        q->innerVertices_[1] = NPerm(0, 1, 3, 2);
+        q->innerVertices_[0] = NPerm4(3, 2, 0, 1);
+        q->innerVertices_[1] = NPerm4(0, 1, 3, 2);
         bdry_[vertices[2]] = q;
 
-        q = new BdryQuad(this, vertices * NPerm(1, 2, 0, 3));
+        q = new BdryQuad(this, vertices * NPerm4(1, 2, 0, 3));
         q->innerTet_[0] = innerTet_[1];
         q->innerTet_[1] = innerTet_[0];
-        q->innerVertices_[0] = NPerm(0, 3, 1, 2);
-        q->innerVertices_[1] = NPerm(1, 2, 0, 3);
+        q->innerVertices_[0] = NPerm4(0, 3, 1, 2);
+        q->innerVertices_[1] = NPerm4(1, 2, 0, 3);
         bdry_[vertices[3]] = q;
     }
 
-    TruncHalfTet::TruncHalfTet(NTetrahedron *outerTet, int type) :
-            Block(outerTet, 8, 10) {
-        innerTet_[1]->joinTo(2, innerTet_[0], NPerm());
-        innerTet_[1]->joinTo(1, innerTet_[2], NPerm());
-        innerTet_[1]->joinTo(0, innerTet_[3], NPerm());
-        innerTet_[2]->joinTo(0, innerTet_[4], NPerm());
-        innerTet_[3]->joinTo(1, innerTet_[4], NPerm());
-        innerTet_[3]->joinTo(3, innerTet_[5], NPerm());
-        innerTet_[5]->joinTo(2, innerTet_[6], NPerm());
-        innerTet_[4]->joinTo(2, innerTet_[7], NPerm());
+    TruncHalfTet::TruncHalfTet(NTetrahedron *outerTet, int type,
+            NTriangulation* insertInto):
+            Block(outerTet, 8, 10, insertInto) {
+        innerTet_[1]->joinTo(2, innerTet_[0], NPerm4());
+        innerTet_[1]->joinTo(1, innerTet_[2], NPerm4());
+        innerTet_[1]->joinTo(0, innerTet_[3], NPerm4());
+        innerTet_[2]->joinTo(0, innerTet_[4], NPerm4());
+        innerTet_[3]->joinTo(1, innerTet_[4], NPerm4());
+        innerTet_[3]->joinTo(3, innerTet_[5], NPerm4());
+        innerTet_[5]->joinTo(2, innerTet_[6], NPerm4());
+        innerTet_[4]->joinTo(2, innerTet_[7], NPerm4());
 
-        NPerm vertices(
+        NPerm4 vertices(
             NEdge::edgeVertex[type][0],
             NEdge::edgeVertex[type][1],
             NEdge::edgeVertex[5 - type][0],
@@ -751,128 +761,128 @@ namespace {
         BdryQuad* q;
         BdryHex* h;
 
-        h = new BdryHex(this, vertices * NPerm(1, 3, 2, 0));
+        h = new BdryHex(this, vertices * NPerm4(1, 3, 2, 0));
         h->innerTet_[0] = innerTet_[2];
         h->innerTet_[1] = innerTet_[7];
         h->innerTet_[2] = innerTet_[5];
         h->innerTet_[3] = innerTet_[4];
-        h->innerVertices_[0] = NPerm(2, 0, 1, 3);
-        h->innerVertices_[1] = NPerm(1, 2, 0, 3);
-        h->innerVertices_[2] = NPerm(0, 3, 2, 1);
-        h->innerVertices_[3] = NPerm(0, 2, 1, 3);
+        h->innerVertices_[0] = NPerm4(2, 0, 1, 3);
+        h->innerVertices_[1] = NPerm4(1, 2, 0, 3);
+        h->innerVertices_[2] = NPerm4(0, 3, 2, 1);
+        h->innerVertices_[3] = NPerm4(0, 2, 1, 3);
         bdry_[vertices[0]] = h;
 
-        h = new BdryHex(this, vertices * NPerm(0, 3, 2, 1));
+        h = new BdryHex(this, vertices * NPerm4(0, 3, 2, 1));
         h->innerTet_[0] = innerTet_[0];
         h->innerTet_[1] = innerTet_[7];
         h->innerTet_[2] = innerTet_[6];
         h->innerTet_[3] = innerTet_[3];
-        h->innerVertices_[0] = NPerm(1, 2, 3, 0);
-        h->innerVertices_[1] = NPerm(3, 2, 0, 1);
-        h->innerVertices_[2] = NPerm(0, 2, 1, 3);
-        h->innerVertices_[3] = NPerm(0, 1, 3, 2);
+        h->innerVertices_[0] = NPerm4(1, 2, 3, 0);
+        h->innerVertices_[1] = NPerm4(3, 2, 0, 1);
+        h->innerVertices_[2] = NPerm4(0, 2, 1, 3);
+        h->innerVertices_[3] = NPerm4(0, 1, 3, 2);
         bdry_[vertices[1]] = h;
 
-        q = new BdryQuad(this, vertices * NPerm(3, 1, 0, 2));
+        q = new BdryQuad(this, vertices * NPerm4(3, 1, 0, 2));
         q->innerTet_[0] = innerTet_[2];
         q->innerTet_[1] = innerTet_[0];
-        q->innerVertices_[0] = NPerm(3, 1, 0, 2);
-        q->innerVertices_[1] = NPerm(0, 2, 3, 1);
+        q->innerVertices_[0] = NPerm4(3, 1, 0, 2);
+        q->innerVertices_[1] = NPerm4(0, 2, 3, 1);
         bdry_[vertices[2]] = q;
 
-        q = new BdryQuad(this, vertices * NPerm(2, 0, 1, 3));
+        q = new BdryQuad(this, vertices * NPerm4(2, 0, 1, 3));
         q->innerTet_[0] = innerTet_[6];
         q->innerTet_[1] = innerTet_[5];
-        q->innerVertices_[0] = NPerm(3, 2, 1, 0);
-        q->innerVertices_[1] = NPerm(1, 2, 3, 0);
+        q->innerVertices_[0] = NPerm4(3, 2, 1, 0);
+        q->innerVertices_[1] = NPerm4(1, 2, 3, 0);
         bdry_[vertices[3]] = q;
 
         link_[vertices[2]] = innerTet_[6];
-        linkVertices_[vertices[2]] = vertices * NPerm(3, 2, 0, 1);
+        linkVertices_[vertices[2]] = vertices * NPerm4(3, 2, 0, 1);
 
         link_[vertices[3]] = innerTet_[7];
-        linkVertices_[vertices[3]] = vertices * NPerm(3, 1, 2, 0);
+        linkVertices_[vertices[3]] = vertices * NPerm4(3, 1, 2, 0);
     }
 
-    TruncTet::TruncTet(NTetrahedron *outerTet) :
-            Block(outerTet, 11, 16) {
-        innerTet_[0]->joinTo(2, innerTet_[4], NPerm());
-        innerTet_[1]->joinTo(3, innerTet_[7], NPerm());
-        innerTet_[2]->joinTo(0, innerTet_[6], NPerm());
-        innerTet_[3]->joinTo(1, innerTet_[9], NPerm());
-        innerTet_[5]->joinTo(3, innerTet_[4], NPerm());
-        innerTet_[5]->joinTo(1, innerTet_[6], NPerm());
-        innerTet_[8]->joinTo(0, innerTet_[7], NPerm());
-        innerTet_[8]->joinTo(2, innerTet_[9], NPerm());
-        innerTet_[4]->joinTo(1, innerTet_[10], NPerm());
-        innerTet_[6]->joinTo(3, innerTet_[10], NPerm());
-        innerTet_[7]->joinTo(2, innerTet_[10], NPerm());
-        innerTet_[9]->joinTo(0, innerTet_[10], NPerm());
+    TruncTet::TruncTet(NTetrahedron *outerTet, NTriangulation* insertInto) :
+            Block(outerTet, 11, 16, insertInto) {
+        innerTet_[0]->joinTo(2, innerTet_[4], NPerm4());
+        innerTet_[1]->joinTo(3, innerTet_[7], NPerm4());
+        innerTet_[2]->joinTo(0, innerTet_[6], NPerm4());
+        innerTet_[3]->joinTo(1, innerTet_[9], NPerm4());
+        innerTet_[5]->joinTo(3, innerTet_[4], NPerm4());
+        innerTet_[5]->joinTo(1, innerTet_[6], NPerm4());
+        innerTet_[8]->joinTo(0, innerTet_[7], NPerm4());
+        innerTet_[8]->joinTo(2, innerTet_[9], NPerm4());
+        innerTet_[4]->joinTo(1, innerTet_[10], NPerm4());
+        innerTet_[6]->joinTo(3, innerTet_[10], NPerm4());
+        innerTet_[7]->joinTo(2, innerTet_[10], NPerm4());
+        innerTet_[9]->joinTo(0, innerTet_[10], NPerm4());
 
         BdryHex* h;
 
-        h = new BdryHex(this, NPerm(2, 1, 3, 0));
+        h = new BdryHex(this, NPerm4(2, 1, 3, 0));
         h->innerTet_[0] = innerTet_[2];
         h->innerTet_[1] = innerTet_[8];
         h->innerTet_[2] = innerTet_[3];
         h->innerTet_[3] = innerTet_[9];
-        h->innerVertices_[0] = NPerm(2, 0, 1, 3);
-        h->innerVertices_[1] = NPerm(1, 2, 0, 3);
-        h->innerVertices_[2] = NPerm(0, 1, 2, 3);
-        h->innerVertices_[3] = NPerm(0, 2, 1, 3);
+        h->innerVertices_[0] = NPerm4(2, 0, 1, 3);
+        h->innerVertices_[1] = NPerm4(1, 2, 0, 3);
+        h->innerVertices_[2] = NPerm4(0, 1, 2, 3);
+        h->innerVertices_[3] = NPerm4(0, 2, 1, 3);
         bdry_[0] = h;
 
-        h = new BdryHex(this, NPerm(3, 2, 0, 1));
+        h = new BdryHex(this, NPerm4(3, 2, 0, 1));
         h->innerTet_[0] = innerTet_[3];
         h->innerTet_[1] = innerTet_[5];
         h->innerTet_[2] = innerTet_[0];
         h->innerTet_[3] = innerTet_[4];
-        h->innerVertices_[0] = NPerm(3, 1, 2, 0);
-        h->innerVertices_[1] = NPerm(2, 3, 1, 0);
-        h->innerVertices_[2] = NPerm(1, 2, 3, 0);
-        h->innerVertices_[3] = NPerm(1, 3, 2, 0);
+        h->innerVertices_[0] = NPerm4(3, 1, 2, 0);
+        h->innerVertices_[1] = NPerm4(2, 3, 1, 0);
+        h->innerVertices_[2] = NPerm4(1, 2, 3, 0);
+        h->innerVertices_[3] = NPerm4(1, 3, 2, 0);
         bdry_[1] = h;
 
-        h = new BdryHex(this, NPerm(0, 3, 1, 2));
+        h = new BdryHex(this, NPerm4(0, 3, 1, 2));
         h->innerTet_[0] = innerTet_[0];
         h->innerTet_[1] = innerTet_[8];
         h->innerTet_[2] = innerTet_[1];
         h->innerTet_[3] = innerTet_[7];
-        h->innerVertices_[0] = NPerm(0, 2, 3, 1);
-        h->innerVertices_[1] = NPerm(3, 0, 2, 1);
-        h->innerVertices_[2] = NPerm(2, 3, 0, 1);
-        h->innerVertices_[3] = NPerm(2, 0, 3, 1);
+        h->innerVertices_[0] = NPerm4(0, 2, 3, 1);
+        h->innerVertices_[1] = NPerm4(3, 0, 2, 1);
+        h->innerVertices_[2] = NPerm4(2, 3, 0, 1);
+        h->innerVertices_[3] = NPerm4(2, 0, 3, 1);
         bdry_[2] = h;
 
-        h = new BdryHex(this, NPerm(1, 0, 2, 3));
+        h = new BdryHex(this, NPerm4(1, 0, 2, 3));
         h->innerTet_[0] = innerTet_[1];
         h->innerTet_[1] = innerTet_[5];
         h->innerTet_[2] = innerTet_[2];
         h->innerTet_[3] = innerTet_[6];
-        h->innerVertices_[0] = NPerm(1, 3, 0, 2);
-        h->innerVertices_[1] = NPerm(0, 1, 3, 2);
-        h->innerVertices_[2] = NPerm(3, 0, 1, 2);
-        h->innerVertices_[3] = NPerm(3, 1, 0, 2);
+        h->innerVertices_[0] = NPerm4(1, 3, 0, 2);
+        h->innerVertices_[1] = NPerm4(0, 1, 3, 2);
+        h->innerVertices_[2] = NPerm4(3, 0, 1, 2);
+        h->innerVertices_[3] = NPerm4(3, 1, 0, 2);
         bdry_[3] = h;
 
         link_[0] = innerTet_[0];
-        linkVertices_[0] = NPerm(1, 2, 3, 0);
+        linkVertices_[0] = NPerm4(1, 2, 3, 0);
 
         link_[1] = innerTet_[1];
-        linkVertices_[1] = NPerm(1, 2, 3, 0);
+        linkVertices_[1] = NPerm4(1, 2, 3, 0);
 
         link_[2] = innerTet_[2];
-        linkVertices_[2] = NPerm(1, 2, 3, 0);
+        linkVertices_[2] = NPerm4(1, 2, 3, 0);
 
         link_[3] = innerTet_[3];
-        linkVertices_[3] = NPerm(1, 2, 3, 0);
+        linkVertices_[3] = NPerm4(1, 2, 3, 0);
     }
 
     inline Bdry::~Bdry() {
         // Empty virtual destructor.
     }
 
-    inline Bdry::Bdry(Block* block, NPerm outerVertices) :
+    inline Bdry::Bdry(Block* block, NPerm4 outerVertices) :
             block_(block), outerVertices_(outerVertices) {
     }
 
@@ -881,7 +891,7 @@ namespace {
         BdryQuad* dest = static_cast<BdryQuad*>(other);
 
         // Get the map from *this* 012 to *dest* tetrahedron vertices.
-        NPerm destMap = block_->outerTet()->
+        NPerm4 destMap = block_->outerTet()->
             adjacentGluing(outerVertices_[3]) * outerVertices_;
 
         if (destMap != dest->outerVertices_) {
@@ -901,7 +911,7 @@ namespace {
                 dest->innerVertices_[i] * innerVertices_[i].inverse());
     }
 
-    inline BdryQuad::BdryQuad(Block* block, NPerm outerVertices) :
+    inline BdryQuad::BdryQuad(Block* block, NPerm4 outerVertices) :
             Bdry(block, outerVertices) {
     }
 
@@ -909,15 +919,15 @@ namespace {
         NTetrahedron* layering = block_->layeringTetrahedron();
 
         layering->joinTo(0, innerTet_[1],
-            innerVertices_[1] * NPerm(3, 2, 1, 0));
+            innerVertices_[1] * NPerm4(3, 2, 1, 0));
         layering->joinTo(2, innerTet_[0],
-            innerVertices_[0] * NPerm(1, 0, 3, 2));
+            innerVertices_[0] * NPerm4(1, 0, 3, 2));
 
         innerTet_[0] = innerTet_[1] = layering;
-        innerVertices_[0] = NPerm();
-        innerVertices_[1] = NPerm(2, 3, 0, 1);
+        innerVertices_[0] = NPerm4();
+        innerVertices_[1] = NPerm4(2, 3, 0, 1);
 
-        outerVertices_ = outerVertices_ * NPerm(1, 2);
+        outerVertices_ = outerVertices_ * NPerm4(1, 2);
     }
 
     void BdryHex::joinTo(Bdry* other) {
@@ -925,7 +935,7 @@ namespace {
         BdryHex* dest = static_cast<BdryHex*>(other);
 
         // Get the map from *this* 012 to *dest* tetrahedron vertices.
-        NPerm destMap = block_->outerTet()->
+        NPerm4 destMap = block_->outerTet()->
             adjacentGluing(outerVertices_[3]) * outerVertices_;
 
         if (destMap.sign() != dest->outerVertices_.sign())
@@ -940,7 +950,7 @@ namespace {
                 dest->innerVertices_[i] * innerVertices_[i].inverse());
     }
 
-    inline BdryHex::BdryHex(Block* block, NPerm outerVertices) :
+    inline BdryHex::BdryHex(Block* block, NPerm4 outerVertices) :
             Bdry(block, outerVertices) {
     }
 
@@ -950,28 +960,28 @@ namespace {
         NTetrahedron* layering2 = block_->layeringTetrahedron();
         NTetrahedron* layering3 = block_->layeringTetrahedron();
 
-        layering0->joinTo(1, innerTet_[3], innerVertices_[3] * NPerm(1, 3));
-        layering0->joinTo(2, innerTet_[2], innerVertices_[2] * NPerm(2, 3));
-        layering1->joinTo(3, layering0, NPerm());
+        layering0->joinTo(1, innerTet_[3], innerVertices_[3] * NPerm4(1, 3));
+        layering0->joinTo(2, innerTet_[2], innerVertices_[2] * NPerm4(2, 3));
+        layering1->joinTo(3, layering0, NPerm4());
         layering1->joinTo(1, innerTet_[1],
-            innerVertices_[1] * NPerm(2, 3, 0, 1));
-        layering2->joinTo(0, layering0, NPerm());
+            innerVertices_[1] * NPerm4(2, 3, 0, 1));
+        layering2->joinTo(0, layering0, NPerm4());
         layering2->joinTo(1, innerTet_[0],
-            innerVertices_[0] * NPerm(1, 3, 2, 0));
-        layering3->joinTo(0, layering1, NPerm());
-        layering3->joinTo(3, layering2, NPerm());
+            innerVertices_[0] * NPerm4(1, 3, 2, 0));
+        layering3->joinTo(0, layering1, NPerm4());
+        layering3->joinTo(3, layering2, NPerm4());
 
         innerTet_[0] = layering2;
         innerTet_[1] = layering1;
         innerTet_[2] = layering3;
         innerTet_[3] = layering3;
 
-        innerVertices_[0] = NPerm(0, 3, 1, 2);
-        innerVertices_[1] = NPerm(1, 0, 3, 2);
-        innerVertices_[2] = NPerm(3, 2, 0, 1);
-        innerVertices_[3] = NPerm(3, 0, 1, 2);
+        innerVertices_[0] = NPerm4(0, 3, 1, 2);
+        innerVertices_[1] = NPerm4(1, 0, 3, 2);
+        innerVertices_[2] = NPerm4(3, 2, 0, 1);
+        innerVertices_[3] = NPerm4(3, 0, 1, 2);
 
-        outerVertices_ = outerVertices_ * NPerm(1, 2);
+        outerVertices_ = outerVertices_ * NPerm4(1, 2);
     }
 
     void BdryHex::rotate() {
@@ -980,16 +990,17 @@ namespace {
         innerTet_[1] = innerTet_[2];
         innerTet_[2] = t;
 
-        NPerm p = innerVertices_[0];
+        NPerm4 p = innerVertices_[0];
         innerVertices_[0] = innerVertices_[1];
         innerVertices_[1] = innerVertices_[2];
         innerVertices_[2] = p;
-        innerVertices_[3] = innerVertices_[3] * NPerm(1, 2, 0, 3);
+        innerVertices_[3] = innerVertices_[3] * NPerm4(1, 2, 0, 3);
 
-        outerVertices_ = outerVertices_ * NPerm(1, 2, 0, 3);
+        outerVertices_ = outerVertices_ * NPerm4(1, 2, 0, 3);
     }
 
-    TetBlockSet::TetBlockSet(const NNormalSurface* s, unsigned long tetIndex) {
+    TetBlockSet::TetBlockSet(const NNormalSurface* s, unsigned long tetIndex,
+            NTriangulation* insertInto) {
         unsigned long i, j;
         for (i = 0; i < 4; ++i)
             triCount_[i] = s->getTriangleCoord(tetIndex, i).longValue();
@@ -1021,30 +1032,30 @@ namespace {
             else {
                 triPrism_[i] = new Block*[triCount_[i]];
                 for (j = 0; j < triCount_[i]; ++j)
-                    triPrism_[i][j] = new TriPrism(tet, i);
+                    triPrism_[i][j] = new TriPrism(tet, i, insertInto);
             }
         }
 
         if (quadCount_ == 0) {
             quadPrism_ = 0;
             truncHalfTet_[0] = truncHalfTet_[1] = 0;
-            truncTet_ = new TruncTet(tet);
+            truncTet_ = new TruncTet(tet, insertInto);
         } else {
             if (quadCount_ > 1) {
                 quadPrism_ = new Block*[quadCount_ - 1];
                 for (j = 0; j < quadCount_ - 1; ++j)
-                    quadPrism_[j] = new QuadPrism(tet, quadType_);
+                    quadPrism_[j] = new QuadPrism(tet, quadType_, insertInto);
             } else
                 quadPrism_ = 0;
 
-            truncHalfTet_[0] = new TruncHalfTet(tet, 5 - quadType_);
-            truncHalfTet_[1] = new TruncHalfTet(tet, quadType_);
+            truncHalfTet_[0] = new TruncHalfTet(tet, 5 - quadType_, insertInto);
+            truncHalfTet_[1] = new TruncHalfTet(tet, quadType_, insertInto);
 
             truncTet_ = 0;
         }
 
         for (i = 0; i < 4; ++i) {
-            vertexNbd_[i] = new NTetrahedron();
+            vertexNbd_[i] = insertInto->newTetrahedron();
 
             if (triCount_[i] > 0)
                 triPrism_[i][0]->attachVertexNbd(vertexNbd_[i], i);
@@ -1128,28 +1139,6 @@ namespace {
     inline NTetrahedron* TetBlockSet::vertexNbd(int vertex) {
         return vertexNbd_[vertex];
     }
-
-    void TetBlockSet::insertInto(NTriangulation* tri) {
-        unsigned long i, j;
-        for (i = 0; i < 4; ++i)
-            if (triPrism_[i])
-                for (j = 0; j < triCount_[i]; ++j)
-                    triPrism_[i][j]->insertInto(tri);
-
-        if (quadCount_ == 0)
-            truncTet_->insertInto(tri);
-        else {
-            if (quadPrism_)
-                for (j = 0; j < quadCount_ - 1; ++j)
-                    quadPrism_[j]->insertInto(tri);
-
-            truncHalfTet_[0]->insertInto(tri);
-            truncHalfTet_[1]->insertInto(tri);
-        }
-
-        for (i = 0; i < 4; ++i)
-            tri->addTetrahedron(vertexNbd_[i]);
-    }
 }
 
 // ------------------------------------------------------------------------
@@ -1158,6 +1147,7 @@ namespace {
 
 NTriangulation* NNormalSurface::cutAlong() const {
     NTriangulation* ans = new NTriangulation();
+    NPacket::ChangeEventSpan span(ans);
 
     unsigned long nTet = getTriangulation()->getNumberOfTetrahedra();
     if (nTet == 0)
@@ -1166,14 +1156,14 @@ NTriangulation* NNormalSurface::cutAlong() const {
     unsigned long i;
     TetBlockSet** sets = new TetBlockSet*[nTet];
     for (i = 0; i < nTet; ++i)
-        sets[i] = new TetBlockSet(this, i);
+        sets[i] = new TetBlockSet(this, i, ans);
 
     NTriangulation::FaceIterator fit;
     NFace* f;
     unsigned long tet0, tet1;
     int face0, face1;
     int fromVertex0, fromVertex1;
-    NPerm gluing;
+    NPerm4 gluing;
     unsigned long quadBlocks;
     for (fit = getTriangulation()->getFaces().begin();
             fit != getTriangulation()->getFaces().end(); ++fit) {
@@ -1204,10 +1194,9 @@ NTriangulation* NNormalSurface::cutAlong() const {
         sets[tet0]->hexBlock(face0)->joinTo(face0, sets[tet1]->hexBlock(face1));
     }
 
-    for (i = 0; i < nTet; ++i) {
-        sets[i]->insertInto(ans);
+    // All done!  Clean up.
+    for (i = 0; i < nTet; ++i)
         delete sets[i];
-    }
     delete[] sets;
 
     return ans;
@@ -1221,7 +1210,7 @@ NTriangulation* NNormalSurface::crush() const {
     NTriangulation* ans = new NTriangulation(*triangulation);
     unsigned long nTet = ans->getNumberOfTetrahedra();
     if (nTet == 0)
-        return new NTriangulation();
+        return ans;
 
     // Work out which tetrahedra contain which quad types.
     int* quads = new int[nTet];
@@ -1241,8 +1230,8 @@ NTriangulation* NNormalSurface::crush() const {
     NTetrahedron* tet;
     NTetrahedron* adj;
     int adjQuads;
-    NPerm adjPerm;
-    NPerm swap;
+    NPerm4 adjPerm;
+    NPerm4 swap;
     int face, adjFace;
     for (whichTet = 0; whichTet < static_cast<long>(nTet); whichTet++)
         if (quads[whichTet] == -1) {
@@ -1262,7 +1251,7 @@ NTriangulation* NNormalSurface::crush() const {
                 adjPerm = tet->adjacentGluing(face);
                 adjFace = adjPerm[face];
                 while (adj && (adjQuads >= 0)) {
-                    swap = NPerm(adjFace,
+                    swap = NPerm4(adjFace,
                         vertexSplitPartner[adjQuads][adjFace]);
 
                     adjFace = swap[adjFace];
@@ -1296,5 +1285,103 @@ NTriangulation* NNormalSurface::crush() const {
     return ans;
 }
 
+NTriBool NNormalSurface::isCompressingDisc(bool knownConnected) const {
+    // Is it even a disc?
+    if (! hasRealBoundary())
+        return NTriBool::False;
+    if (getEulerCharacteristic() != 1)
+        return NTriBool::False;
+
+    if (! knownConnected) {
+        NTriBool ans = isConnected();
+        if (! ans.isTrue())
+            return ans;
+    }
+
+    // Yep, it's a disc (and hence two-sided).
+
+    // Count the number of boundary spheres that our triangulation has
+    // to begin with.
+    unsigned long origSphereCount = 0;
+    NTriangulation::BoundaryComponentIterator bit;
+    for (bit = getTriangulation()->getBoundaryComponents().begin();
+            bit != getTriangulation()->getBoundaryComponents().end(); ++bit)
+        if ((*bit)->getEulerCharacteristic() == 2)
+            ++origSphereCount;
+
+    // Now cut along the disc, and see if we get an extra sphere as a
+    // result.  If not, the disc boundary is non-trivial and so the disc
+    // is compressing.
+    std::auto_ptr<NTriangulation> cut(cutAlong());
+    if (! cut.get())
+        return NTriBool::Unknown;
+
+    if (cut->getNumberOfBoundaryComponents() ==
+            getTriangulation()->getNumberOfBoundaryComponents()) {
+        // The boundary of the disc is not a separating curve in the
+        // boundary of the triangulation.  Therefore we might end up
+        // converting a torus boundary into a sphere boundary, but the
+        // disc is compressing regardless.
+        return NTriBool::True;
+    }
+
+    unsigned long newSphereCount = 0;
+    for (bit = cut->getBoundaryComponents().begin();
+            bit != cut->getBoundaryComponents().end(); ++bit)
+        if ((*bit)->getEulerCharacteristic() == 2)
+            ++newSphereCount;
+
+    if (newSphereCount == origSphereCount)
+        return NTriBool::True;
+    else
+        return NTriBool::False;
+}
+
+NTriBool NNormalSurface::isIncompressible() const {
+    // Make the surface two-sided if necessary.
+    {
+        NTriBool twoSided = isTwoSided();
+        if (twoSided.isUnknown())
+            return NTriBool::Unknown;
+        if (twoSided.isFalse()) {
+            // Convert this into a two-sided surface and rerun the
+            // algorithm.
+            NNormalSurfaceVector* v =
+                static_cast<NNormalSurfaceVector*>(rawVector()->clone());
+            (*v) *= 2;
+            NNormalSurface doubleSurface(getTriangulation(), v);
+            return doubleSurface.isIncompressible();
+        }
+    }
+
+    // Rule out spheres.
+    // From the preconditions, we can assume this surface to be
+    // closed, compact and connected.
+    if (getEulerCharacteristic() == 2)
+        return NTriBool::False;
+
+    // Cut along this surface and look for compressing discs.
+    std::auto_ptr<NTriangulation> cut(cutAlong());
+    if (! cut.get())
+        return NTriBool::Unknown;
+
+    cut->splitIntoComponents(0, false);
+    NTriBool found;
+    for (NPacket* comp = cut->getFirstTreeChild(); comp;
+            comp = comp->getNextTreeSibling()) {
+        // Since the original manifold is irreducible and this surface
+        // is connected, we know that the sliced-open manifold is irreducible
+        // also.
+        found = static_cast<NTriangulation*>(comp)->hasCompressingDisc();
+        if (found.isUnknown())
+            return NTriBool::Unknown;
+        if (found.isTrue())
+            return NTriBool::False;
+    }
+
+    // No compressing discs were found.
+    return NTriBool::True;
+}
+
 } // namespace regina
 
