@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,8 +26,9 @@
 
 /* end stub */
 
-/*! \file nnormalsurface.h
- *  \brief Deals with an individual normal surface.
+/*! \file surfaces/nnormalsurface.h
+ *  \brief Deals with an individual normal surface in a 3-manifold
+ *  triangulation.
  */
 
 #ifndef __NNORMALSURFACE_H
@@ -36,11 +37,12 @@
 #endif
 
 #include <utility>
+#include "regina-core.h"
 #include "shareableobject.h"
 #include "file/nfilepropertyreader.h"
+#include "maths/nperm4.h"
 #include "maths/nray.h"
 #include "surfaces/ndisctype.h"
-#include "triangulation/nperm.h"
 #include "utilities/nbooleans.h"
 #include "utilities/nproperty.h"
 
@@ -48,14 +50,14 @@ namespace regina {
 
 /**
  * \addtogroup surfaces Normal Surfaces
- * Normal surfaces in triangulations.
+ * Normal surfaces in 3-manifold triangulations.
  * @{
  */
 
 /**
- * Lists which vertex splits split which pairs of vertices.
+ * Lists which vertex splits separate which pairs of vertices.
  * There are three vertex splits, numbered 0,1,2.  Each vertex
- * split split the four tetrahedron vertices 0,1,2,3 into two pairs.
+ * split separates the four tetrahedron vertices 0,1,2,3 into two pairs.
  * <tt>vertexSplit[i][j]</tt> is the number of the vertex split that
  * keeps vertices <tt>i</tt> and <tt>j</tt> together.
  *
@@ -79,6 +81,14 @@ extern const int vertexSplitMeeting[4][4][2];
  * Vertex split number \c i splits the vertex pairs
  * <tt>vertexSplitDefn[i][0,1]</tt> and
  * <tt>vertexSplitDefn[i][2,3]</tt>.
+ *
+ * It is guaranteed that:
+ *
+ * - <tt>vertexSplitDefn[i][0] < vertexSplitDefn[i][1]</tt>;
+ *
+ * - <tt>vertexSplitDefn[i][2] < vertexSplitDefn[i][3]</tt>;
+ *
+ * - <tt>vertexSplitDefn[i][0] < vertexSplitDefn[i][2]</tt>.
  */
 extern const int vertexSplitDefn[3][4];
 
@@ -118,9 +128,9 @@ extern const char vertexSplitString[3][6];
  * the corresponding array.  This is necessary because of a bug in gcc 2.95.
  */
 #ifdef __DOXYGEN
-extern const NPerm triDiscArcs[4][3];
+extern const NPerm4 triDiscArcs[4][3];
 #else
-extern const NPerm __triDiscArcs[12];
+extern const NPerm4 __triDiscArcs[12];
 #define triDiscArcs(i, j) __triDiscArcs[(3 * (i)) + (j)]
 #endif
 
@@ -142,9 +152,9 @@ extern const NPerm __triDiscArcs[12];
  * the corresponding array.  This is necessary because of a bug in gcc 2.95.
  */
 #ifdef __DOXYGEN
-extern const NPerm quadDiscArcs[3][4];
+extern const NPerm4 quadDiscArcs[3][4];
 #else
-extern const NPerm __quadDiscArcs[12];
+extern const NPerm4 __quadDiscArcs[12];
 #define quadDiscArcs(i, j) __quadDiscArcs[(4 * (i)) + (j)]
 #endif
 
@@ -166,9 +176,9 @@ extern const NPerm __quadDiscArcs[12];
  * the corresponding array.  This is necessary because of a bug in gcc 2.95.
  */
 #ifdef __DOXYGEN
-extern const NPerm octDiscArcs[3][8];
+extern const NPerm4 octDiscArcs[3][8];
 #else
-extern const NPerm __octDiscArcs[24];
+extern const NPerm4 __octDiscArcs[24];
 #define octDiscArcs(i, j) __octDiscArcs[(8 * (i)) + (j)]
 #endif
 
@@ -219,11 +229,16 @@ class NXMLNormalSurfaceReader;
  *   <tt>class(const NVector<NLargeInteger>& cloneMe)</tt> must be
  *   declared and implemented; these will usually just call the
  *   corresponding superclass constructors.</li>
- *   <li>Virtual functions <tt>NVector<NLargeInteger>* clone() const</tt> and
- *   <tt>bool allowsAlmostNormal() const</tt> must be declared but not
+ *   <li>Virtual functions <tt>NNormalSurfaceVector* clone() const</tt>,
+ *   <tt>bool allowsAlmostNormal() const</tt> and
+ *   <tt>bool allowsSpun() const </tt> must be declared but not
  *   implemented.  The registry utilities will take care of their
  *   implementations.</li>
- *   <li>All abstract functions must be implemented.</li>
+ *   <li>All abstract functions must be implemented.
+ *   Note that coordinate functions such as getTriangleCoord() must return the
+ *   \e total number of discs of the requested type; if your new coordinate
+ *   system adorns discs with extra information (such as orientation) then
+ *   your implementation must compute the appropriate sum.</li>
  *   <li>Static public functions <tt>void
  *   makeZeroVector(const NTriangulation*)</tt>,
  *   <tt>NMatrixInt* makeMatchingEquations(NTriangulation*)</tt> and
@@ -237,7 +252,7 @@ class NXMLNormalSurfaceReader;
  *
  * \ifacespython Not present.
  */
-class NNormalSurfaceVector : public NRay {
+class REGINA_API NNormalSurfaceVector : public NRay {
     public:
         /**
          * Creates a new vector all of whose entries are initialised to
@@ -253,6 +268,19 @@ class NNormalSurfaceVector : public NRay {
          */
         NNormalSurfaceVector(const NVector<NLargeInteger>& cloneMe);
 
+        /**
+         * A virtual destructor.  This is required because here we introduce
+         * virtual functions into the NRay hierarchy.
+         */
+        virtual ~NNormalSurfaceVector();
+
+        /**
+         * Creates a newly allocated clone of this vector.
+         * The clone will be of the same subclass of NNormalSurfaceVector
+         * as this vector.
+         */
+        virtual NNormalSurfaceVector* clone() const = 0;
+
         /**
          * Determines if the specific underlying coordinate system
          * allows for almost normal surfaces, that is, allows for
@@ -265,6 +293,18 @@ class NNormalSurfaceVector : public NRay {
          */
         virtual bool allowsAlmostNormal() const = 0;
 
+        /**
+         * Determines if the specific underlying coordinate system
+         * allows for spun normal surfaces; that is, surfaces with
+         * infinitely many triangles.
+         *
+         * Note that this has nothing to do with whether or not this
+         * specific surface \e contains infinitely many triangles.
+         *
+         * @return \c true if and only if spun normal surfaces are allowed.
+         */
+        virtual bool allowsSpun() const = 0;
+
         /**
          * Determines if this normal surface has more than one
          * octagonal disc.  It may be assumed that at most one
@@ -339,15 +379,15 @@ class NNormalSurfaceVector : public NRay {
         virtual const NVertex* isVertexLink(NTriangulation* triang) const;
         /**
          * Determines if a rational multiple of the normal surface represented
-         * is the link of a single thin edge.
+         * is the thin link of a single edge.
          *
-         * If there are two different thin edges <i>e1</i> and <i>e2</i> for
-         * which the surface could be expressed as either the link of
-         * <i>e1</i> or the link of <i>e2</i>, the pair
+         * If there are two different edges <i>e1</i> and <i>e2</i> for
+         * which the surface could be expressed as the thin link of
+         * either <i>e1</i> or <i>e2</i>, the pair
          * (<i>e1</i>,<i>e2</i>) will be returned.
-         * If the surface is the link of only one thin edge <i>e</i>,
+         * If the surface is the thin link of only one edge <i>e</i>,
          * the pair (<i>e</i>,0) will be returned.
-         * If the surface is not the link of any thin edges, the pair
+         * If the surface is not the thin link of any edges, the pair
          * (0,0) will be returned.
          *
          * The default implementation for this routine involves counting the
@@ -356,7 +396,7 @@ class NNormalSurfaceVector : public NRay {
          * they can provide a faster implementation.
          *
          * @param triang the triangulation in which this normal surface lives.
-         * @return a pair containing the thin edge(s) linked by this surface,
+         * @return a pair containing the edge(s) linked by this surface,
          * as described above.
          */
         virtual std::pair<const NEdge*, const NEdge*> isThinEdgeLink(
@@ -551,7 +591,8 @@ class NNormalSurfaceVector : public NRay {
  * \todo \featurelong Determine which faces in the solution space a
  * normal surface belongs to.
  */
-class NNormalSurface : public ShareableObject, public NFilePropertyReader {
+class REGINA_API NNormalSurface :
+        public ShareableObject, public NFilePropertyReader {
     protected:
         NNormalSurfaceVector* vector;
             /**< Contains the coordinates of the normal surface in whichever
@@ -610,6 +651,9 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
         /**
          * Creates a newly allocated clone of this normal surface.
          *
+         * The name of the normal surface will \e not be copied to the
+         * clone; instead the clone will have an empty name.
+         *
          * @return a clone of this normal surface.
          */
         NNormalSurface* clone() const;
@@ -629,6 +673,11 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
          * tetrahedron and a vertex of that tetrahedron that the
          * triangle surrounds.
          *
+         * If you are using a coordinate system that adorns discs with
+         * additional information (such as orientation), this routine
+         * returns the \e total number of triangles in the given
+         * tetrahedron of the given type.
+         *
          * @param tetIndex the index in the triangulation of the
          * tetrahedron in which the requested triangles reside;
          * this should be between 0 and
@@ -649,6 +698,11 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
          * vertices.  See \a vertexSplit for more details on vertex
          * splittings.
          *
+         * If you are using a coordinate system that adorns discs with
+         * additional information (such as orientation), this routine
+         * returns the \e total number of quadrilaterals in the given
+         * tetrahedron of the given type.
+         *
          * @param tetIndex the index in the triangulation of the
          * tetrahedron in which the requested quadrilaterals reside;
          * this should be between 0 and
@@ -669,6 +723,11 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
          * vertices.  See \a vertexSplit for more details on vertex
          * splittings.
          *
+         * If you are using a coordinate system that adorns discs with
+         * additional information (such as orientation), this routine
+         * returns the \e total number of octagons in the given
+         * tetrahedron of the given type.
+         *
          * @param tetIndex the index in the triangulation of the
          * tetrahedron in which the requested octagons reside;
          * this should be between 0 and
@@ -967,15 +1026,15 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
         virtual const NVertex* isVertexLink() const;
         /**
          * Determines whether or not a rational multiple of this surface
-         * is the link of a single thin edge.
+         * is the thin link of a single edge.
          *
-         * If there are two different thin edges <i>e1</i> and <i>e2</i> for
-         * which this surface could be expressed as either the link of
-         * <i>e1</i> or the link of <i>e2</i>, the pair
+         * If there are two different edges <i>e1</i> and <i>e2</i> for
+         * which this surface could be expressed as the thin link of
+         * either <i>e1</i> or <i>e2</i>, the pair
          * (<i>e1</i>,<i>e2</i>) will be returned.
-         * If this surface is the link of only one thin edge <i>e</i>,
+         * If this surface is the thin link of only one edge <i>e</i>,
          * the pair (<i>e</i>,0) will be returned.
-         * If this surface is not the link of any thin edges, the pair
+         * If this surface is not the thin link of any edges, the pair
          * (0,0) will be returned.
          *
          * Note that the results of this routine are not cached.
@@ -986,7 +1045,7 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
          *
          * \ifacespython This routine returns a tuple of size 2.
          *
-         * @return a pair containing the thin edge(s) linked by this surface,
+         * @return a pair containing the edge(s) linked by this surface,
          * as described above.
          */
         virtual std::pair<const NEdge*, const NEdge*> isThinEdgeLink() const;
@@ -1023,6 +1082,83 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
          */
         NLargeInteger isCentral() const;
 
+        /**
+         * Determines whether this surface represents a compressing disc
+         * in the underlying 3-manifold.
+         *
+         * Let this surface be \a D and let the underlying 3-manifold
+         * be \a M with boundary \a B.  To be a compressing disc, \a D must
+         * be a properly embedded disc in \a M, and the boundary of \a D
+         * must not bound a disc in \a B.
+         *
+         * This routine returns an NTriBool since it is possible that
+         * the result cannot be determined (for instance, if there
+         * are too many normal discs).
+         *
+         * The implementation of this routine is somewhat inefficient at
+         * present, since it cuts along the disc, retriangulates and then
+         * examines the resulting boundary components.
+         *
+         * \pre This normal surface is compact and embedded.
+         * \pre This normal surface contains no octagonal discs.
+         *
+         * \todo \opt Reimplement this to avoid cutting along surfaces.
+         * \todo \prob Check for absurdly large numbers of discs and bail
+         * accordingly.
+         *
+         * @param knownConnected \c true if this normal surface is
+         * already known to be connected (for instance, if it came from
+         * an enumeration of vertex normal surfaces), or \c false if
+         * we should not assume any such information about this surface.
+         * @return true if this surface is a compressing disc, false if
+         * this surface is not a compressing disc, or unknown if the
+         * result cannot be determined.
+         */
+        NTriBool isCompressingDisc(bool knownConnected = false) const;
+
+        /**
+         * Determines whether this is an incompressible surface within
+         * the surrounding 3-manifold.  At present, this routine is only
+         * implemented for surfaces embedded within \e closed 3-manifold
+         * triangulations.
+         *
+         * Let \a D be some disc embedded in the underlying 3-manifold,
+         * and let \a B be the boundary of \a D.  We call \a D a
+         * <i>compressing disc</i> for this surface if (i) the intersection
+         * of \a D with this surface is the boundary \a B, and
+         * (ii) although \a B bounds a disc within the 3-manifold, it
+         * does not bound a disc within this surface.
+         *
+         * We declare this surface to be \e incompressible if there are
+         * no such compressing discs.  For our purposes, spheres are never
+         * considered incompressible (so if this surface is a sphere then
+         * this routine will always return \c false).
+         *
+         * This test is designed exclusively for two-sided surfaces.
+         * If this surface is one-sided, the incompressibility test will
+         * be run on its two-sided double cover.
+         *
+         * This routine returns an NTriBool since it is possible that
+         * the result cannot be determined (for instance, if there
+         * are too many normal discs).
+         *
+         * \warning This routine can become \e extremely slow, to the
+         * point of infeasibility.  This is because the underlying
+         * algorithm cuts along this surface, retriangulates (possibly
+         * using a very large number of tetrahedra), and then runs a new
+         * (exponentially slow) normal surface enumeration.
+         *
+         * \pre The underlying 3-manifold triangulation is valid and closed.
+         * \pre The underlying 3-manifold is irreducible.
+         * \pre This normal surface is compact, embedded and connected.
+         * \pre This normal surface contains no octagonal discs.
+         *
+         * @return true if this surface is incompressible, false if
+         * this surface is not incompressible (or if it is a sphere),
+         * or unknown if the result cannot be determined.
+         */
+        NTriBool isIncompressible() const;
+
         /**
          * Cuts the associated triangulation along this surface and
          * returns a newly created resulting triangulation.
@@ -1148,7 +1284,7 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
         /**
          * Determines whether this and the given surface can be placed
          * within the surrounding triangulation so that they do not intersect
-         * anywhere at all.
+         * anywhere at all, without changing either normal isotopy class.
          *
          * This is a global constraint, and therefore gives a stronger test
          * than locallyCompatible().  However, this global constraint is
@@ -1190,8 +1326,8 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
          * coordinate system its raw vector uses.  Unless you already know
          * the coordinate system in advance (i.e., you created the surface
          * yourself), it is best to keep to the coordinate-system-agnostic
-         * access functions such as NNormalSurfaceVector::getTriCoord() and
-         * NNormalSurfaceVector::getQuadCoord().
+         * access functions such as NNormalSurfaceVector::getTriangleCoord()
+         * and NNormalSurfaceVector::getQuadCoord().
          *
          * \ifacespython Not present.
          *
@@ -1242,7 +1378,7 @@ class NNormalSurface : public ShareableObject, public NFilePropertyReader {
          * and J. Hyam Rubinstein, J. Differential Geom. 65 (2003),
          * no. 1, 61--168.  For a proof in quadrilateral-octagon coordinates,
          * see "Quadrilateral-octagon coordinates for almost normal surfaces",
-         * Benjamin A. Burton, preprint, arXiv:0904.3041.
+         * Benjamin A. Burton, Experiment. Math. 19 (2010), 285-315.
          *
          * Note that the surface that this routine returns (if any) depends
          * upon the triangulation, and so this surface must be destroyed
@@ -1308,6 +1444,8 @@ inline NNormalSurfaceVector::NNormalSurfaceVector(unsigned length) :
 inline NNormalSurfaceVector::NNormalSurfaceVector(
         const NVector<NLargeInteger>& cloneMe) : NRay(cloneMe) {
 }
+inline NNormalSurfaceVector::~NNormalSurfaceVector() {
+}
 
 // Inline functions for NNormalSurface
 
