@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -56,7 +56,7 @@ const int NNormalSurfaceList::AN_STANDARD = 102;
 const int NNormalSurfaceList::EDGE_WEIGHT = 200;
 const int NNormalSurfaceList::FACE_ARCS = 201;
 
-#define REGISTER_FLAVOUR(id_name, class, n, a, t) \
+#define REGISTER_FLAVOUR(id_name, class, n, a, s) \
     case NNormalSurfaceList::id_name: \
         return class::makeZeroVector(triangulation);
 
@@ -70,7 +70,7 @@ NNormalSurfaceVector* makeZeroVector(const NTriangulation* triangulation,
 }
 
 #undef REGISTER_FLAVOUR
-#define REGISTER_FLAVOUR(id_name, class, n, a, t) \
+#define REGISTER_FLAVOUR(id_name, class, n, a, s) \
     case NNormalSurfaceList::id_name: \
         return class::makeMatchingEquations(triangulation);
 
@@ -84,11 +84,9 @@ NMatrixInt* makeMatchingEquations(NTriangulation* triangulation,
 }
 
 #undef REGISTER_FLAVOUR
-#define REGISTER_FLAVOUR(id_name, cls, n, a, test) \
+#define REGISTER_FLAVOUR(id_name, cls, n, a, s) \
     case NNormalSurfaceList::id_name: \
-        if (! (test)) \
-            noSurfaces = true; \
-        else if (list->embedded) \
+        if (list->embedded) \
             constraints = cls::makeEmbeddedConstraints(triang); \
         break;
 
@@ -112,35 +110,20 @@ void* NNormalSurfaceList::Enumerator::run(void*) {
     } else {
         // The catch-all double description method.
 
-        // Perform any pre-enumeration tests and fetch any necessary
-        // validity constraints.
+        // Fetch any necessary validity constraints.
         NEnumConstraintList* constraints = 0;
-        bool noSurfaces = false;
         switch(list->flavour) {
             // Import cases from the flavour registry.
             #include "surfaces/flavourregistry.h"
         }
 
-        if (noSurfaces) {
-            // There are no normal surfaces at all.
-            triang->insertChildLast(list);
-
-            if (progress) {
-                progress->incCompleted();
-                progress->setFinished();
-            }
-            return 0;
-        }
-
         // Form the matching equations and starting cone.
         NMatrixInt* eqns = makeMatchingEquations(triang, list->flavour);
-        NNormalSurfaceVector* base = makeZeroVector(triang, list->flavour);
 
         // Find the normal surfaces.
-        NDoubleDescription::enumerateExtremalRays(SurfaceInserter(*list,
-            triang), *base, *eqns, constraints, progress);
+        enumerateExtremalRays(list->flavour,
+            SurfaceInserter(*list, triang), *eqns, constraints, progress);
 
-        delete base;
         delete eqns;
         delete constraints;
     }
@@ -156,6 +139,22 @@ void* NNormalSurfaceList::Enumerator::run(void*) {
     return 0;
 }
 
+#undef REGISTER_FLAVOUR
+#define REGISTER_FLAVOUR(id_name, class, n, an, s) \
+    case id_name: NDoubleDescription::enumerateExtremalRays<class>( \
+        results, subspace, constraints, progress); \
+        return true;
+
+bool NNormalSurfaceList::enumerateExtremalRays(int flavour,
+        const SurfaceInserter& results, const NMatrixInt& subspace,
+        const NEnumConstraintList* constraints, NProgressNumber* progress) {
+    switch(flavour) {
+        // Import cases from the flavour registry:
+        #include "surfaces/flavourregistry.h"
+    }
+    return false;
+}
+
 NNormalSurfaceList* NNormalSurfaceList::enumerate(NTriangulation* owner,
         int newFlavour, bool embeddedOnly, NProgressManager* manager) {
     NNormalSurfaceList* ans = new NNormalSurfaceList(newFlavour, embeddedOnly);
@@ -184,13 +183,10 @@ NNormalSurfaceList* NNormalSurfaceList::enumerateStandardDirect(
         NNormalSurfaceVectorStandard::makeEmbeddedConstraints(owner);
     NMatrixInt* eqns = makeMatchingEquations(owner,
         NNormalSurfaceList::STANDARD);
-    NNormalSurfaceVector* base = makeZeroVector(owner,
-        NNormalSurfaceList::STANDARD);
 
-    NDoubleDescription::enumerateExtremalRays(SurfaceInserter(*list, owner),
-        *base, *eqns, constraints);
+    NDoubleDescription::enumerateExtremalRays<NNormalSurfaceVectorStandard>(
+        SurfaceInserter(*list, owner), *eqns, constraints);
 
-    delete base;
     delete eqns;
     delete constraints;
 
@@ -209,13 +205,10 @@ NNormalSurfaceList* NNormalSurfaceList::enumerateStandardANDirect(
         NNormalSurfaceVectorANStandard::makeEmbeddedConstraints(owner);
     NMatrixInt* eqns = makeMatchingEquations(owner,
         NNormalSurfaceList::AN_STANDARD);
-    NNormalSurfaceVector* base = makeZeroVector(owner,
-        NNormalSurfaceList::AN_STANDARD);
 
-    NDoubleDescription::enumerateExtremalRays(SurfaceInserter(*list, owner),
-        *base, *eqns, constraints);
+    NDoubleDescription::enumerateExtremalRays<NNormalSurfaceVectorANStandard>(
+        SurfaceInserter(*list, owner), *eqns, constraints);
 
-    delete base;
     delete eqns;
     delete constraints;
 
@@ -229,7 +222,7 @@ NTriangulation* NNormalSurfaceList::getTriangulation() const {
 }
 
 #undef REGISTER_FLAVOUR
-#define REGISTER_FLAVOUR(id_name, c, n, almost_normal, t) \
+#define REGISTER_FLAVOUR(id_name, c, n, almost_normal, s) \
     case id_name: return almost_normal;
 
 bool NNormalSurfaceList::allowsAlmostNormal() const {
@@ -243,7 +236,21 @@ bool NNormalSurfaceList::allowsAlmostNormal() const {
 }
 
 #undef REGISTER_FLAVOUR
-#define REGISTER_FLAVOUR(id_name, c, name, a, t) \
+#define REGISTER_FLAVOUR(id_name, c, n, a, spun) \
+    case id_name: return spun;
+
+bool NNormalSurfaceList::allowsSpun() const {
+    switch(flavour) {
+        // Import cases from the flavour registry...
+        #include "surfaces/flavourregistry.h"
+        // ... and legacy cases:
+        case AN_LEGACY: return false;
+    }
+    return false;
+}
+
+#undef REGISTER_FLAVOUR
+#define REGISTER_FLAVOUR(id_name, c, name, a, s) \
     case id_name: o << name; break;
 
 void NNormalSurfaceList::writeTextShort(std::ostream& o) const {
@@ -266,7 +273,7 @@ void NNormalSurfaceList::writeTextShort(std::ostream& o) const {
 }
 
 #undef REGISTER_FLAVOUR
-#define REGISTER_FLAVOUR(id_name, c, name, a, t) \
+#define REGISTER_FLAVOUR(id_name, c, name, a, s) \
     case id_name: o << name << '\n'; break;
 
 void NNormalSurfaceList::writeTextLong(std::ostream& o) const {
@@ -305,7 +312,7 @@ void NNormalSurfaceList::writePacket(NFile& out) const {
 }
 
 #undef REGISTER_FLAVOUR
-#define REGISTER_FLAVOUR(id_name, c, name, a, t) \
+#define REGISTER_FLAVOUR(id_name, c, name, a, s) \
     case id_name: out << regina::xml::xmlEncodeSpecialChars(name); break;
 
 void NNormalSurfaceList::writeXMLPacketData(std::ostream& out) const {
@@ -333,7 +340,7 @@ void NNormalSurfaceList::writeXMLPacketData(std::ostream& out) const {
 }
 
 #undef REGISTER_FLAVOUR
-#define REGISTER_FLAVOUR(id_name, c, n, a, t) \
+#define REGISTER_FLAVOUR(id_name, c, n, a, s) \
     case id_name: break;
 
 NNormalSurfaceList* NNormalSurfaceList::readPacket(NFile& in,
@@ -364,6 +371,113 @@ NNormalSurfaceList* NNormalSurfaceList::readPacket(NFile& in,
     return ans;
 }
 
+NNormalSurfaceList* NNormalSurfaceList::filterForLocallyCompatiblePairs()
+        const {
+    // Sanity check:
+    if (! embedded)
+        return 0;
+
+    NNormalSurfaceList* ans = new NNormalSurfaceList();
+    ans->flavour = flavour;
+    ans->embedded = true;
+
+    // Find all surfaces that have a compatible partner.
+    std::vector<NNormalSurface*>::const_iterator first, second;
+    for (first = surfaces.begin(); first != surfaces.end(); ++first) {
+        for (second = surfaces.begin(); second != surfaces.end();
+                ++second) {
+            if (second == first)
+                continue;
+
+            if ((*first)->locallyCompatible(**second)) {
+                ans->surfaces.push_back((*first)->clone());
+                break;
+            }
+        }
+    }
+
+    getTriangulation()->insertChildLast(ans);
+    return ans;
+}
+
+NNormalSurfaceList* NNormalSurfaceList::filterForDisjointPairs() const {
+    // Sanity check:
+    if (! embedded)
+        return 0;
+
+    NNormalSurfaceList* ans = new NNormalSurfaceList();
+    ans->flavour = flavour;
+    ans->embedded = true;
+
+    // Collect all the surfaces that we might care about.
+    // This means non-empty, connected and compact.
+    std::vector<NNormalSurface*> interesting;
+    for (std::vector<NNormalSurface*>::const_iterator it = surfaces.begin();
+            it != surfaces.end(); ++it) {
+        if ((*it)->isEmpty())
+            continue;
+        if (! (*it)->isCompact())
+            continue;
+        if (! (*it)->isConnected().isTrue())
+            continue;
+
+        interesting.push_back(*it);
+    }
+
+    // Find all surfaces that have a disjoint partner.
+    std::vector<NNormalSurface*>::iterator first, second;
+    for (first = interesting.begin(); first != interesting.end(); ++first) {
+        for (second = interesting.begin(); second != interesting.end();
+                ++second) {
+            if (second == first)
+                continue;
+
+            if ((*first)->disjoint(**second)) {
+                ans->surfaces.push_back((*first)->clone());
+                break;
+            }
+        }
+    }
+
+    getTriangulation()->insertChildLast(ans);
+    return ans;
+}
+
+NNormalSurfaceList* NNormalSurfaceList::filterForPotentiallyIncompressible()
+        const {
+    NNormalSurfaceList* ans = new NNormalSurfaceList();
+    ans->flavour = flavour;
+    ans->embedded = true;
+
+    NTriangulation* t;
+#ifdef DEBUG
+    int which = 0;
+#endif
+    for (std::vector<NNormalSurface*>::const_iterator it = surfaces.begin();
+            it != surfaces.end(); ++it) {
+#ifdef DEBUG
+        std::cout << "Processing surface " << which++ << "..." << std::endl;
+#endif
+        if ((*it)->isVertexLinking())
+            continue;
+        if ((*it)->isThinEdgeLink().first)
+            continue;
+
+        // If we have a one-sided surface, don't worry about taking the
+        // two-sided double cover.  If the complement of the one-sided
+        // surface has a compressing disc, then the complement of the
+        // double cover has the same compressing disc, and this surface
+        // can happily be tossed away.
+        t = (*it)->cutAlong();
+        if (! t->hasSimpleCompressingDisc())
+            ans->surfaces.push_back((*it)->clone());
+        delete t;
+    }
+
+    getTriangulation()->insertChildLast(ans);
+    return ans;
+}
+
 NPacket* NNormalSurfaceList::internalClonePacket(NPacket* /* parent */) const {
     NNormalSurfaceList* ans = new NNormalSurfaceList();
     ans->flavour = flavour;
