@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nnormalsurfacelist.h
+/*! \file surfaces/nnormalsurfacelist.h
  *  \brief Contains a packet representing a collection of normal
  *  surfaces in a 3-manifold.
  */
@@ -39,6 +39,7 @@
 #include <algorithm>
 #include <iterator>
 #include <vector>
+#include "regina-core.h"
 #include "packet/npacket.h"
 #include "surfaces/nnormalsurface.h"
 #include "surfaces/nsurfaceset.h"
@@ -80,7 +81,7 @@ class NXMLNormalSurfaceListReader;
  * \todo \feature Generate facets of the solution space representing
  * embedded surfaces.
  */
-class NNormalSurfaceList : public NPacket, public NSurfaceSet {
+class REGINA_API NNormalSurfaceList : public NPacket, public NSurfaceSet {
     public:
         static const int packetType;
 
@@ -104,7 +105,7 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
          * Represents quadrilateral-octagon coordinates for octagonal
          * almost normal surfaces.  For details, see
          * "Quadrilateral-octagon coordinates for almost normal surfaces",
-         * Benjamin A. Burton, preprint, arXiv:0904.3041.
+         * Benjamin A. Burton, Experiment. Math. 19 (2010), 285-315.
          */
         static const int AN_QUAD_OCT;
 
@@ -222,14 +223,14 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
          * solutions in standard coordinates directly.  For details on
          * this procedure see "Converting between quadrilateral and
          * standard solution sets in normal surface theory",
-         * Benjamin A. Burton, preprint, arXiv:0901.2629.
+         * Benjamin A. Burton, Algebr. Geom. Topol. 9 (2009), 2121-2174.
          *
          * This routine allows the user to force a direct enumeration in
          * standard space, \e without going via quadrilateral space.
          * The algorithm used is the souped-up double description method
-         * in standard coordinates as described in "Optimising the
+         * in standard coordinates as described in "Optimizing the
          * double description method for normal surface enumeration",
-         * Benjamin A. Burton, to appear in Math. Comp., arXiv:0808.4050.
+         * Benjamin A. Burton, Math. Comp. 79 (2010), 453-484.
          *
          * Users will generally not want to call this routine, since it
          * is often much slower than enumerate() and it gives precisely
@@ -269,6 +270,7 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
 
         virtual int getFlavour() const;
         virtual bool allowsAlmostNormal() const;
+        virtual bool allowsSpun() const;
         virtual bool isEmbeddedOnly() const;
         virtual NTriangulation* getTriangulation() const;
         virtual unsigned long getNumberOfSurfaces() const;
@@ -303,7 +305,8 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
          * enumerating surfaces directly in standard tri-quad coordinates.
          * The underlying algorithm is described in detail in "Converting
          * between quadrilateral and standard solution sets in normal
-         * surface theory", Benjamin A. Burton, preprint, arXiv:0901.2629.
+         * surface theory", Benjamin A. Burton, Algebr. Geom. Topol. 9 (2009),
+         * 2121-2174.
          *
          * Typically users do not need to call this routine directly,
          * since the standard enumerate() routine will use it implicitly
@@ -392,7 +395,7 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
          * links are all spheres and/or discs.  The underlying algorithm
          * is described in detail in "Converting between quadrilateral and
          * standard solution sets in normal surface theory",
-         * Benjamin A. Burton, preprint, arXiv:0901.2629.
+         * Benjamin A. Burton, Algebr. Geom. Topol. 9 (2009), 2121-2174.
          *
          * It should be noted that this routine does \e not simply convert
          * vectors from one form to another; instead it converts a full
@@ -449,6 +452,137 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
          */
         NNormalSurfaceList* standardANToQuadOct() const;
 
+        /**
+         * Creates a new list filled with the surfaces from this list
+         * that have at least one locally compatible partner.
+         * In other words, a surface \a S from this list will be placed
+         * in the new list if and only if there is some other surface \a T
+         * in this list for which \a S and \a T are locally compatible.
+         * See NNormalSurface::locallyCompatible() for further details on
+         * compatibility testing.
+         *
+         * The new list will be inserted as a new child packet of the
+         * underlying triangulation (specifically, as the final child).  As a
+         * convenience, the new list will also be returned from this routine.
+         *
+         * This original list is not altered in any way.  Likewise,
+         * the surfaces in the new list are deep copies of the originals
+         * (so they can be altered without affecting the original surfaces).
+         *
+         * \pre This list contains only embedded normal surfaces.  More
+         * precisely, isEmbeddedOnly() must return \c true.
+         *
+         * \warning If this list contains a vertex link (plus at least
+         * one other surface), then the new list will be identical to
+         * the old (i.e., every surface will be copied across).
+         *
+         * @return the new list, which will also have been inserted as
+         * a new child packet of the underlying triangulation.
+         */
+        NNormalSurfaceList* filterForLocallyCompatiblePairs() const;
+
+        /**
+         * Creates a new list filled with the surfaces from this list
+         * that have at least one disjoint partner.
+         * In other words, a surface \a S from this list will be placed
+         * in the new list if and only if there is some other surface \a T
+         * in this list for which \a S and \a T can be made to intersect
+         * nowhere at all, without changing either normal isotopy class.
+         * See NNormalSurface::disjoint() for further details on disjointness
+         * testing.
+         *
+         * This routine cannot deal with empty, disconnected or
+         * non-compact surfaces.  Such surfaces will be silently
+         * ignored, and will not be used in any disjointness tests (in
+         * particular, they will never be considered as a "disjoint partner"
+         * for any other surface).
+         *
+         * The new list will be inserted as a new child packet of the
+         * underlying triangulation (specifically, as the final child).  As a
+         * convenience, the new list will also be returned from this routine.
+         *
+         * This original list is not altered in any way.  Likewise,
+         * the surfaces in the new list are deep copies of the originals
+         * (so they can be altered without affecting the original surfaces).
+         *
+         * \pre This list contains only embedded normal surfaces.  More
+         * precisely, isEmbeddedOnly() must return \c true.
+         * \pre All surfaces within this list are stored using the same
+         * flavour of coordinate system (i.e., the same subclass of
+         * NNormalSurfaceVector).
+         *
+         * \warning If this list contains a vertex link (plus at least
+         * one other surface), then the new list will be identical to
+         * the old (i.e., every surface will be copied across).
+         *
+         * \todo Deal properly with surfaces that are too large to handle.
+         *
+         * @return the new list, which will also have been inserted as
+         * a new child packet of the underlying triangulation.
+         */
+        NNormalSurfaceList* filterForDisjointPairs() const;
+
+        /**
+         * Creates a new list filled with only the surfaces from this list
+         * that "might" represent two-sided incompressible surfaces.
+         * More precisely, we consider all two-sided surfaces in this list,
+         * as well as the two-sided double covers of all one-sided surfaces
+         * in this list (see below for details on how one-sided surfaces
+         * are handled).  Each of these surfaces is examined using
+         * relatively fast heuristic tests for incompressibility.  Any
+         * surface that is definitely \e not incompressible is thrown
+         * away, and all other surfaces are placed in the new list.
+         *
+         * Therefore, it is guaranteed that every incompressible surface
+         * from the old list will be placed in the new list.  However,
+         * it is not known whether any given surface in the new list is
+         * indeed incompressible.
+         *
+         * See NNormalSurface::isIncompressible() for the definition of
+         * incompressibility that is used here.  Note in particular that
+         * spheres are \e never considered incompressible.
+         *
+         * As indicated above, this filter works exclusively with two-sided
+         * surfaces.  If a surface in this list is one-sided, the heuristic
+         * incompressibility tests will be run on its two-sided double cover.
+         * Nevertheless, if the tests pass, the original one-sided surface
+         * (not the double cover) will be added to the new list.
+         *
+         * The new list will be inserted as a new child packet of the
+         * underlying triangulation (specifically, as the final child).  As a
+         * convenience, the new list will also be returned from this routine.
+         *
+         * This original list is not altered in any way.  Likewise,
+         * the surfaces in the new list are deep copies of the originals
+         * (so they can be altered without affecting the original surfaces).
+         *
+         * Currently the heuristic tests include (i) throwing away
+         * all vertex links and thin edge links, and then
+         * (ii) cutting along the remaining surfaces and running
+         * NTriangulation::hasSimpleCompressingDisc() on the resulting
+         * bounded triangulations.  For more details on these tests
+         * see "The Weber-Seifert dodecahedral space is non-Haken",
+         * Benjamin A. Burton, J. Hyam Rubinstein and Stephan Tillmann,
+         * to appear in Trans. Amer. Math. Soc., arXiv:0909.4625.
+         *
+         * \pre The underlying 3-manifold triangulation is valid and closed.
+         * In particular, it has no ideal vertices.
+         * \pre This list contains only embedded normal surfaces.  More
+         * precisely, isEmbeddedOnly() must return \c true.
+         * \pre This list contains only compact, connected normal surfaces.
+         * \pre No surfaces in this list contain any octagonal discs.
+         *
+         * \warning The behaviour of this routine is subject to change
+         * in future versions of Regina, since additional tests may be
+         * added to improve the power of this filtering.
+         *
+         * \todo Add progress tracking.
+         *
+         * @return the new list, which will also have been inserted as
+         * a new child packet of the underlying triangulation.
+         */
+        NNormalSurfaceList* filterForPotentiallyIncompressible() const;
+
         /**
          * Returns a newly created matrix containing the matching
          * equations that were used to create this normal surface list.
@@ -468,7 +602,7 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
 
     protected:
         /**
-         * Creates a new normal surface list performing no intialisation
+         * Creates a new normal surface list performing no initialisation
          * whatsoever other than property initialisation.
          */
         NNormalSurfaceList();
@@ -543,7 +677,7 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
              * surface list.
              *
              * The given surface will be deallocated with the other
-             * surfaces in this list.
+             * surfaces in this list when the list is eventually destroyed.
              *
              * @param surface the normal surface to insert.
              * @return this output iterator.
@@ -555,7 +689,7 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
              *
              * The given vector will be owned by the newly created
              * normal surface and will be deallocated with the other
-             * surfaces in this list.
+             * surfaces in this list when the list is eventually destroyed.
              *
              * \warning The behaviour of this routine has changed!
              * As of Regina 4.6, this routine no longer checks for
@@ -625,6 +759,25 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
          */
         NNormalSurfaceList(int newFlavour, bool embeddedOnly);
 
+        /**
+         * Calls NDoubleDescription::enumerateExtremalRays() with the
+         * given arguments, and using the vector class that corresponds
+         * to the given coordinate flavour.
+         *
+         * All parameters not listed below are taken directly from
+         * NDoubleDescription::enumerateExtremalRays().
+         *
+         * @param flavour the flavour of coordinate system to be used;
+         * this must be one of the predefined coordinate system constants
+         * in NNormalSurfaceList.
+         * @return \c true if NDoubleDescription::enumerateExtremalRays()
+         * was successfully called, or \c false if the given flavour
+         * does not correspond to a known coordinate system.
+         */
+        static bool enumerateExtremalRays(int flavour,
+            const SurfaceInserter& results, const NMatrixInt& subspace,
+            const NEnumConstraintList* constraints, NProgressNumber* progress);
+
         /**
          * Enumerates all embedded vertex surfaces in (standard normal
          * or standard almost normal) coordinates by first enumerating
@@ -823,8 +976,8 @@ class NNormalSurfaceList : public NPacket, public NSurfaceSet {
  * constants in NNormalSurfaceList.
  * @return a new zero vector of the correct class and length.
  */
-NNormalSurfaceVector* makeZeroVector(const NTriangulation* triangulation,
-    int flavour);
+REGINA_API NNormalSurfaceVector* makeZeroVector(
+        const NTriangulation* triangulation, int flavour);
 /**
  * Creates a new set of normal surface matching equations for the
  * given triangulation using the given flavour of coordinate system.
@@ -842,7 +995,7 @@ NNormalSurfaceVector* makeZeroVector(const NTriangulation* triangulation,
  * constants in NNormalSurfaceList.
  * @return a newly allocated set of matching equations.
  */
-NMatrixInt* makeMatchingEquations(NTriangulation* triangulation,
+REGINA_API NMatrixInt* makeMatchingEquations(NTriangulation* triangulation,
     int flavour);
 
 /*@}*/
