@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nsanstandard.h
+/*! \file surfaces/nsanstandard.h
  *  \brief Implements almost normal surface vectors using standard
  *  triangle-quad-oct coordinates.
  */
@@ -36,6 +36,7 @@
 #define __NSANSTANDARD_H
 #endif
 
+#include "regina-core.h"
 #include "surfaces/nnormalsurface.h"
 
 namespace regina {
@@ -64,7 +65,7 @@ class NMatrixInt;
  *
  * \ifacespython Not present.
  */
-class NNormalSurfaceVectorANStandard : public NNormalSurfaceVector {
+class REGINA_API NNormalSurfaceVectorANStandard : public NNormalSurfaceVector {
     public:
         /**
          * Creates a new vector all of whose entries are initialised to
@@ -81,6 +82,7 @@ class NNormalSurfaceVectorANStandard : public NNormalSurfaceVector {
         NNormalSurfaceVectorANStandard(const NVector<NLargeInteger>& cloneMe);
 
         virtual bool allowsAlmostNormal() const;
+        virtual bool allowsSpun() const;
 
         virtual NLargeInteger getTriangleCoord(unsigned long tetIndex,
             int vertex, NTriangulation* triang) const;
@@ -93,7 +95,7 @@ class NNormalSurfaceVectorANStandard : public NNormalSurfaceVector {
         virtual NLargeInteger getFaceArcs(unsigned long faceIndex,
             int faceVertex, NTriangulation* triang) const;
 
-        virtual NVector<NLargeInteger>* clone() const;
+        virtual NNormalSurfaceVector* clone() const;
 
         static NNormalSurfaceVector* makeZeroVector(
             const NTriangulation* triangulation);
