@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nsquad.h
+/*! \file surfaces/nsquad.h
  *  \brief Implements normal surface vectors using quad coordinates.
  */
 
@@ -35,6 +35,7 @@
 #define __NSQUAD_H
 #endif
 
+#include "regina-core.h"
 #include "surfaces/nsmirrored.h"
 
 namespace regina {
@@ -59,7 +60,8 @@ class NMatrixInt;
  *
  * \ifacespython Not present.
  */
-class NNormalSurfaceVectorQuad : public NNormalSurfaceVectorMirrored {
+class REGINA_API NNormalSurfaceVectorQuad :
+        public NNormalSurfaceVectorMirrored {
     public:
         /**
          * Creates a new vector all of whose entries are initialised to
@@ -78,12 +80,13 @@ class NNormalSurfaceVectorQuad : public NNormalSurfaceVectorMirrored {
         virtual NNormalSurfaceVector* makeMirror(NTriangulation* triang) const;
 
         virtual bool allowsAlmostNormal() const;
+        virtual bool allowsSpun() const;
         virtual const NVertex* isVertexLink(NTriangulation* triang) const;
 
         virtual NLargeInteger getOctCoord(unsigned long tetIndex,
             int octType, NTriangulation* triang) const;
 
-        virtual NVector<NLargeInteger>* clone() const;
+        virtual NNormalSurfaceVector* clone() const;
 
         static NNormalSurfaceVector* makeZeroVector(
             const NTriangulation* triangulation);
