@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -44,7 +44,7 @@ void NXMLNormalSurfaceReader::startElement(const std::string&,
     name = props.lookup("name");
 }
 
-#define REGISTER_FLAVOUR(id_name, class, n, a, t) \
+#define REGISTER_FLAVOUR(id_name, class, n, a, s) \
     if (flavour == NNormalSurfaceList::id_name) \
         vec = new class(vecLen); \
     else
