@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nedge.h
+/*! \file triangulation/nedge.h
  *  \brief Deals with edges in a triangulation.
  */
 
@@ -36,14 +36,18 @@
 #endif
 
 #include <deque>
+#include "regina-core.h"
 #include "shareableobject.h"
-#include "triangulation/nperm.h"
-#include "triangulation/ntetrahedron.h"
+#include "maths/nperm4.h"
+#include "utilities/nmarkedvector.h"
+// NOTE: More #includes follow after the class declarations.
 
 namespace regina {
 
-class NComponent;
 class NBoundaryComponent;
+class NComponent;
+class NTetrahedron;
+class NVertex;
 
 /**
  * \weakgroup triangulation
@@ -104,7 +108,7 @@ extern const int edgeEnd[6];
  * Details how an edge in the skeleton forms part of an individual
  * tetrahedron.
  */
-class NEdgeEmbedding {
+class REGINA_API NEdgeEmbedding {
     private:
         NTetrahedron* tetrahedron;
             /**< The tetrahedron in which this edge is contained. */
@@ -163,13 +167,15 @@ class NEdgeEmbedding {
 
         /**
          * Returns a mapping from vertices (0,1) of this edge to the
-         * corresponding vertex numbers in getTetrahedron(), as described
-         * in NTetrahedron::getEdgeMapping().
+         * corresponding vertex numbers in getTetrahedron().  This
+         * permutation also maps (2,3) to the two remaining tetrahedron
+         * vertices in a manner that preserves orientation as you walk
+         * around the edge.  See NTetrahedron::getEdgeMapping() for details.
          *
          * @return a mapping from the vertices of this edge to the
          * vertices of getTetrahedron().
          */
-        NPerm getVertices() const;
+        NPerm4 getVertices() const;
 };
 
 /**
@@ -177,7 +183,7 @@ class NEdgeEmbedding {
  * Edges are highly temporary; once a triangulation changes, all its
  * edge objects will be deleted and new ones will be created.
  */
-class NEdge : public ShareableObject, public NMarkedElement {
+class REGINA_API NEdge : public ShareableObject, public NMarkedElement {
     public:
         /**
          * A table that maps vertices of a tetrahedron to edge numbers.
@@ -247,7 +253,7 @@ class NEdge : public ShareableObject, public NMarkedElement {
          * This lookup table replaces the deprecated routine
          * regina::edgeOrdering().
          */
-        static const NPerm ordering[6];
+        static const NPerm4 ordering[6];
 
     private:
         std::deque<NEdgeEmbedding> embeddings;
@@ -375,6 +381,11 @@ class NEdge : public ShareableObject, public NMarkedElement {
 
 /*@}*/
 
+} // namespace regina
+// Some more headers that are required for inline functions:
+#include "triangulation/ntetrahedron.h"
+namespace regina {
+
 // Inline functions for NEdge
 
 inline NEdge::NEdge(NComponent* myComponent) : component(myComponent),
@@ -452,7 +463,7 @@ inline int NEdgeEmbedding::getEdge() const {
     return edge;
 }
 
-inline NPerm NEdgeEmbedding::getVertices() const {
+inline NPerm4 NEdgeEmbedding::getVertices() const {
     return tetrahedron->getEdgeMapping(edge);
 }
 
