@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -125,6 +125,20 @@ NTriangulation* NExampleTriangulation::threeSphere() {
     return ans;
 }
 
+NTriangulation* NExampleTriangulation::bingsHouse() {
+    NTriangulation* ans = new NTriangulation();
+    ans->setPacketLabel("Bing's house with two rooms");
+
+    NTetrahedron* r = ans->newTetrahedron();
+    NTetrahedron* s = ans->newTetrahedron();
+    r->joinTo(0, r, NPerm4(0, 1));
+    s->joinTo(0, s, NPerm4(0, 1));
+    r->joinTo(3, s, NPerm4(3, 1, 0, 2));
+    s->joinTo(3, r, NPerm4(3, 1, 0, 2));
+
+    return ans;
+}
+
 NTriangulation* NExampleTriangulation::s2xs1() {
     NTriangulation* ans = new NTriangulation();
     ans->setPacketLabel("S2 x S1");
@@ -143,9 +157,8 @@ NTriangulation* NExampleTriangulation::rp2xs1() {
 
     NTetrahedron* r = ans->getTetrahedron(0);
     NTetrahedron* t = ans->getTetrahedron(2);
-    r->joinTo(1, t, NPerm(2, 3, 0, 1));
-    r->joinTo(3, t, NPerm(2, 3, 0, 1));
-    ans->gluingsHaveChanged();
+    r->joinTo(1, t, NPerm4(2, 3, 0, 1));
+    r->joinTo(3, t, NPerm4(2, 3, 0, 1));
 
     return ans;
 }
@@ -179,9 +192,9 @@ NTriangulation* NExampleTriangulation::poincareHomologySphere() {
     return ans;
 }
 
-NTriangulation* NExampleTriangulation::seifertWeber() {
+NTriangulation* NExampleTriangulation::weberSeifert() {
     NTriangulation* ans = new NTriangulation();
-    ans->setPacketLabel("Seifert-Weber dodecahedral space");
+    ans->setPacketLabel("Weber-Seifert dodecahedral space");
 
     // Bah.  Dehydration strings are somewhat impenetrable,
     // but the alternative is 23 lines of hard-coded tetrahedron gluings.
@@ -195,6 +208,14 @@ NTriangulation* NExampleTriangulation::seifertWeber() {
     return ans;
 }
 
+NTriangulation* NExampleTriangulation::seifertWeber() {
+    // Kept for backward compatibility.
+    // Use the old name in the packet label.
+    NTriangulation* ans = weberSeifert();
+    ans->setPacketLabel("Seifert-Weber dodecahedral space");
+    return ans;
+}
+
 NTriangulation* NExampleTriangulation::smallClosedOrblHyperbolic() {
     NTriangulation* ans = new NTriangulation();
     ans->setPacketLabel("Closed orientable hyperbolic 3-manifold");
@@ -228,16 +249,13 @@ NTriangulation* NExampleTriangulation::solidKleinBottle() {
 
     // A three-tetrahedron solid Klein bottle is described in section
     // 3.5.1 of Benjamin Burton's PhD thesis.
-    NTetrahedron* r = new NTetrahedron();
-    NTetrahedron* s = new NTetrahedron();
-    NTetrahedron* t = new NTetrahedron();
-    s->joinTo(0, r, NPerm(0, 1, 2, 3));
-    s->joinTo(3, r, NPerm(3, 0, 1, 2));
-    s->joinTo(1, t, NPerm(3, 0, 1, 2));
-    s->joinTo(2, t, NPerm(0, 1, 2, 3));
-    ans->addTetrahedron(r);
-    ans->addTetrahedron(s);
-    ans->addTetrahedron(t);
+    NTetrahedron* r = ans->newTetrahedron();
+    NTetrahedron* s = ans->newTetrahedron();
+    NTetrahedron* t = ans->newTetrahedron();
+    s->joinTo(0, r, NPerm4(0, 1, 2, 3));
+    s->joinTo(3, r, NPerm4(3, 0, 1, 2));
+    s->joinTo(1, t, NPerm4(3, 0, 1, 2));
+    s->joinTo(2, t, NPerm4(0, 1, 2, 3));
 
     return ans;
 }
@@ -248,14 +266,12 @@ NTriangulation* NExampleTriangulation::figureEightKnotComplement() {
 
     // The two-tetrahedron figure eight knot complement is described at
     // the beginning of chapter 8 of Richard Rannard's PhD thesis.
-    NTetrahedron* r = new NTetrahedron();
-    NTetrahedron* s = new NTetrahedron();
-    r->joinTo(0, s, NPerm(1, 3, 0, 2));
-    r->joinTo(1, s, NPerm(2, 0, 3, 1));
-    r->joinTo(2, s, NPerm(0, 3, 2, 1));
-    r->joinTo(3, s, NPerm(2, 1, 0, 3));
-    ans->addTetrahedron(r);
-    ans->addTetrahedron(s);
+    NTetrahedron* r = ans->newTetrahedron();
+    NTetrahedron* s = ans->newTetrahedron();
+    r->joinTo(0, s, NPerm4(1, 3, 0, 2));
+    r->joinTo(1, s, NPerm4(2, 0, 3, 1));
+    r->joinTo(2, s, NPerm4(0, 3, 2, 1));
+    r->joinTo(3, s, NPerm4(2, 1, 0, 3));
 
     return ans;
 }
@@ -273,10 +289,9 @@ NTriangulation* NExampleTriangulation::gieseking() {
     NTriangulation* ans = new NTriangulation();
     ans->setPacketLabel("Gieseking manifold");
 
-    NTetrahedron* r = new NTetrahedron();
-    r->joinTo(0, r, NPerm(1, 2, 0, 3));
-    r->joinTo(2, r, NPerm(0, 2, 3, 1));
-    ans->addTetrahedron(r);
+    NTetrahedron* r = ans->newTetrahedron();
+    r->joinTo(0, r, NPerm4(1, 2, 0, 3));
+    r->joinTo(2, r, NPerm4(0, 2, 3, 1));
 
     return ans;
 }
@@ -287,19 +302,15 @@ NTriangulation* NExampleTriangulation::cuspedGenusTwoTorus() {
 
     // We create this by first constructing an ordinary solid genus two
     // torus and then converting the real boundary to an ideal vertex.
-    NTetrahedron* r = new NTetrahedron();
-    NTetrahedron* s = new NTetrahedron();
-    NTetrahedron* t = new NTetrahedron();
-    NTetrahedron* u = new NTetrahedron();
-    r->joinTo(0, s, NPerm());
-    r->joinTo(1, t, NPerm(1, 2, 3, 0));
-    r->joinTo(2, u, NPerm(1, 0, 3, 2));
-    s->joinTo(3, t, NPerm());
-    t->joinTo(1, u, NPerm());
-    ans->addTetrahedron(r);
-    ans->addTetrahedron(s);
-    ans->addTetrahedron(t);
-    ans->addTetrahedron(u);
+    NTetrahedron* r = ans->newTetrahedron();
+    NTetrahedron* s = ans->newTetrahedron();
+    NTetrahedron* t = ans->newTetrahedron();
+    NTetrahedron* u = ans->newTetrahedron();
+    r->joinTo(0, s, NPerm4());
+    r->joinTo(1, t, NPerm4(1, 2, 3, 0));
+    r->joinTo(2, u, NPerm4(1, 0, 3, 2));
+    s->joinTo(3, t, NPerm4());
+    t->joinTo(1, u, NPerm4());
     ans->finiteToIdeal();
 
     return ans;
