@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nface.h
+/*! \file triangulation/nface.h
  *  \brief Deals with faces in a triangulation.
  */
 
@@ -35,14 +35,19 @@
 #define __NFACE_H
 #endif
 
+#include "regina-core.h"
 #include "shareableobject.h"
-#include "triangulation/nperm.h"
-#include "triangulation/ntetrahedron.h"
+#include "maths/nperm4.h"
+#include "utilities/nmarkedvector.h"
+// NOTE: More #includes follow after the class declarations.
 
 namespace regina {
 
-class NComponent;
 class NBoundaryComponent;
+class NComponent;
+class NEdge;
+class NTetrahedron;
+class NVertex;
 
 /**
  * \weakgroup triangulation
@@ -53,7 +58,7 @@ class NBoundaryComponent;
  * Details how a face in the skeleton forms part of an individual
  * tetrahedron.
  */
-class NFaceEmbedding {
+class REGINA_API NFaceEmbedding {
     private:
         NTetrahedron* tetrahedron;
             /**< The tetrahedron in which this face is contained. */
@@ -101,7 +106,7 @@ class NFaceEmbedding {
          * @return a mapping from the vertices of this face to the
          * vertices of getTetrahedron().
          */
-        NPerm getVertices() const;
+        NPerm4 getVertices() const;
 };
 
 /**
@@ -109,7 +114,7 @@ class NFaceEmbedding {
  * Faces are highly temporary; once a triangulation changes, all its
  * face objects will be deleted and new ones will be created.
  */
-class NFace : public ShareableObject, public NMarkedElement {
+class REGINA_API NFace : public ShareableObject, public NMarkedElement {
     public:
         static const int TRIANGLE;
             /**< Specifies a face with no identified vertices or edges. */
@@ -151,7 +156,7 @@ class NFace : public ShareableObject, public NMarkedElement {
          * This lookup table replaces the deprecated routine
          * regina::faceOrdering().
          */
-        static const NPerm ordering[4];
+        static const NPerm4 ordering[4];
 
     private:
         NFaceEmbedding* embeddings[2];
@@ -316,7 +321,7 @@ class NFace : public ShareableObject, public NMarkedElement {
          * @return a mapping from vertices (0,1) of the requested edge to
          * the vertices of this face.
          */
-        NPerm getEdgeMapping(int edge) const;
+        NPerm4 getEdgeMapping(int edge) const;
 
         void writeTextShort(std::ostream& out) const;
 
@@ -336,6 +341,11 @@ class NFace : public ShareableObject, public NMarkedElement {
 
 /*@}*/
 
+} // namespace regina
+// Some more headers that are required for inline functions:
+#include "triangulation/ntetrahedron.h"
+namespace regina {
+
 // Inline functions for NFace
 
 inline NFace::NFace(NComponent* myComponent) : nEmbeddings(0),
@@ -409,7 +419,7 @@ inline int NFaceEmbedding::getFace() const {
     return face;
 }
 
-inline NPerm NFaceEmbedding::getVertices() const {
+inline NPerm4 NFaceEmbedding::getVertices() const {
     return tetrahedron->getFaceMapping(face);
 }
 
