@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nfacepair.h
+/*! \file triangulation/nfacepair.h
  *  \brief Deals with simple pairs of face numbers.
  */
 
@@ -35,6 +35,8 @@
 #define __NFACEPAIR_H
 #endif
 
+#include "regina-core.h"
+
 namespace regina {
 
 /**
@@ -60,7 +62,7 @@ namespace regina {
  * The before-the-start and past-the-end values are internally
  * represented as (0,0) and (3,3) respectively.
  */
-class NFacePair {
+class REGINA_API NFacePair {
     private:
         unsigned first;
             /**< The smaller of the two faces in this pair. */
