@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nisomorphism.h
+/*! \file triangulation/nisomorphism.h
  *  \brief Deals with combinatorial isomorphisms of triangulations.
  */
 
@@ -35,8 +35,9 @@
 #define __NISOMORPHISM_H
 #endif
 
+#include "regina-core.h"
 #include "shareableobject.h"
-#include "triangulation/nperm.h"
+#include "maths/nperm4.h"
 #include "triangulation/ntetface.h"
 
 namespace regina {
@@ -82,14 +83,14 @@ class NTriangulation;
  *
  * \todo \feature Composition of isomorphisms.
  */
-class NIsomorphism : public ShareableObject {
+class REGINA_API NIsomorphism : public ShareableObject {
     protected:
         unsigned nTetrahedra;
             /**< The number of tetrahedra in the source triangulation. */
         int* mTetImage;
             /**< The tetrahedron of the destination triangulation that
                  each tetrahedron of the source triangulation maps to. */
-        NPerm* mFacePerm;
+        NPerm4* mFacePerm;
             /**< The permutation applied to the four faces of each
                  source tetrahedron. */
 
@@ -165,7 +166,7 @@ class NIsomorphism : public ShareableObject {
          * @return a read-write reference to the permutation applied to the
          * four faces of the source tetrahedron.
          */
-        NPerm& facePerm(unsigned sourceTet);
+        NPerm4& facePerm(unsigned sourceTet);
         /**
          * Determines the permutation that is applied to the four faces
          * of the given source tetrahedron under this isomorphism.
@@ -179,7 +180,7 @@ class NIsomorphism : public ShareableObject {
          * @return the permutation applied to the four faces of the
          * source tetrahedron.
          */
-        NPerm facePerm(unsigned sourceTet) const;
+        NPerm4 facePerm(unsigned sourceTet) const;
         /**
          * Determines the image of the given source tetrahedron face
          * under this isomorphism.  Note that a value only is returned; this
@@ -206,7 +207,8 @@ class NIsomorphism : public ShareableObject {
         bool isIdentity() const;
 
         /**
-         * Applies this isomorphism to the given triangulation.
+         * Applies this isomorphism to the given triangulation and
+         * returns the result as a new triangulation.
          *
          * The given triangulation (call this T) is not modified in any way.
          * A new triangulation (call this S) is returned, so that this
@@ -244,6 +246,40 @@ class NIsomorphism : public ShareableObject {
          */
         NTriangulation* apply(const NTriangulation* original) const;
 
+        /**
+         * Applies this isomorphism to the given triangulation,
+         * modifying the given triangulation directly.
+         *
+         * This is similar to apply(), except that instead of creating a
+         * new triangulation, the tetrahedra and vertices of the given
+         * triangulation are modified directly.
+         *
+         * See apply() for further details on how this operation is performed.
+         *
+         * As with apply(), there are several preconditions to this routine.
+         * This routine does a small amount of sanity checking (and returns
+         * without changes if an error is detected), but it certainly does
+         * not check the entire set of preconditions.  It is up to the
+         * caller of this routine to verify that all of the following
+         * preconditions are met.
+         *
+         * \pre The number of tetrahedra in the given triangulation is
+         * precisely the number returned by getSourceTetrahedra() for
+         * this isomorphism.
+         * \pre This is a valid isomorphism (i.e., it has been properly
+         * initialised, so that all tetrahedron images are non-negative
+         * and distinct, and all face permutations are real permutations
+         * of (0,1,2,3).
+         * \pre Each tetrahedron image for this isomorphism lies
+         * between 0 and <tt>getSourceTetrahedra()-1</tt> inclusive
+         * (i.e., this isomorphism does not represent a mapping from a
+         * smaller triangulation into a larger triangulation).
+         *
+         * @param tri the triangulation to which this isomorphism
+         * should be applied.
+         */
+        void applyInPlace(NTriangulation* tri) const;
+
         void writeTextShort(std::ostream& out) const;
         void writeTextLong(std::ostream& out) const;
 
@@ -283,7 +319,7 @@ class NIsomorphism : public ShareableObject {
  *
  * \ifacespython Not present.
  */
-class NIsomorphismDirect : public NIsomorphism {
+class REGINA_API NIsomorphismDirect : public NIsomorphism {
     public:
         /**
          * Creates a new isomorphism with no initialisation.
@@ -310,7 +346,7 @@ inline NIsomorphism::NIsomorphism(unsigned sourceTetrahedra) :
         mTetImage(sourceTetrahedra > 0 ?
             new int[sourceTetrahedra] : 0),
         mFacePerm(sourceTetrahedra > 0 ?
-            new NPerm[sourceTetrahedra] : 0) {
+            new NPerm4[sourceTetrahedra] : 0) {
 }
 inline NIsomorphism::~NIsomorphism() {
     delete[] mTetImage;
@@ -328,10 +364,10 @@ inline int NIsomorphism::tetImage(unsigned sourceTet) const {
     return mTetImage[sourceTet];
 }
 
-inline NPerm& NIsomorphism::facePerm(unsigned sourceTet) {
+inline NPerm4& NIsomorphism::facePerm(unsigned sourceTet) {
     return mFacePerm[sourceTet];
 }
-inline NPerm NIsomorphism::facePerm(unsigned sourceTet) const {
+inline NPerm4 NIsomorphism::facePerm(unsigned sourceTet) const {
     return mFacePerm[sourceTet];
 }
 
