@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,9 +26,11 @@
 
 /* end stub */
 
+#include <cassert>
 #include <iostream>
 #include <iomanip>
 #include <sstream>
+#include <stack>
 
 #include "file/nfile.h"
 #include "triangulation/ntriangulation.h"
@@ -48,12 +50,91 @@
 
 namespace regina {
 
-void NTriangulation::clearAllProperties() {
-    if (calculatedSkeleton) {
-        deleteSkeleton();
-        calculatedSkeleton = false;
+void NTriangulation::addTetrahedron(NTetrahedron* t) {
+    // Make this a no-op if the tetrahedron has already been added.
+    if (t->tri == this)
+        return;
+    assert(t->tri == 0);
+
+    ChangeEventSpan span(this);
+
+    t->tri = this;
+    tetrahedra.push_back(t);
+
+    // Aggressively add neighbours of t (recursively).
+    // First check whether this is even necessary.
+    bool moreToAdd = false;
+    int i;
+    for (i = 0; i < 4; ++i)
+        if (t->adjacentTetrahedron(i) && ! t->adjacentTetrahedron(i)->tri) {
+            moreToAdd = true;
+            break;
+        }
+    if (moreToAdd) {
+        // Yep, it's necessary.. off we go.
+        std::stack<NTetrahedron*> toFollow;
+        toFollow.push(t);
+
+        NTetrahedron* next;
+        NTetrahedron* adj;
+        while (! toFollow.empty()) {
+            next = toFollow.top();
+            toFollow.pop();
+            for (i = 0; i < 4; ++i) {
+                adj = next->adjacentTetrahedron(i);
+                if (adj && ! adj->tri) {
+                    adj->tri = this;
+                    tetrahedra.push_back(adj);
+                    toFollow.push(adj);
+                }
+            }
+        }
     }
 
+    clearAllProperties();
+}
+
+void NTriangulation::swapContents(NTriangulation& other) {
+    ChangeEventSpan span1(this);
+    ChangeEventSpan span2(&other);
+
+    clearAllProperties();
+    other.clearAllProperties();
+
+    tetrahedra.swap(other.tetrahedra);
+
+    TetrahedronIterator it;
+    for (it = tetrahedra.begin(); it != tetrahedra.end(); ++it)
+        (*it)->tri = this;
+    for (it = other.tetrahedra.begin(); it != other.tetrahedra.end(); ++it)
+        (*it)->tri = &other;
+}
+
+void NTriangulation::moveContentsTo(NTriangulation& dest) {
+    ChangeEventSpan span1(this);
+    ChangeEventSpan span2(&dest);
+
+    clearAllProperties();
+    dest.clearAllProperties();
+
+    TetrahedronIterator it;
+    for (it = tetrahedra.begin(); it != tetrahedra.end(); ++it) {
+        // This is an abuse of NMarkedVector, since for a brief moment
+        // each tetrahedron belongs to both vectors
+        // tetrahedra and dest.tetrahedra.
+        // However, the subsequent clear() operation does not touch the
+        // tetrahedron markings (indices), and so we end up with the
+        // correct result (i.e., the markings are correct for dest).
+        (*it)->tri = &dest;
+        dest.tetrahedra.push_back(*it);
+    }
+    tetrahedra.clear();
+}
+
+void NTriangulation::clearAllProperties() {
+    if (calculatedSkeleton)
+        deleteSkeleton();
+
     fundamentalGroup.clear();
     H1.clear();
     H1Rel.clear();
@@ -84,7 +165,7 @@ void NTriangulation::writeTextLong(std::ostream& out) const {
     NTetrahedron* adjTet;
     unsigned tetPos;
     int face, vertex, start, end;
-    NPerm adjPerm;
+    NPerm4 adjPerm;
 
     out << "Tetrahedron gluing:\n";
     out << "  Tet  |  glued to:      (012)      (013)      (023)      (123)\n";
@@ -154,7 +235,7 @@ void NTriangulation::writeTextLong(std::ostream& out) const {
 void NTriangulation::writePacket(NFile& out) const {
     NTetrahedron* tet;
     NTetrahedron* adjTet;
-    NPerm adjPerm;
+    NPerm4 adjPerm;
     int face;
     long tetPos, adjPos;
 
@@ -235,28 +316,24 @@ void NTriangulation::writePacket(NFile& out) const {
 
 NTriangulation* NTriangulation::readPacket(NFile& in, NPacket* /* parent */) {
     NTriangulation* triang = new NTriangulation();
-    NTetrahedron* tet;
 
     // Create new tetrahedra.
     unsigned long nTet = in.readULong();
-    for (unsigned long i=0; i<nTet; i++) {
-        tet = new NTetrahedron();
-        tet->setDescription(in.readString());
-        triang->addTetrahedron(tet);
-    }
+    for (unsigned long i=0; i<nTet; i++)
+        triang->newTetrahedron(in.readString());
 
     // Read in the joins.
     long tetPos, altPos;
     int face;
-    NPerm adjPerm;
+    NPerm4 adjPerm;
 
     tetPos = in.readLong();
     while (tetPos >= 0) {
         face = in.readInt();
         altPos = in.readLong();
-        adjPerm = in.readChar();
+        adjPerm.setPermCode(in.readChar());
         triang->tetrahedra[tetPos]->joinTo(face,
-            triang->tetrahedra[altPos], NPerm(adjPerm));
+            triang->tetrahedra[altPos], adjPerm);
         tetPos = in.readLong();
     }
 
@@ -378,7 +455,7 @@ NTriangulation* NTriangulation::enterTextTriangulation(std::istream& in,
     out << '\n';
 
     for (long i=0; i<nTet; i++)
-        triang->addTetrahedron(new NTetrahedron());
+        triang->newTetrahedron();
 
     // Read in the joins.
     long tetPos, altPos;
@@ -446,11 +523,10 @@ NTriangulation* NTriangulation::enterTextTriangulation(std::istream& in,
         }
 
         tet->joinTo(face, altTet,
-            NPerm(vertices[0], vertices[3], vertices[1], vertices[4],
+            NPerm4(vertices[0], vertices[3], vertices[1], vertices[4],
                 vertices[2], vertices[5], face, altFace));
         out << '\n';
     }
-    triang->gluingsHaveChanged();
 
     out << "Finished reading gluings.\n";
     out << "The triangulation has been successfully created.\n";
@@ -513,22 +589,24 @@ void NTriangulation::deleteSkeleton() {
     faces.clear();
     components.clear();
     boundaryComponents.clear();
+
+    calculatedSkeleton = false;
 }
 
 void NTriangulation::cloneFrom(const NTriangulation& X) {
-    ChangeEventBlock(this);
+    ChangeEventSpan span(this);
 
     removeAllTetrahedra();
 
     TetrahedronIterator it;
     for (it = X.tetrahedra.begin(); it != X.tetrahedra.end(); it++)
-        addTetrahedron(new NTetrahedron((*it)->getDescription()));
+        newTetrahedron((*it)->getDescription());
 
     // Make the gluings.
     long tetPos, adjPos;
     NTetrahedron* tet;
     NTetrahedron* adjTet;
-    NPerm adjPerm;
+    NPerm4 adjPerm;
     int face;
     tetPos = 0;
     for (it = X.tetrahedra.begin(); it != X.tetrahedra.end(); it++) {
@@ -547,7 +625,6 @@ void NTriangulation::cloneFrom(const NTriangulation& X) {
         }
         tetPos++;
     }
-    gluingsHaveChanged();
 
     // Properties:
     if (X.fundamentalGroup.known())
@@ -572,19 +649,19 @@ void NTriangulation::cloneFrom(const NTriangulation& X) {
 }
 
 void NTriangulation::insertTriangulation(const NTriangulation& X) {
-    ChangeEventBlock(this);
+    ChangeEventSpan span(this);
 
     unsigned long norig = getNumberOfTetrahedra();
 
     TetrahedronIterator it;
     for (it = X.tetrahedra.begin(); it != X.tetrahedra.end(); it++)
-        addTetrahedron(new NTetrahedron((*it)->getDescription()));
+        newTetrahedron((*it)->getDescription());
 
     // Make the gluings.
     long tetPos, adjPos;
     NTetrahedron* tet;
     NTetrahedron* adjTet;
-    NPerm adjPerm;
+    NPerm4 adjPerm;
     int face;
     tetPos = 0;
     for (it = X.tetrahedra.begin(); it != X.tetrahedra.end(); it++) {
@@ -603,8 +680,6 @@ void NTriangulation::insertTriangulation(const NTriangulation& X) {
         }
         tetPos++;
     }
-
-    gluingsHaveChanged();
 }
 
 void NTriangulation::insertConstruction(unsigned long nTetrahedra,
@@ -612,29 +687,25 @@ void NTriangulation::insertConstruction(unsigned long nTetrahedra,
     if (nTetrahedra == 0)
         return;
 
+    ChangeEventSpan span(this);
+
     NTetrahedron** tet = new NTetrahedron*[nTetrahedra];
 
     unsigned i, j;
-    NPerm p;
+    NPerm4 p;
 
     for (i = 0; i < nTetrahedra; i++)
-        tet[i] = new NTetrahedron();
+        tet[i] = newTetrahedron();
 
     for (i = 0; i < nTetrahedra; i++)
         for (j = 0; j < 4; j++)
             if (adjacencies[i][j] >= 0 &&
                     ! tet[i]->adjacentTetrahedron(j)) {
-                p = NPerm(gluings[i][j][0], gluings[i][j][1],
+                p = NPerm4(gluings[i][j][0], gluings[i][j][1],
                     gluings[i][j][2], gluings[i][j][3]);
                 tet[i]->joinTo(j, tet[adjacencies[i][j]], p);
             }
 
-    // It's not until here that we actually modify this triangulation.
-    ChangeEventBlock(this);
-
-    for (i = 0; i < nTetrahedra; i++)
-        addTetrahedron(tet[i]);
-
     delete[] tet;
 }
 
@@ -665,7 +736,7 @@ std::string NTriangulation::dumpConstruction() const {
 
     unsigned long nTetrahedra = tetrahedra.size();
     NTetrahedron* tet;
-    NPerm p;
+    NPerm4 p;
     unsigned long t;
     int f, i;
 
