@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -27,6 +27,7 @@
 /* end stub */
 
 #include <algorithm>
+#include <set>
 #include "triangulation/ntriangulation.h"
 
 namespace regina {
@@ -38,8 +39,8 @@ namespace {
     //     face embedding permutation for the internal face is the identity.
     // Also, threeTwoVertices[i] refers to face i of the new tetrahedron for
     //     each i.
-    const NPerm threeTwoVertices[3] = {
-        NPerm(3,1,2,0), NPerm(3,2,0,1), NPerm(3,0,1,2)
+    const NPerm4 threeTwoVertices[3] = {
+        NPerm4(3,1,2,0), NPerm4(3,2,0,1), NPerm4(3,0,1,2)
     };
 
     // Mapping from vertices (0,1,2) of each external face of a new tetrahedron
@@ -48,8 +49,8 @@ namespace {
     //     edge embedding permutation for the internal edge is the identity.
     // Also, twoThreeVertices[i] refers to face i of the new tetrahedron for
     //     each i.
-    const NPerm twoThreeVertices[2] = {
-        NPerm(1,2,3,0), NPerm(0,2,3,1)
+    const NPerm4 twoThreeVertices[2] = {
+        NPerm4(1,2,3,0), NPerm4(0,2,3,1)
     };
 
     // A helper routine that uses union-find to test whether a graph
@@ -95,8 +96,8 @@ bool NTriangulation::threeTwoMove(NEdge* e, bool check, bool perform) {
 
     // Find the unwanted tetrahedra.
     NTetrahedron* oldTet[3];
-    NPerm oldVertexPerm[3];
-    stdhash::hash_set<NTetrahedron*, HashPointer> oldTets;
+    NPerm4 oldVertexPerm[3];
+    std::set<NTetrahedron*> oldTets;
     int oldPos = 0;
     for (std::deque<NEdgeEmbedding>::const_iterator it = embs.begin();
             it != embs.end(); it++) {
@@ -116,17 +117,17 @@ bool NTriangulation::threeTwoMove(NEdge* e, bool check, bool perform) {
     #endif
 
     // Perform the move.
-    ChangeEventBlock block(this);
+    ChangeEventSpan span(this);
     int oldPos2, newPos, newPos2;
 
-    // Allocate the new tetrahedra.
+    // Create the new tetrahedra.
     NTetrahedron* newTet[2];
     for (newPos = 0; newPos < 2; newPos++)
-        newTet[newPos] = new NTetrahedron();
+        newTet[newPos] = newTetrahedron();
 
     // Find the gluings from (0,1,2) of the new tetrahedron faces
     // to the vertices of the old tetrahedra.
-    NPerm gluings[2][3];
+    NPerm4 gluings[2][3];
     for (oldPos = 0; oldPos < 3; oldPos++)
         for (newPos = 0; newPos < 2; newPos++)
             gluings[newPos][oldPos] = oldVertexPerm[oldPos] *
@@ -191,11 +192,7 @@ bool NTriangulation::threeTwoMove(NEdge* e, bool check, bool perform) {
 
     // Remove the old tetrahedra from the triangulation.
     for (oldPos = 0; oldPos < 3; oldPos++)
-        delete removeTetrahedron(oldTet[oldPos]);
-
-    // Insert the new tetrahedra into the triangulation.
-    for (newPos = 0; newPos < 2; newPos++)
-        addTetrahedron(newTet[newPos]);
+        removeTetrahedron(oldTet[oldPos]);
 
     // Glue the faces of the new tetrahedra.
     for (oldPos = 0; oldPos < 3; oldPos++)
@@ -204,10 +201,9 @@ bool NTriangulation::threeTwoMove(NEdge* e, bool check, bool perform) {
                 newTet[newPos]->joinTo(oldPos, adjTet[newPos][oldPos],
                     gluings[newPos][oldPos] *
                     threeTwoVertices[oldPos].inverse());
-    newTet[0]->joinTo(3, newTet[1], NPerm());
+    newTet[0]->joinTo(3, newTet[1], NPerm4());
 
-    // Tidy up.
-    gluingsHaveChanged();
+    // Done!.
     return true;
 }
 
@@ -220,7 +216,7 @@ bool NTriangulation::twoThreeMove(NFace* f, bool check, bool perform) {
 
     // Find the unwanted tetrahedra.
     NTetrahedron* oldTet[2];
-    NPerm oldVertexPerm[2];
+    NPerm4 oldVertexPerm[2];
     int oldPos;
     for (oldPos = 0; oldPos < 2; oldPos++) {
         oldTet[oldPos] = f->getEmbedding(oldPos).getTetrahedron();
@@ -239,17 +235,17 @@ bool NTriangulation::twoThreeMove(NFace* f, bool check, bool perform) {
     #endif
 
     // Actually perform the move.
-    ChangeEventBlock block(this);
+    ChangeEventSpan span(this);
     int oldPos2, newPos, newPos2;
 
     // Allocate the new tetrahedra.
     NTetrahedron* newTet[3];
     for (newPos = 0; newPos < 3; newPos++)
-        newTet[newPos] = new NTetrahedron();
+        newTet[newPos] = newTetrahedron();
 
     // Find the gluings from (0,1,2) of the new tetrahedron faces
     // to the vertices of the old tetrahedra.
-    NPerm gluings[3][2];
+    NPerm4 gluings[3][2];
     for (oldPos = 0; oldPos < 2; oldPos++)
         for (newPos = 0; newPos < 3; newPos++)
             gluings[newPos][oldPos] = oldVertexPerm[oldPos] *
@@ -314,11 +310,7 @@ bool NTriangulation::twoThreeMove(NFace* f, bool check, bool perform) {
 
     // Remove the old tetrahedra from the triangulation.
     for (oldPos = 0; oldPos < 2; oldPos++)
-        delete removeTetrahedron(oldTet[oldPos]);
-
-    // Insert the new tetrahedra into the triangulation.
-    for (newPos = 0; newPos < 3; newPos++)
-        addTetrahedron(newTet[newPos]);
+        removeTetrahedron(oldTet[oldPos]);
 
     // Glue the faces of the new tetrahedra.
     for (oldPos = 0; oldPos < 2; oldPos++)
@@ -327,13 +319,12 @@ bool NTriangulation::twoThreeMove(NFace* f, bool check, bool perform) {
                 newTet[newPos]->joinTo(oldPos, adjTet[newPos][oldPos],
                     gluings[newPos][oldPos] *
                     twoThreeVertices[oldPos].inverse());
-    NPerm internalPerm = NPerm(0,1,3,2);
+    NPerm4 internalPerm = NPerm4(0,1,3,2);
     newTet[0]->joinTo(2, newTet[1], internalPerm);
     newTet[1]->joinTo(2, newTet[2], internalPerm);
     newTet[2]->joinTo(2, newTet[0], internalPerm);
 
-    // Tidy up.
-    gluingsHaveChanged();
+    // Done!
     return true;
 }
 
@@ -349,7 +340,7 @@ bool NTriangulation::fourFourMove(NEdge* e, int newAxis, bool check,
 
     // Find the unwanted tetrahedra.
     NTetrahedron* oldTet[4];
-    stdhash::hash_set<NTetrahedron*, HashPointer> oldTets;
+    std::set<NTetrahedron*> oldTets;
     int oldPos = 0;
     for (std::deque<NEdgeEmbedding>::const_iterator it = embs.begin();
             it != embs.end(); it++) {
@@ -368,18 +359,16 @@ bool NTriangulation::fourFourMove(NEdge* e, int newAxis, bool check,
     #endif
 
     // Perform the 4-4 move as a 2-3 move followed by a 3-2 move.
-    ChangeEventBlock block(this);
+    ChangeEventSpan span(this);
     NFace* face23 = (newAxis == 0 ?
         oldTet[0]->getFace(embs[0].getVertices()[2]) :
         oldTet[1]->getFace(embs[1].getVertices()[2]));
     int edge32 = embs[3].getEdge();
 
     twoThreeMove(face23, false, true);
-    calculateSkeleton();
     threeTwoMove(oldTet[3]->getEdge(edge32), false, true);
 
-    // Tidy up.  Note that gluingsHaveChanged() was already called by
-    // twoThreeMove() and threeTwoMove().
+    // Done!
     return true;
 }
 
@@ -392,7 +381,7 @@ bool NTriangulation::twoZeroMove(NEdge* e, bool check, bool perform) {
     }
 
     NTetrahedron* tet[2];
-    NPerm perm[2];
+    NPerm4 perm[2];
 
     int i = 0;
     for (std::deque<NEdgeEmbedding>::const_iterator it =
@@ -442,12 +431,12 @@ bool NTriangulation::twoZeroMove(NEdge* e, bool check, bool perform) {
     #endif
 
     // Actually perform the move.
-    ChangeEventBlock block(this);
+    ChangeEventSpan span(this);
 
     // Unglue faces from the doomed tetrahedra and glue them to each
     // other.
-    NPerm crossover = tet[0]->adjacentGluing(perm[0][2]);
-    NPerm gluing;
+    NPerm4 crossover = tet[0]->adjacentGluing(perm[0][2]);
+    NPerm4 gluing;
     NTetrahedron* top;
     NTetrahedron* bottom;
     int topFace;
@@ -473,8 +462,8 @@ bool NTriangulation::twoZeroMove(NEdge* e, bool check, bool perform) {
     }
 
     // Finally remove and dispose of the tetrahedra.
-    delete removeTetrahedron(tet[0]);
-    delete removeTetrahedron(tet[1]);
+    removeTetrahedron(tet[0]);
+    removeTetrahedron(tet[1]);
 
     // Tidy up.
     // Properties have already been cleared in removeTetrahedron().
@@ -530,7 +519,7 @@ bool NTriangulation::twoZeroMove(NVertex* v, bool check, bool perform) {
     #endif
 
     // Actually perform the move.
-    ChangeEventBlock block(this);
+    ChangeEventSpan span(this);
 
     // Unglue faces from the doomed tetrahedra and glue them to each
     // other.
@@ -542,13 +531,13 @@ bool NTriangulation::twoZeroMove(NVertex* v, bool check, bool perform) {
     } else if (! bottom) {
         tet[0]->unjoin(vertex[0]);
     } else {
-        NPerm crossover;
+        NPerm4 crossover;
         if (vertex[0] == 0)
             crossover = tet[0]->adjacentGluing(1);
         else
             crossover = tet[0]->adjacentGluing(0);
         int topFace = tet[0]->adjacentFace(vertex[0]);
-        NPerm gluing = tet[1]->adjacentGluing(vertex[1]) *
+        NPerm4 gluing = tet[1]->adjacentGluing(vertex[1]) *
             crossover * top->adjacentGluing(topFace);
         tet[0]->unjoin(vertex[0]);
         tet[1]->unjoin(vertex[1]);
@@ -556,8 +545,8 @@ bool NTriangulation::twoZeroMove(NVertex* v, bool check, bool perform) {
     }
 
     // Finally remove and dispose of the tetrahedra.
-    delete removeTetrahedron(tet[0]);
-    delete removeTetrahedron(tet[1]);
+    removeTetrahedron(tet[0]);
+    removeTetrahedron(tet[1]);
 
     // Tidy up.
     // Properties have already been cleared in removeTetrahedron().
@@ -576,22 +565,18 @@ bool NTriangulation::twoOneMove(NEdge* e, int edgeEnd,
 
     const NEdgeEmbedding& emb = e->getEmbeddings().front();
     NTetrahedron* oldTet = emb.getTetrahedron();
-    NPerm oldVertices = emb.getVertices();
+    NPerm4 oldVertices = emb.getVertices();
 
     NTetrahedron* top = oldTet->adjacentTetrahedron(oldVertices[edgeEnd]);
     int otherEdgeEnd = 1 - edgeEnd;
 
-    if (check) {
+    if (check)
         if (! top)
             return false;
-        if (oldTet->getVertex(oldVertices[edgeEnd])->isBoundary() &&
-                oldTet->getVertex(oldVertices[otherEdgeEnd])->isBoundary())
-            return false;
-    }
 
     NFace* centreFace = oldTet->getFace(oldVertices[edgeEnd]);
     NFace* bottomFace = oldTet->getFace(oldVertices[otherEdgeEnd]);
-    NPerm bottomToTop =
+    NPerm4 bottomToTop =
         oldTet->adjacentGluing(oldVertices[edgeEnd]);
     int topGlued[2];
     NEdge* flatEdge[2];
@@ -623,7 +608,7 @@ bool NTriangulation::twoOneMove(NEdge* e, int edgeEnd,
     #endif
 
     // Go ahead and perform the move.
-    ChangeEventBlock block(this);
+    ChangeEventSpan span(this);
 
     // First glue together the two faces that will be flattened.
     NTetrahedron* adjTet[2];
@@ -639,8 +624,8 @@ bool NTriangulation::twoOneMove(NEdge* e, int edgeEnd,
         adjFace[0] = top->adjacentFace(topGlued[0]);
         adjFace[1] = top->adjacentFace(topGlued[1]);
 
-        NPerm gluing = top->adjacentGluing(topGlued[1])
-            * NPerm(topGlued[0], topGlued[1])
+        NPerm4 gluing = top->adjacentGluing(topGlued[1])
+            * NPerm4(topGlued[0], topGlued[1])
             * adjTet[0]->adjacentGluing(adjFace[0]);
         top->unjoin(topGlued[0]);
         top->unjoin(topGlued[1]);
@@ -648,19 +633,18 @@ bool NTriangulation::twoOneMove(NEdge* e, int edgeEnd,
     }
 
     // Now make the new tetrahedron and glue it to itself.
-    NTetrahedron* newTet = new NTetrahedron();
-    addTetrahedron(newTet);
-    newTet->joinTo(2, newTet, NPerm(2,3));
+    NTetrahedron* newTet = newTetrahedron();
+    newTet->joinTo(2, newTet, NPerm4(2,3));
 
     // Glue the new tetrahedron into the remaining structure.
     if (oldTet->adjacentTetrahedron(oldVertices[otherEdgeEnd]) == top) {
         // The top of the new tetrahedron must be glued to the bottom.
         int topFace = bottomToTop[oldVertices[otherEdgeEnd]];
-        NPerm bottomFacePerm = NPerm(oldVertices[edgeEnd],
+        NPerm4 bottomFacePerm = NPerm4(oldVertices[edgeEnd],
             oldVertices[otherEdgeEnd], oldVertices[2], oldVertices[3]);
-        NPerm gluing = bottomFacePerm.inverse() *
+        NPerm4 gluing = bottomFacePerm.inverse() *
             top->adjacentGluing(topFace) * bottomToTop *
-            bottomFacePerm * NPerm(0,1);
+            bottomFacePerm * NPerm4(0,1);
         top->unjoin(topFace);
         newTet->joinTo(0, newTet, gluing);
     } else {
@@ -669,17 +653,17 @@ bool NTriangulation::twoOneMove(NEdge* e, int edgeEnd,
         NTetrahedron* adjTop = top->adjacentTetrahedron(topFace);
         NTetrahedron* adjBottom = oldTet->adjacentTetrahedron(bottomFace);
 
-        NPerm bottomFacePerm = NPerm(oldVertices[edgeEnd],
+        NPerm4 bottomFacePerm = NPerm4(oldVertices[edgeEnd],
             oldVertices[otherEdgeEnd], oldVertices[2], oldVertices[3]);
 
         if (adjTop) {
-            NPerm topGluing = top->adjacentGluing(topFace) *
-                bottomToTop * bottomFacePerm * NPerm(0,1);
+            NPerm4 topGluing = top->adjacentGluing(topFace) *
+                bottomToTop * bottomFacePerm * NPerm4(0,1);
             top->unjoin(topFace);
             newTet->joinTo(0, adjTop, topGluing);
         }
         if (adjBottom) {
-            NPerm bottomGluing = oldTet->adjacentGluing(bottomFace) *
+            NPerm4 bottomGluing = oldTet->adjacentGluing(bottomFace) *
                 bottomFacePerm;
             oldTet->unjoin(bottomFace);
             newTet->joinTo(1, adjBottom, bottomGluing);
@@ -687,8 +671,8 @@ bool NTriangulation::twoOneMove(NEdge* e, int edgeEnd,
     }
 
     // Finally remove and dispose of the unwanted tetrahedra.
-    delete removeTetrahedron(oldTet);
-    delete removeTetrahedron(top);
+    removeTetrahedron(oldTet);
+    removeTetrahedron(top);
 
     // Tidy up.
     // Properties have already been cleared in removeTetrahedron().
@@ -698,7 +682,7 @@ bool NTriangulation::twoOneMove(NEdge* e, int edgeEnd,
 bool NTriangulation::openBook(NFace* f, bool check, bool perform) {
     const NFaceEmbedding& emb = f->getEmbedding(0);
     NTetrahedron* tet = emb.getTetrahedron();
-    NPerm vertices = emb.getVertices();
+    NPerm4 vertices = emb.getVertices();
 
     // Check that the face has exactly two boundary edges.
     // Note that this will imply that the face joins two tetrahedra.
@@ -739,7 +723,6 @@ bool NTriangulation::openBook(NFace* f, bool check, bool perform) {
     // Actually perform the move.
     // Don't bother with a block since this is so simple.
     tet->unjoin(emb.getFace());
-    gluingsHaveChanged();
     return true;
 }
 
@@ -755,8 +738,8 @@ bool NTriangulation::closeBook(NEdge* e, bool check, bool perform) {
 
     NTetrahedron* t0 = front.getTetrahedron();
     NTetrahedron* t1 = back.getTetrahedron();
-    NPerm p0 = front.getVertices();
-    NPerm p1 = back.getVertices();
+    NPerm4 p0 = front.getVertices();
+    NPerm4 p1 = back.getVertices();
 
     if (check) {
         if (t0->getFace(p0[3]) == t1->getFace(p1[2]))
@@ -772,12 +755,8 @@ bool NTriangulation::closeBook(NEdge* e, bool check, bool perform) {
         NEdge* f1 = t1->getEdge(NEdge::edgeNumber[p1[0]][p1[3]]);
         NEdge* f2 = t1->getEdge(NEdge::edgeNumber[p1[1]][p1[3]]);
 
-        if (e1 == f1 || e2 == f2)
-            return false;
         if (e1 == e2 && f1 == f2)
             return false;
-        if (e1 == f2 && f1 == e2)
-            return false;
     }
 
     if (! perform)
@@ -790,8 +769,7 @@ bool NTriangulation::closeBook(NEdge* e, bool check, bool perform) {
     // Actually perform the move.
     // Don't bother with a block since this is so simple.
 
-    t0->joinTo(p0[3], t1, p1 * NPerm(2, 3) * p0.inverse());
-    gluingsHaveChanged();
+    t0->joinTo(p0[3], t1, p1 * NPerm4(2, 3) * p0.inverse());
     return true;
 }
 
@@ -859,9 +837,13 @@ bool NTriangulation::collapseEdge(NEdge* e, bool check, bool perform) {
 
     std::deque<NEdgeEmbedding>::const_iterator it;
     NTetrahedron* tet = 0;
-    NPerm p;
+    NPerm4 p;
 
     if (check) {
+        // Note: We never check whether the edge is valid, but this
+        // comes automatically from the other tests.  In particular, an
+        // invalid edge must join the same vertex to itself.
+
         // CHECK 0: The tetrahedra around the edge must be distinct.
         // We check this as follows:
         //
@@ -1071,31 +1053,39 @@ bool NTriangulation::collapseEdge(NEdge* e, bool check, bool perform) {
     #endif
 
     // Perform the move.
-    ChangeEventBlock block(this);
-    NPerm topPerm, botPerm;
+    ChangeEventSpan span(this);
+    NPerm4 topPerm, botPerm;
     NTetrahedron *top, *bot;
 
     // Clone the edge embeddings because we cannot rely on skeletal
     // objects once we start changing the triangulation.
-    std::deque<NEdgeEmbedding> embClones(embs);
+    unsigned degree = embs.size();
+    NTetrahedron** embTet = new NTetrahedron*[degree];
+    NPerm4* embVertices = new NPerm4[degree];
 
-    for (it = embClones.begin(); it != embClones.end(); it++) {
-        tet = (*it).getTetrahedron();
-        p = (*it).getVertices();
-
-        top = tet->adjacentTetrahedron(p[0]);
-        topPerm = tet->adjacentGluing(p[0]);
-        bot = tet->adjacentTetrahedron(p[1]);
-        botPerm = tet->adjacentGluing(p[1]);
-
-        tet->isolate();
-        if (top && bot)
-            top->joinTo(topPerm[p[0]], bot,
-                botPerm * NPerm(p[0], p[1]) * topPerm.inverse());
-
-        delete removeTetrahedron(tet);
+    unsigned i;
+    for (i = 0, it = embs.begin(); it != embs.end(); ++i, ++it) {
+        embTet[i] = (*it).getTetrahedron();
+        embVertices[i] = (*it).getVertices();
     }
 
+    for (i = 0; i < degree; ++i) {
+        top = embTet[i]->adjacentTetrahedron(embVertices[i][0]);
+        topPerm = embTet[i]->adjacentGluing(embVertices[i][0]);
+        bot = embTet[i]->adjacentTetrahedron(embVertices[i][1]);
+        botPerm = embTet[i]->adjacentGluing(embVertices[i][1]);
+
+        embTet[i]->isolate();
+        if (top && bot)
+            top->joinTo(topPerm[embVertices[i][0]], bot,
+                botPerm * NPerm4(embVertices[i][0], embVertices[i][1]) *
+                topPerm.inverse());
+
+        removeTetrahedron(embTet[i]);
+    }
+
+    delete[] embVertices;
+    delete[] embTet;
     return true;
 }
 
@@ -1104,7 +1094,7 @@ void NTriangulation::reorderTetrahedraBFS(bool reverse) {
     if (n == 0)
         return;
 
-    ChangeEventBlock block(this);
+    ChangeEventSpan span(this);
 
     // Run a breadth-first search over all tetrahedra.
     NTetrahedron** ordered = new NTetrahedron*[n];
@@ -1147,10 +1137,10 @@ void NTriangulation::reorderTetrahedraBFS(bool reverse) {
 
     if (reverse) {
         for (i = n; i > 0; )
-            addTetrahedron(ordered[--i]);
+            tetrahedra.push_back(ordered[--i]);
     } else {
         for (i = 0; i < n; )
-            addTetrahedron(ordered[i++]);
+            tetrahedra.push_back(ordered[i++]);
     }
 
     delete[] used;
