@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file boostutils.h
+/*! \file utilities/boostutils.h
  *  \brief Miscellaneous utility classes taken or modified from the
  *  Boost C++ libraries.
  *
@@ -68,6 +68,8 @@
 #define __BOOSTUTILS_H
 #endif
 
+#include "regina-core.h"
+
 namespace regina {
 
 /**
@@ -275,7 +277,7 @@ inline T next(T it) {
  * @author This class was taken and modified from the Boost C++ libraries
  * (<tt>http://www.boost.org/</tt>).
  */
-class noncopyable {
+class REGINA_API noncopyable {
     protected:
         /**
          * A constructor which does nothing.
