@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nbitmask.h
+/*! \file utilities/nbitmask.h
  *  \brief Provides optimised bitmasks of arbitrary length.
  */
 
@@ -38,6 +38,8 @@
 #include <algorithm>
 #include <iostream>
 
+#include "regina-core.h"
+#include "regina-config.h"
 #include "utilities/bitmanip.h"
 
 namespace regina {
@@ -77,9 +79,11 @@ namespace regina {
  * operations (such as subset testing, bit counting and so on).  Be
  * careful!
  *
+ * \testpart
+ *
  * \ifacespython Not present.
  */
-class NBitmask {
+class REGINA_API NBitmask {
     private:
         typedef unsigned Piece;
             /**< The types of the machine-native pieces into which this
@@ -213,6 +217,11 @@ class NBitmask {
 
         /**
          * Sets all bits of this bitmask to \c false.
+         *
+         * \warning The length of this bitmask must already have been
+         * initialised.  In particular, if the default constructor was
+         * used, you must call the one-argument reset(unsigned) before you
+         * can use this routine.
          */
         void reset();
 
@@ -228,6 +237,35 @@ class NBitmask {
          */
         void reset(unsigned length);
 
+        /**
+         * Sets this bitmask to a copy of the given bitmask.
+         *
+         * \warning The length of this bitmask must already have been
+         * initialised.  In particular, if the default constructor was
+         * used, you must still call reset(unsigned) before you can
+         * assign a value with this routine.
+         *
+         * \pre This and the given bitmask have the same length.
+         *
+         * @param other the bitmask to clone.
+         * @return a reference to this bitmask.
+         */
+        NBitmask& operator = (const NBitmask& other);
+
+        /**
+         * Leaves the first \a numBits bits of this bitmask intact, but
+         * sets all subsequent bits to \c false.  In other words, this
+         * routine "truncates" this bitmask to the given number of bits.
+         *
+         * This routine does not change the \e length of this bitmask
+         * (as passed to the contructor or to reset()).
+         *
+         * \pre \a numBits is at most the length of this bitmask.
+         *
+         * @param numBits the number of bits that will \e not be cleared.
+         */
+        void truncate(unsigned numBits);
+
         /**
          * Sets this to the intersection of this and the given bitmask.
          * Every bit that is unset in \a other will be unset in this bitmask.
@@ -250,6 +288,29 @@ class NBitmask {
          */
         NBitmask& operator |= (const NBitmask& other);
 
+        /**
+         * Sets this to the exclusive disjunction (XOR) of this and the
+         * given bitmask.  Every bit that is set in \a other will be
+         * flipped in this bitmask.
+         *
+         * \pre This and the given bitmask have the same length.
+         *
+         * @param other the bitmask to XOR with this.
+         * @return a reference to this bitmask.
+         */
+        NBitmask& operator ^= (const NBitmask& other);
+
+        /**
+         * Sets this to the set difference of this and the given bitmask.
+         * Every bit that is set in \a other will be cleared in this bitmask.
+         *
+         * \pre This and the given bitmask have the same length.
+         *
+         * @param other the bitmask to XOR with this.
+         * @return a reference to this bitmask.
+         */
+        NBitmask& operator -= (const NBitmask& other);
+
         /**
          * Negates every bit in this bitmask.  All \c true bits will be
          * set to \c false and vice versa.
@@ -329,6 +390,22 @@ class NBitmask {
          */
         unsigned bits() const;
 
+        /**
+         * Returns the index of the first \c true bit in this bitmask,
+         * or -1 if there are no \c true bits.
+         *
+         * @return the index of the first \c true bit.
+         */
+        int firstBit() const;
+
+        /**
+         * Returns the index of the last \c true bit in this bitmask,
+         * or -1 if there are no \c true bits.
+         *
+         * @return the index of the last \c true bit.
+         */
+        int lastBit() const;
+
         /**
          * Determines whether at most one bit is set to \c true in this
          * bitmask.
@@ -341,12 +418,6 @@ class NBitmask {
          */
         bool atMostOneBit() const;
 
-    private:
-        /**
-         * Disable the assignment operator by making it private.
-         */
-        NBitmask& operator = (const NBitmask&);
-
     friend std::ostream& operator << (std::ostream& out, const NBitmask& mask);
 };
 
@@ -365,7 +436,7 @@ class NBitmask {
  * @param mask the bitmask to write.
  * @return a reference to the given output stream.
  */
-std::ostream& operator << (std::ostream& out, const NBitmask& mask);
+REGINA_API std::ostream& operator << (std::ostream& out, const NBitmask& mask);
 
 /**
  * A small but extremely fast bitmask class that can store up to
@@ -386,6 +457,8 @@ std::ostream& operator << (std::ostream& out, const NBitmask& mask);
  *
  * \pre Type \a T is an unsigned integral numeric type.
  *
+ * \testpart
+ *
  * \ifacespython Not present.
  */
 template <typename T>
@@ -406,6 +479,9 @@ class NBitmask1 {
          *
          * The integer argument is merely for compatibility with
          * the NBitmask constructor, and will be ignored.
+         *
+         * \warning This is \e not a constructor that initialises the
+         * bitmask to a given pattern.
          */
         inline NBitmask1(unsigned) : mask(0) {
         }
@@ -435,6 +511,32 @@ class NBitmask1 {
             mask = 0;
         }
 
+        /**
+         * Sets this bitmask to a copy of the given bitmask.
+         *
+         * @param other the bitmask to clone.
+         * @return a reference to this bitmask.
+         */
+        NBitmask1<T>& operator = (const NBitmask1<T>& other) {
+            mask = other.mask;
+            return *this;
+        }
+
+        /**
+         * Leaves the first \a numBits bits of this bitmask intact, but
+         * sets all subsequent bits to \c false.  In other words, this
+         * routine "truncates" this bitmask to the given number of bits.
+         *
+         * This routine does not change the \e length of this bitmask
+         * (as passed to the contructor or to reset()).
+         *
+         * @param numBits the number of bits that will \e not be cleared.
+         */
+        inline void truncate(unsigned numBits) {
+            if (numBits < 8 * sizeof(T))
+                mask &= ((T(1) << numBits) - T(1));
+        }
+
         /**
          * Returns the value of the given bit of this bitmask.
          *
@@ -535,6 +637,32 @@ class NBitmask1 {
             return *this;
         }
 
+        /**
+         * Sets this to the exclusive disjunction (XOR) of this and the
+         * given bitmask.  Every bit that is set in \a other will be
+         * flipped in this bitmask.
+         *
+         * @param other the bitmask to XOR with this.
+         * @return a reference to this bitmask.
+         */
+        inline NBitmask1<T>& operator ^= (const NBitmask1<T>& other) {
+            mask ^= other.mask;
+            return *this;
+        }
+
+        /**
+         * Sets this to the set difference of this and the given bitmask.
+         * Every bit that is set in \a other will be cleared in this bitmask.
+         *
+         * @param other the bitmask to XOR with this.
+         * @return a reference to this bitmask.
+         */
+        inline NBitmask1<T>& operator -= (const NBitmask1<T>& other) {
+            mask |= other.mask;
+            mask ^= other.mask;
+            return *this;
+        }
+
         /**
          * Negates every bit in this bitmask.  All \c true bits will be
          * set to \c false and vice versa.
@@ -617,6 +745,26 @@ class NBitmask1 {
             return BitManipulator<T>::bits(mask);
         }
 
+        /**
+         * Returns the index of the first \c true bit in this bitmask,
+         * or -1 if there are no \c true bits.
+         *
+         * @return the index of the first \c true bit.
+         */
+        inline int firstBit() const {
+            return BitManipulator<T>::firstBit(mask);
+        }
+
+        /**
+         * Returns the index of the last \c true bit in this bitmask,
+         * or -1 if there are no \c true bits.
+         *
+         * @return the index of the last \c true bit.
+         */
+        inline int lastBit() const {
+            return BitManipulator<T>::lastBit(mask);
+        }
+
         /**
          * Determines whether at most one bit is set to \c true in this
          * bitmask.
@@ -631,12 +779,6 @@ class NBitmask1 {
             return BitManipulator<T>::bits(mask) <= 1;
         }
 
-    private:
-        /**
-         * Disable the assignment operator by making it private.
-         */
-        NBitmask1<T>& operator = (const NBitmask1<T>&);
-
     template <typename X>
     friend std::ostream& operator << (std::ostream& out,
         const NBitmask1<X>& mask);
@@ -682,6 +824,8 @@ std::ostream& operator << (std::ostream& out, const NBitmask1<T>& mask) {
  *
  * \pre Types \a T and \a U are unsigned integral numeric types.
  *
+ * \testpart
+ *
  * \ifacespython Not present.
  */
 template <typename T, typename U = T>
@@ -704,6 +848,9 @@ class NBitmask2 {
          *
          * The integer argument is merely for compatibility with
          * the NBitmask constructor, and will be ignored.
+         *
+         * \warning This is \e not a constructor that initialises the
+         * bitmask to a given pattern.
          */
         inline NBitmask2(unsigned) : low(0), high(0) {
         }
@@ -713,7 +860,7 @@ class NBitmask2 {
          *
          * @param cloneMe the bitmask to clone.
          */
-        inline NBitmask2(const NBitmask2<T>& cloneMe) :
+        inline NBitmask2(const NBitmask2<T, U>& cloneMe) :
                 low(cloneMe.low), high(cloneMe.high) {
         }
 
@@ -736,6 +883,39 @@ class NBitmask2 {
             high = 0;
         }
 
+        /**
+         * Sets this bitmask to a copy of the given bitmask.
+         *
+         * @param other the bitmask to clone.
+         * @return a reference to this bitmask.
+         */
+        NBitmask2<T, U>& operator = (const NBitmask2<T, U>& other) {
+            low = other.low;
+            high = other.high;
+            return *this;
+        }
+
+        /**
+         * Leaves the first \a numBits bits of this bitmask intact, but
+         * sets all subsequent bits to \c false.  In other words, this
+         * routine "truncates" this bitmask to the given number of bits.
+         *
+         * This routine does not change the \e length of this bitmask
+         * (as passed to the contructor or to reset()).
+         *
+         * @param numBits the number of bits that will \e not be cleared.
+         */
+        inline void truncate(unsigned numBits) {
+            if (numBits < 8 * sizeof(T)) {
+                low &= ((T(1) << numBits) - T(1));
+                high = 0;
+            } else {
+                numBits -= 8 * sizeof(T);
+                if (numBits < 8 * sizeof(U))
+                    high &= ((U(1) << numBits) - U(1));
+            }
+        }
+
         /**
          * Returns the value of the given bit of this bitmask.
          *
@@ -856,6 +1036,35 @@ class NBitmask2 {
             return *this;
         }
 
+        /**
+         * Sets this to the exclusive disjunction (XOR) of this and the
+         * given bitmask.  Every bit that is set in \a other will be
+         * flipped in this bitmask.
+         *
+         * @param other the bitmask to XOR with this.
+         * @return a reference to this bitmask.
+         */
+        inline NBitmask2<T, U>& operator ^= (const NBitmask2<T, U>& other) {
+            low ^= other.low;
+            high ^= other.high;
+            return *this;
+        }
+
+        /**
+         * Sets this to the set difference of this and the given bitmask.
+         * Every bit that is set in \a other will be cleared in this bitmask.
+         *
+         * @param other the bitmask to XOR with this.
+         * @return a reference to this bitmask.
+         */
+        inline NBitmask2<T, U>& operator -= (const NBitmask2<T, U>& other) {
+            low |= other.low;
+            low ^= other.low;
+            high |= other.high;
+            high ^= other.high;
+            return *this;
+        }
+
         /**
          * Negates every bit in this bitmask.  All \c true bits will be
          * set to \c false and vice versa.
@@ -942,6 +1151,37 @@ class NBitmask2 {
             return BitManipulator<T>::bits(low) + BitManipulator<U>::bits(high);
         }
 
+        /**
+         * Returns the index of the first \c true bit in this bitmask,
+         * or -1 if there are no \c true bits.
+         *
+         * @return the index of the first \c true bit.
+         */
+        inline int firstBit() const {
+            // -1 case does not work out of the box in the second IF branch
+            // due to the 8 * sizeof(T).
+            if (low)
+                return BitManipulator<T>::firstBit(low);
+            else if (high)
+                return 8 * sizeof(T) + BitManipulator<U>::firstBit(high);
+            else
+                return -1;
+        }
+
+        /**
+         * Returns the index of the last \c true bit in this bitmask,
+         * or -1 if there are no \c true bits.
+         *
+         * @return the index of the last \c true bit.
+         */
+        inline int lastBit() const {
+            // -1 case works out of the box in the second IF branch.
+            if (high)
+                return 8 * sizeof(T) + BitManipulator<U>::lastBit(high);
+            else
+                return BitManipulator<T>::lastBit(low);
+        }
+
         /**
          * Determines whether at most one bit is set to \c true in this
          * bitmask.
@@ -957,12 +1197,6 @@ class NBitmask2 {
                 BitManipulator<U>::bits(high)) <= 1;
         }
 
-    private:
-        /**
-         * Disable the assignment operator by making it private.
-         */
-        NBitmask2<T, U>& operator = (const NBitmask2<T, U>&);
-
     template <typename X, typename Y>
     friend std::ostream& operator << (std::ostream& out,
         const NBitmask2<X, Y>& mask);
@@ -990,6 +1224,126 @@ std::ostream& operator << (std::ostream& out, const NBitmask2<T, U>& mask) {
     return out;
 }
 
+#ifndef __DOXYGEN
+/**
+ * An internal template that helps choose the correct bitmask type for
+ * a given (hard-coded) number of bits.
+ *
+ * Please do not use this class directly, since this template is internal
+ * and subject to change in future versions of Regina.  Instead please
+ * use the convenience typedefs NBitmaskLen8, NBitmaskLen16, NBitmaskLen32
+ * and NBitmaskLen64.
+ *
+ * The reason this template exists is to circumvent the fact that we cannot
+ * use sizeof() in a #if statement.  The boolean argument to this template
+ * should always be left as the default.
+ */
+template <bool IntHolds4Bytes = (sizeof(unsigned int) >= 4)>
+struct InternalBitmaskLen32;
+
+template <>
+struct InternalBitmaskLen32<true> {
+    typedef NBitmask1<unsigned int> Type;
+};
+
+template <>
+struct InternalBitmaskLen32<false> {
+    // The standard guarantees that sizeof(long) >= 4.
+    typedef NBitmask1<unsigned long> Type;
+};
+
+/**
+ * An internal template that helps choose the correct bitmask type for
+ * a given (hard-coded) number of bits.
+ *
+ * Please do not use this class directly, since this template is internal
+ * and subject to change in future versions of Regina.  Instead please
+ * use the convenience typedefs NBitmaskLen8, NBitmaskLen16, NBitmaskLen32
+ * and NBitmaskLen64.
+ *
+ * The reason this template exists is to circumvent the fact that we cannot
+ * use sizeof() in a #if statement.  The boolean argument to this template
+ * should always be left as the default.
+ */
+template <bool LongHolds8Bytes = (sizeof(unsigned long) >= 8)>
+struct InternalBitmaskLen64;
+
+template <>
+struct InternalBitmaskLen64<true> {
+    typedef NBitmask1<unsigned long> Type;
+};
+
+template <>
+struct InternalBitmaskLen64<false> {
+#ifdef LONG_LONG_FOUND
+    // The C standard guarantees that sizeof(long long) >= 8.
+    // However, the C++ standard does not require long long to exist at
+    // all (hence the LONG_LONG_FOUND test).
+    typedef NBitmask1<unsigned long long> Type;
+#else
+    // The standard guarantees that sizeof(long) >= 4.
+    // Therefore two longs will be enough for 64 bits.
+    typedef NBitmask2<unsigned long> Type;
+#endif
+};
+#endif // End block for doxygen to ignore.
+
+/**
+ * A convenience typedef that gives a small and extremely fast bitmask
+ * class capable of holding at least 8 true-or-false bits.
+ *
+ * This bitmask class is guaranteed to be an instantiation of the
+ * template class NBitmask1.
+ *
+ * The particular instantiation is subject to change between different
+ * platforms, different compilers and/or different versions of Regina.
+ *
+ * \ifacespython Not present.
+ */
+typedef NBitmask1<unsigned char> NBitmaskLen8;
+
+/**
+ * A convenience typedef that gives a small and extremely fast bitmask
+ * class capable of holding at least 16 true-or-false bits.
+ *
+ * This bitmask class is guaranteed to be an instantiation of the
+ * template class NBitmask1.
+ *
+ * The particular instantiation is subject to change between different
+ * platforms, different compilers and/or different versions of Regina.
+ *
+ * \ifacespython Not present.
+ */
+typedef NBitmask1<unsigned int> NBitmaskLen16;
+
+/**
+ * A convenience typedef that gives a small and extremely fast bitmask
+ * class capable of holding at least 32 true-or-false bits.
+ *
+ * This bitmask class is guaranteed to be an instantiation of the
+ * template class NBitmask1.
+ *
+ * The particular instantiation is subject to change between different
+ * platforms, different compilers and/or different versions of Regina.
+ *
+ * \ifacespython Not present.
+ */
+typedef InternalBitmaskLen32<>::Type NBitmaskLen32;
+
+/**
+ * A convenience typedef that gives a small and extremely fast bitmask
+ * class capable of holding at least 64 true-or-false bits.
+ *
+ * This bitmask class is guaranteed to be an instantiation of
+ * \e either the template class NBitmask1 or the template class NBitmask2.
+ *
+ * The particular instantiation is subject to change between different
+ * platforms, different compilers and/or different versions of Regina.
+ *
+ * \ifacespython Not present.
+ */
+typedef InternalBitmaskLen64<>::Type NBitmaskLen64;
+
 /*@}*/
 
 // Inline functions for NBitmask
@@ -1027,6 +1381,23 @@ inline void NBitmask::reset(unsigned length) {
     std::fill(mask, mask + pieces, 0);
 }
 
+inline NBitmask& NBitmask::operator = (const NBitmask& other) {
+    std::copy(other.mask, other.mask + pieces, mask);
+    return *this;
+}
+
+inline void NBitmask::truncate(unsigned numBits) {
+    unsigned skip = numBits / (8 * sizeof(Piece));
+    numBits = numBits % (8 * sizeof(Piece));
+
+    Piece* piece = mask + skip;
+    if (piece < mask + pieces) {
+        (*piece) &= ((Piece(1) << numBits) - Piece(1));
+        for (++piece; piece < mask + pieces; ++piece)
+            *piece = 0;
+    }
+}
+
 inline bool NBitmask::get(unsigned index) const {
     return (mask[index / (8 * sizeof(Piece))] &
         (Piece(1) << (index % (8 * sizeof(Piece)))));
@@ -1052,6 +1423,20 @@ inline NBitmask& NBitmask::operator |= (const NBitmask& other) {
     return *this;
 }
 
+inline NBitmask& NBitmask::operator ^= (const NBitmask& other) {
+    for (unsigned i = 0; i < pieces; ++i)
+        mask[i] ^= other.mask[i];
+    return *this;
+}
+
+inline NBitmask& NBitmask::operator -= (const NBitmask& other) {
+    for (unsigned i = 0; i < pieces; ++i) {
+        mask[i] |= other.mask[i];
+        mask[i] ^= other.mask[i];
+    }
+    return *this;
+}
+
 inline void NBitmask::flip() {
     for (unsigned i = 0; i < pieces; ++i)
         mask[i] = ~mask[i];
@@ -1089,6 +1474,22 @@ inline unsigned NBitmask::bits() const {
     return ans;
 }
 
+inline int NBitmask::firstBit() const {
+    for (unsigned i = 0; i < pieces; ++i)
+        if (mask[i])
+            return 8 * sizeof(Piece) * i +
+                BitManipulator<Piece>::firstBit(mask[i]);
+    return -1;
+}
+
+inline int NBitmask::lastBit() const {
+    for (int i = pieces - 1; i >= 0; --i)
+        if (mask[i])
+            return 8 * sizeof(Piece) * i +
+                BitManipulator<Piece>::lastBit(mask[i]);
+    return -1;
+}
+
 inline bool NBitmask::atMostOneBit() const {
     unsigned bits = 0;
     for (unsigned i = 0; i < pieces; ++i) {
