@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nbooleans.h
+/*! \file utilities/nbooleans.h
  *  \brief Provides various types that extend the standard boolean.
  */
 
@@ -36,6 +36,7 @@
 #endif
 
 #include <iostream>
+#include "regina-core.h"
 
 namespace regina {
 
@@ -48,7 +49,7 @@ namespace regina {
  * A three-way extension of the boolean type.
  * Three-way booleans can be true, false or unknown.
  */
-class NTriBool {
+class REGINA_API NTriBool {
     private:
         enum Code { codeTrue = 1, codeFalse = -1, codeUnknown = 0 };
             /**< Internal codes for the three possible values. */
@@ -357,13 +358,13 @@ class NTriBool {
  * @param set the three-way boolean to write.
  * @return a reference to \a out.
  */
-std::ostream& operator << (std::ostream& out, const NTriBool& set);
+REGINA_API std::ostream& operator << (std::ostream& out, const NTriBool& set);
 
 /**
  * An exception thrown when an illegal integer conversion is attempted
  * with NTriBool.
  */
-class NTriBool_Illegal_Integer_Conversion {
+class REGINA_API NTriBool_Illegal_Integer_Conversion {
 };
 
 /**
@@ -371,7 +372,7 @@ class NTriBool_Illegal_Integer_Conversion {
  * NBoolSet objects are small enough to pass about by value instead of
  * by reference.
  */
-class NBoolSet {
+class REGINA_API NBoolSet {
     private:
         unsigned char elements;
             /**< The first two bits of this character represent whether
@@ -658,7 +659,7 @@ class NBoolSet {
  * @param set the boolean set to write.
  * @return a reference to \a out.
  */
-std::ostream& operator << (std::ostream& out, const NBoolSet& set);
+REGINA_API std::ostream& operator << (std::ostream& out, const NBoolSet& set);
 
 /*@}*/
 
@@ -836,9 +837,9 @@ inline NBoolSet::NBoolSet(const NBoolSet& cloneMe) :
 }
 inline NBoolSet::NBoolSet(bool insertTrue, bool insertFalse) : elements(0) {
     if (insertTrue)
-        elements |= eltTrue;
+        elements = static_cast<unsigned char>(elements | eltTrue);
     if (insertFalse)
-        elements |= eltFalse;
+        elements = static_cast<unsigned char>(elements | eltFalse);
 }
 
 inline bool NBoolSet::hasTrue() const {
@@ -852,22 +853,22 @@ inline bool NBoolSet::contains(bool value) const {
 }
 
 inline void NBoolSet::insertTrue() {
-    elements |= eltTrue;
+    elements = static_cast<unsigned char>(elements | eltTrue);
 }
 inline void NBoolSet::insertFalse() {
-    elements |= eltFalse;
+    elements = static_cast<unsigned char>(elements | eltFalse);
 }
 inline void NBoolSet::removeTrue() {
-    elements &= eltFalse;
+    elements = static_cast<unsigned char>(elements & eltFalse);
 }
 inline void NBoolSet::removeFalse() {
-    elements &= eltTrue;
+    elements = static_cast<unsigned char>(elements & eltTrue);
 }
 inline void NBoolSet::empty() {
     elements = 0;
 }
 inline void NBoolSet::fill() {
-    elements = eltTrue | eltFalse;
+    elements = static_cast<unsigned char>(eltTrue | eltFalse);
 }
 
 inline bool NBoolSet::operator == (const NBoolSet& other) const {
@@ -900,31 +901,31 @@ inline NBoolSet& NBoolSet::operator = (bool member) {
     return *this;
 }
 inline NBoolSet& NBoolSet::operator |= (const NBoolSet& other) {
-    elements |= other.elements;
+    elements = static_cast<unsigned char>(elements | other.elements);
     return *this;
 }
 inline NBoolSet& NBoolSet::operator &= (const NBoolSet& other) {
-    elements &= other.elements;
+    elements = static_cast<unsigned char>(elements & other.elements);
     return *this;
 }
 inline NBoolSet& NBoolSet::operator ^= (const NBoolSet& other) {
-    elements ^= other.elements;
+    elements = static_cast<unsigned char>(elements ^ other.elements);
     return *this;
 }
 
 inline NBoolSet NBoolSet::operator | (const NBoolSet& other) const {
     NBoolSet ans;
-    ans.elements = elements | other.elements;
+    ans.elements = static_cast<unsigned char>(elements | other.elements);
     return ans;
 }
 inline NBoolSet NBoolSet::operator & (const NBoolSet& other) const {
     NBoolSet ans;
-    ans.elements = elements & other.elements;
+    ans.elements = static_cast<unsigned char>(elements & other.elements);
     return ans;
 }
 inline NBoolSet NBoolSet::operator ^ (const NBoolSet& other) const {
     NBoolSet ans;
-    ans.elements = elements ^ other.elements;
+    ans.elements = static_cast<unsigned char>(elements ^ other.elements);
     return ans;
 }
 inline NBoolSet NBoolSet::operator ~ () const {
