@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file nmarkedvector.h
+/*! \file utilities/nmarkedvector.h
  *  \brief Provides space-efficient arrays with fast object-to-index lookup.
  */
 
@@ -36,6 +36,7 @@
 #endif
 
 #include <vector>
+#include "regina-core.h"
 
 namespace regina {
 
@@ -70,7 +71,7 @@ namespace regina {
  *
  * \ifacespython Not present.
  */
-class NMarkedElement {
+class REGINA_API NMarkedElement {
     private:
         long marking;
             /**< The index in the NMarkedVector at which this object is
@@ -227,6 +228,15 @@ class NMarkedVector : private std::vector<T*> {
                 (*it)->marking -= (first - last);
             return std::vector<T*>::erase(first, last);
         }
+
+        /**
+         * Swaps the contents of this and the given vector.
+         *
+         * @param other the vector whose contents are to be swapped with this.
+         */
+        inline void swap(NMarkedVector<T>& other) {
+            std::vector<T*>::swap(other);
+        }
 };
 
 /*@}*/
