@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file ptrutils.h
+/*! \file utilities/ptrutils.h
  *  \brief Provides function objects for use in the Standard Template
  *  Library that take pointers as arguments but work with the pointees
  *  instead.
@@ -37,6 +37,9 @@
 #define __PTRUTILS_H
 #endif
 
+#include <memory>
+#include "regina-core.h"
+
 namespace regina {
 
 /**
