@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,7 @@
 
 /* end stub */
 
-/*! \file xmlutils.h
+/*! \file utilities/xmlutils.h
  *  \brief Various classes and routines for XML manipulation, some taken
  *  or modified from the libxml++ library.  The libxml2 library is used
  *  to do most of the underlying work.
@@ -40,12 +40,12 @@
 #define __XMLUTILS_H
 #endif
 
+#include <map>
 #include <string>
 #include <sstream>
 #include <libxml/parser.h>
+#include "regina-core.h"
 #include "utilities/nbooleans.h"
-#include "utilities/hashmap.h"
-#include "utilities/hashutils.h"
 
 namespace regina {
 
@@ -66,29 +66,34 @@ namespace xml {
 class XMLParser;
 
 /**
- * Represents a hashed map from property names to property values.
+ * Represents a map from property names to property values.
  *
  * \ifacespython Not present.
  */
-class XMLPropertyDict : public stdhash::hash_map<std::string,
-        std::string, HashString> {
+class REGINA_API XMLPropertyDict : private std::map<std::string, std::string> {
     public:
         /**
-         * Create a new hashed map.
+         * Create a new map.
          */
         XMLPropertyDict();
 
         /**
          * Return a value for the given key, even if the key does not
-         * exist in the hashed map.
+         * exist in the map.
          *
          * @param key the key to look up.
          * @param defaultVal the value to return if the key does not exist.
          * @return the value associated with the given key, or parameter
-         * \a default if the key does not exist in the hashed map.
+         * \a default if the key does not exist in the map.
          */
         const std::string& lookup(const std::string& key,
             const std::string& defaultVal = std::string()) const;
+
+        using std::map<std::string, std::string>::const_iterator;
+        using std::map<std::string, std::string>::begin;
+        using std::map<std::string, std::string>::end;
+        using std::map<std::string, std::string>::find;
+        using std::map<std::string, std::string>::operator [];
 };
 
 /**
@@ -105,7 +110,7 @@ class XMLPropertyDict : public stdhash::hash_map<std::string,
  * @author This class was taken and modified from the libxml++ library
  * (<tt>http://lusis.org/~ari/xml++/</tt>).
  */
-class XMLParserCallback {
+class REGINA_API XMLParserCallback {
     public:
         /**
          * Default destructor that does nothing.
@@ -126,7 +131,7 @@ class XMLParserCallback {
          * Called when an element's opening tag is encountered.
          *
          * @param n the name of the tag.
-         * @param p a hashed dictionary of all the properties of the tag.
+         * @param p a dictionary of all the properties of the tag.
          */
         virtual void start_element(const std::string& n,
             const regina::xml::XMLPropertyDict& p);
@@ -190,7 +195,7 @@ class XMLParserCallback {
  * @author This class was taken and modified from the libxml++ library
  * (<tt>http://lusis.org/~ari/xml++/</tt>).
  */
-class XMLParser {
+class REGINA_API XMLParser {
     private:
         XMLParserCallback& _parser_callback;
             /**< Provides the callback routines to use with this parser. */
@@ -291,7 +296,7 @@ class XMLParser {
  * @return the converted string with special characters replaced by
  * XML entities.
  */
-std::string xmlEncodeSpecialChars(const std::string& original);
+REGINA_API std::string xmlEncodeSpecialChars(const std::string& original);
 
 /**
  * Returns the given string encoded so it is suitable for use inside an
@@ -305,7 +310,7 @@ std::string xmlEncodeSpecialChars(const std::string& original);
  * changed.
  * @return the string converted to be usable inside an XML comment.
  */
-std::string xmlEncodeComment(const std::string& comment);
+REGINA_API std::string xmlEncodeComment(const std::string& comment);
 
 /**
  * Returns an XML tag with a single property containing the given value.
@@ -395,7 +400,7 @@ inline XMLParser::~XMLParser() {
 }
 
 inline void XMLParser::parse_chunk(const std::string& s) {
-    xmlParseChunk(_context, s.c_str(), s.length(), 0);
+    xmlParseChunk(_context, s.c_str(), static_cast<int>(s.length()), 0);
 }
 
 inline void XMLParser::finish() {
