@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -37,7 +37,8 @@
 #include "pythonoutputstream.h"
 #include "../reginaprefset.h"
 
-#include <kmainwindow.h>
+#include <packet/npacket.h>
+#include <kxmlguiwindow.h>
 
 class CommandEdit;
 class KTextEdit;
@@ -52,7 +53,7 @@ class QLabel;
  * Objects of this class are generally not created directly; instead
  * PacketManager::launchPythonConsole() should be used.
  */
-class PythonConsole : public KMainWindow {
+class PythonConsole : public KXmlGuiWindow {
     Q_OBJECT
 
     private:
@@ -156,6 +157,11 @@ class PythonConsole : public KMainWindow {
          */
         void updatePreferences(const ReginaPrefSet& newPrefs);
 
+        /**
+         * Help -> What's this has been acctivated.
+         */
+        void contextHelpActivated();
+
     private:
         /**
          * Encode special characters so that the given text can be
