@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,12 +26,13 @@
 
 /* end stub */
 
+// Put this before any Qt/KDE stuff so Python 2.3 "slots" doesn't clash.
+#include "pythoninterpreter.h"
+
 #include "regina-config.h"
 #include "file/nglobaldirs.h"
 #include "packet/npacket.h"
 
-// Put this before any Qt/KDE stuff so Python 2.3 "slots" doesn't clash.
-#include "pythoninterpreter.h"
 
 #include "pythonoutputstream.h"
 
@@ -178,6 +179,11 @@ bool PythonInterpreter::executeLine(const std::string& command) {
         return true;
     }
 
+    // Clear the error so it doesn't stay with the thread state and
+    // cause wacky syntax errors later on.  Such behaviour has been
+    // observed on macosx/fink.
+    PyErr_Clear();
+
     // Attempt to compile the command with final newline.
     cmdBuffer[fullCommand.length()] = '\n';
     cmdBuffer[fullCommand.length() + 1] = 0;
@@ -194,6 +200,7 @@ bool PythonInterpreter::executeLine(const std::string& command) {
     }
 
     // Extract the full error details in case we wish to display them later.
+    // Note that PyErr_Fetch() has the side-effect of clearing the error also.
     PyObject *errType, *errValue, *errTrace;
     PyErr_Fetch(&errType, &errValue, &errTrace);
 
@@ -218,7 +225,7 @@ bool PythonInterpreter::executeLine(const std::string& command) {
         return false;
     }
 
-    PyObject* errStr2 = extractErrMsg();
+    PyObject* errStr2 = extractErrMsg(); // Also clears the error.
 
     // Compare the two compile errors.
     if (errStr1 && errStr2) {
