@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -34,10 +34,15 @@
 #include "shortrunner.h"
 
 #include <fstream>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <krun.h>
 #include <kstandarddirs.h>
+#include <ktoolinvocation.h>
 #include <qdir.h>
 #include <qfile.h>
 #include <qfileinfo.h>
+#include <qtextcodec.h>
 #include <qtextstream.h>
 
 namespace {
@@ -64,7 +69,7 @@ QString GraphvizStatus::cacheGraphvizExec;
 QString GraphvizStatus::cacheGraphvizExecFull;
 GraphvizStatus GraphvizStatus::cacheGraphvizStatus;
 
-QCString ReginaFilePref::encodeFilename() const {
+QByteArray ReginaFilePref::encodeFilename() const {
     return static_cast<const char*>(QFile::encodeName(filename));
 }
 
@@ -79,13 +84,13 @@ GraphvizStatus GraphvizStatus::status(const QString& userExec,
     }
 
     // We need a full requery.
-    if (userExec.find("/") < 0) {
+    if (! userExec.contains("/")) {
         // Hunt on the search path.
         fullExec = KStandardDirs::findExe(userExec);
         if (fullExec.isNull())
             return notFound;
     } else
-        fullExec = QFileInfo(userExec).absFilePath();
+        fullExec = QFileInfo(userExec).absoluteFilePath();
 
     // We have a full path to the Graphviz executable.
     QFileInfo info(fullExec);
@@ -105,16 +110,16 @@ GraphvizStatus GraphvizStatus::status(const QString& userExec,
             return notStartable;
     }
 
-    if (output.find("version 1.") >= 0) {
+    if (output.contains("version 1.")) {
         // Only test for "dot", not "/dot".  I'd rather not get tripped
         // up with alternate path separators, and this still
         // distinguishes between the different 1.x graph drawing tools.
-        if (userExec.endsWith("dot", false))
+        if (userExec.endsWith("dot", Qt::CaseInsensitive))
             return version1;
         return version1NotDot;
-    } else if (output.find("version 0.") >= 0)
+    } else if (output.contains("version 0."))
         return unsupported;
-    else if (output.find("version") >= 0) {
+    else if (output.contains("version")) {
         // Assume any other version is >= 2.x.
         return version2;
     } else {
@@ -127,8 +132,8 @@ ReginaPrefSet::ReginaPrefSet() :
         autoDock(true),
         autoFileExtension(true),
         censusFiles(defaultCensusFiles()),
-        displayIcon(true),
         displayTagsInTree(false),
+        handbookInKHelpCenter(true),
         pdfAutoClose(true),
         pdfEmbed(true),
         pythonAutoIndent(true),
@@ -140,7 +145,6 @@ ReginaPrefSet::ReginaPrefSet() :
         surfacesInitialCompat(LocalCompat),
         surfacesInitialTab(Summary),
         treeJumpSize(10),
-        triEditMode(DirectEdit),
         triGAPExec(defaultGAPExec),
         triGraphvizExec(defaultGraphvizExec),
         triInitialTab(Gluings),
@@ -172,11 +176,11 @@ QString ReginaPrefSet::pdfDefaultViewer() {
             return app;
     #endif
 
-    // Try KDE applications (kpdf, okular).
-    if (! (app = KStandardDirs::findExe("kpdf")).isNull())
-        return app;
+    // Try KDE applications (okular, kpdf).
     if (! (app = KStandardDirs::findExe("okular")).isNull())
         return app;
+    if (! (app = KStandardDirs::findExe("kpdf")).isNull())
+        return app;
 
     // Try GNOME applications (evince).
     if (! (app = KStandardDirs::findExe("evince")).isNull())
@@ -192,18 +196,18 @@ QString ReginaPrefSet::pdfDefaultViewer() {
 }
 
 QString ReginaPrefSet::pythonLibrariesConfig() {
-    return QDir::homeDirPath() + "/.regina-libs";
+    return QDir::homePath() + "/.regina-libs";
 }
 
 bool ReginaPrefSet::readPythonLibraries() {
     pythonLibraries.clear();
 
     QFile f(pythonLibrariesConfig());
-    if (! f.open(IO_ReadOnly))
+    if (! f.open(QIODevice::ReadOnly))
         return false;
 
     QTextStream in(&f);
-    in.setEncoding(QTextStream::UnicodeUTF8);
+    in.setCodec(QTextCodec::codecForName("UTF-8"));
 
     bool active;
     QString line = in.readLine();
@@ -215,7 +219,7 @@ bool ReginaPrefSet::readPythonLibraries() {
             line = line.mid(INACTIVE.length());
         }
 
-        line = line.stripWhiteSpace();
+        line = line.trimmed();
 
         // Is it a file at all?  If so, add it.
         if ((! line.isEmpty()) && line[0] != '#')
@@ -230,11 +234,11 @@ bool ReginaPrefSet::readPythonLibraries() {
 
 bool ReginaPrefSet::writePythonLibraries() const {
     QFile f(pythonLibrariesConfig());
-    if (! f.open(IO_WriteOnly))
+    if (! f.open(QIODevice::WriteOnly))
         return false;
 
     QTextStream out(&f);
-    out.setEncoding(QTextStream::UnicodeUTF8);
+    out.setCodec(QTextCodec::codecForName("UTF-8"));
 
     out << "# Python libraries configuration file\n#\n";
     out << "# Automatically generated by the KDE user interface.\n\n";
@@ -249,3 +253,52 @@ bool ReginaPrefSet::writePythonLibraries() const {
     return true;
 }
 
+void ReginaPrefSet::openHandbook(const char* section, const char* handbook,
+        QWidget* parentWidget) {
+    if (handbookInKHelpCenter) {
+        KToolInvocation::invokeHelp(section, handbook);
+        return;
+    }
+
+    QString handbookName = (handbook ? handbook : "regina");
+    QString index = KStandardDirs::locate("html",
+        QString("en/%1/index.html").arg(handbookName));
+    QString page = KStandardDirs::locate("html",
+        QString("en/%1/%2.html").arg(handbookName).arg(section));
+    if (QFileInfo(page).exists()) {
+        // If we're on a mac, just use the default Mac browser.
+#ifdef __APPLE__
+        // Hmm.  Assume this command executes successfully, since on my
+        // fink it returns false even when it *does* execute successfully..!
+        KRun::runCommand(QString("open \"%1\"").arg(page), parentWidget);
+#else
+        if (! KRun::runUrl("file:" + page, "text/html", parentWidget,
+                false /* temp file */, false /* run executables */)) {
+            if (handbook)
+                KMessageBox::sorry(parentWidget, i18n(
+                    "<qt>The requested handbook could not be opened.  "
+                    "Please try pointing your web browser to "
+                    "<tt>%1</tt>.</qt>").arg(page));
+            else
+                KMessageBox::sorry(parentWidget, i18n(
+                    "<qt>The Regina handbook could not be opened.  "
+                    "Please try pointing your web browser to "
+                    "<tt>%1</tt>.</qt>").arg(page));
+        }
+#endif
+    } else {
+        if (handbook)
+            KMessageBox::sorry(parentWidget, i18n(
+                "<qt>The requested handbook could "
+                "not be found.  Perhaps it is not installed?<p>"
+                "The handbook should be accessible as "
+                "<tt>%1/</tt>.</qt>").arg(index));
+        else
+            KMessageBox::sorry(parentWidget, i18n(
+                "<qt>The Regina handbook could "
+                "not be found.  Perhaps it is not installed?<p>"
+                "The handbook should be accessible as "
+                "<tt>%1/</tt>.</qt>").arg(index));
+    }
+}
+
