@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -34,7 +34,10 @@
 #define __REGINAPREFSET_H
 
 #include <qmutex.h>
-#include <qvaluevector.h>
+#include <QLinkedList>
+#include <QString>
+
+class QWidget;
 
 /**
  * A structure holding a single filename which may or may not be active
@@ -56,14 +59,14 @@ struct ReginaFilePref {
      * Return the filename in an 8-bit encoding suitable for passing to
      * low-level file I/O routines.
      */
-    QCString encodeFilename() const;
+    QByteArray encodeFilename() const;
 };
 
 /**
  * A structure holding a list of filenames each of which may or may not
  * be active.
  */
-typedef QValueVector<ReginaFilePref> ReginaFilePrefList;
+typedef QLinkedList<ReginaFilePref> ReginaFilePrefList;
 
 /**
  * Describes the many possible ways in which ReginaPrefSet::triGraphvizExec
@@ -210,14 +213,13 @@ struct ReginaPrefSet {
 
     // The preferences themselves:
 
-    enum TriEditMode { DirectEdit, Dialog };
-        /**< Possible edit modes for triangulation gluings. */
     enum TriTab { Gluings, Skeleton, Algebra, Composition, Surfaces, SnapPea };
-        /**< Available top-level tabs in a triangulation viewer/editor. */
+        /**< Available top-level tabs in a 3-manifold triangulation
+             viewer/editor. */
     enum TriSkeletonTab { SkelComp, FacePairingGraph };
-        /**< Available tabs in a triangulation skeleton viewer. */
+        /**< Available tabs in a 3-manifold triangulation skeleton viewer. */
     enum TriAlgebraTab { Homology, FundGroup, TuraevViro, CellularInfo };
-        /**< Available tabs in a triangulation algebra viewer. */
+        /**< Available tabs in a 3-manifold triangulation algebra viewer. */
     enum SurfacesTab { Summary, Coordinates, Matching, Compatibility };
         /**< Available top-level tabs in a normal surface list viewer. */
     enum SurfacesCompatMatrix { LocalCompat, GlobalCompat };
@@ -231,10 +233,10 @@ struct ReginaPrefSet {
         /**< Should filenames be given an automatic extension? */
     ReginaFilePrefList censusFiles;
         /**< The list of data files to use for census lookups. */
-    bool displayIcon;
-        /**< Should we display the pretty Regina icon? */
     bool displayTagsInTree;
         /**< Should we display packet tags in the visual tree? */
+    bool handbookInKHelpCenter;
+        /**< Should we open the users' handbook in the KDE help center? */
     bool pdfAutoClose;
         /**< Should we close external PDF viewers automatically (e.g.,
              when the packet is refreshed or closed)? */
@@ -273,8 +275,6 @@ struct ReginaPrefSet {
     unsigned treeJumpSize;
         /**< The number of steps corresponding to a jump up or down in
              the packet tree. */
-    TriEditMode triEditMode;
-        /**< The default mode for editing triangulations. */
     QString triGAPExec;
         /**< The executable for starting GAP.  This need not include a
              directory (in which case the search path will be used). */
@@ -284,17 +284,17 @@ struct ReginaPrefSet {
              is neato.  This need not include a directory (in which case
              the search path will be used). */
     TriTab triInitialTab;
-        /**< The initially visible top-level tab for a new triangulation
-             viewer/editor. */
+        /**< The initially visible top-level tab for a new 3-manifold
+             triangulation viewer/editor. */
     TriSkeletonTab triInitialSkeletonTab;
-        /**< The initially visible tab for a new triangulation skeleton
-             viewer. */
+        /**< The initially visible tab for a new 3-manifold triangulation
+             skeleton viewer. */
     TriAlgebraTab triInitialAlgebraTab;
-        /**< The initially visible tab for a new triangulation algebra
-             viewer. */
+        /**< The initially visible tab for a new 3-manifold triangulation
+             algebra viewer. */
     unsigned triSurfacePropsThreshold;
         /**< The maximum number of tetrahedra for which surface-related
-             properties of triangulations will be automatically
+             properties of 3-manifold triangulations will be automatically
              calculated. */
 
     /**
@@ -341,6 +341,15 @@ struct ReginaPrefSet {
      * file.
      */
     bool writePythonLibraries() const;
+
+    /**
+     * Opens the given section of an arbitrary handbook in an appropriate
+     * manner.  If the handbook is in fact the users' handbook then
+     * the argument \a handbook should be 0 (which enables specialised
+     * code for the users' handbook only).
+     */
+    void openHandbook(const char* section, const char* handbook,
+        QWidget* parentWidget);
 };
 
 inline GraphvizStatus::GraphvizStatus() : flag_(unknown.flag_) {
