@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -32,49 +32,66 @@
 #include <klocale.h>
 #include <kstdaction.h>
 
+#include <KActionCollection>
+#include <KStandardAction>
+
 void ReginaPart::setupActions() {
     KAction* act;
 
     // File actions:
-    actSave = KStdAction::save(this, SLOT(fileSave()), actionCollection());
-    KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
+    actSave = KStandardAction::save(this, SLOT(fileSave()), actionCollection());
+    actSave->setWhatsThis(i18n("Save the current data file."));
+    act = KStandardAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
+    act->setWhatsThis(i18n(
+        "Save the current data file, but give it a different name."));
 
     // Edit actions:
-    actCut = KStdAction::cut(0, 0, actionCollection());
+    actCut = KStandardAction::cut(actionCollection());
+    actCut->setWhatsThis(i18n("Cut out the current selection and store it "
+        "in the clipboard."));
     actCut->setEnabled(false);
-    actCopy = KStdAction::copy(0, 0, actionCollection());
+    actCopy = KStandardAction::copy(actionCollection());
+    actCopy->setWhatsThis(i18n("Copy the current selection to the clipboard."));
     actCopy->setEnabled(false);
-    actPaste = KStdAction::paste(0, 0, actionCollection());
+    actPaste = KStandardAction::paste(actionCollection());
+    actPaste->setWhatsThis(i18n("Paste the contents of the clipboard."));
     actPaste->setEnabled(false);
-    actUndo = KStdAction::undo(0, 0, actionCollection());
-    actUndo->setEnabled(false);
-    actRedo = KStdAction::redo(0, 0, actionCollection());
-    actRedo->setEnabled(false);
 
     // Basic packet actions:
-    act = new KAction(i18n("&View/Edit"), "viewmag", CTRL+Key_V,
-        this, SLOT(packetView()), actionCollection(), "tree_view");
+    act = actionCollection()->addAction("tree_view");
+    act->setText(i18n("&View/Edit"));
+    act->setIcon(KIcon("packet_view"));
+    act->setShortcut(tr("Alt+v"));
     act->setToolTip(i18n("View or edit the selected packet"));
     act->setWhatsThis(i18n("View or edit the packet currently selected "
         "in the tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(packetView()) );
     treePacketViewActions.append(act);
 
-    act = new KAction(i18n("&Rename"), "editclear", CTRL+Key_R,
-        this, SLOT(packetRename()), actionCollection(), "tree_rename");
+    act = actionCollection()->addAction("tree_rename");
+    act->setText(i18n("&Rename"));
+    act->setIcon(KIcon("edit-rename"));
+    act->setShortcut(tr("Alt+r"));
     act->setToolTip(i18n("Rename the selected packet"));
     act->setWhatsThis(i18n("Rename the packet currently selected "
         "in the tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(packetRename()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("&Delete"), "editdelete", Key_Delete,
-        this, SLOT(packetDelete()), actionCollection(), "tree_delete");
+    act = actionCollection()->addAction("tree_delete");
+    act->setText(i18n("&Delete"));
+    act->setIcon(KIcon("edit-delete"));
+    act->setShortcut(tr("Delete"));
     act->setToolTip(i18n("Delete the selected packet"));
     act->setWhatsThis(i18n("Delete the packet currently selected "
         "in the tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(packetDelete()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("Refres&h Subtree"), "reload", Key_F5,
-        this, SLOT(subtreeRefresh()), actionCollection(), "tree_refresh");
+    act = actionCollection()->addAction("tree_refresh");
+    act->setText(i18n("Refres&h Subtree"));
+    act->setIcon(KIcon("view-refresh"));
+    act->setShortcut(tr("F5"));
     act->setToolTip(i18n("Refresh the subtree beneath the selected packet"));
     act->setWhatsThis(i18n("Refresh the packet "
         "currently selected in the tree, as well as all of its descendants "
@@ -83,241 +100,324 @@ void ReginaPart::setupActions() {
         "fix-up in case the tree is out of sync with what is happening "
         "elsewhere.  Note that the file is <i>not</i> reloaded from "
         "disc; the tree is just resynced with packet editors and so on."));
+    connect(act, SIGNAL(triggered()), this, SLOT(subtreeRefresh()) );
     treePacketViewActions.append(act);
 
-    act = new KAction(i18n("C&lone Packet"), "editcopy", CTRL+Key_L,
-        this, SLOT(clonePacket()), actionCollection(), "tree_clone");
+    act = actionCollection()->addAction("tree_clone");
+    act->setText(i18n("C&lone Packet"));
+    act->setIcon(KIcon("edit-copy"));
+    act->setShortcut(tr("Alt+l"));
     act->setToolTip(i18n("Clone the selected packet only"));
     act->setWhatsThis(i18n("Clone the packet currently selected in "
         "the tree.  The new clone will be placed alongside the original "
         "packet."));
+    connect(act, SIGNAL(triggered()), this, SLOT(clonePacket()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("Clone Su&btree"), QString::null, 0,
-        this, SLOT(cloneSubtree()), actionCollection(),
-        "tree_clone_subtree");
+    act = actionCollection()->addAction("tree_clone_subtree");
+    act->setText(i18n("Clone Su&btree"));
     act->setToolTip(i18n("Clone the subtree beneath the selected packet"));
     act->setWhatsThis(i18n("Clone the packet currently selected in "
         "the tree, as well as all of its descendants in the tree.  The new "
         "cloned subtree will be placed alongside the original packet."));
+    connect(act, SIGNAL(triggered()), this, SLOT(cloneSubtree()) );
     treePacketEditActions.append(act);
 
     // Tree reorganisation:
-    act = new KAction(i18n("&Higher Level"), "1leftarrow", CTRL+Key_Left,
-        this, SLOT(moveShallow()), actionCollection(), "nav_shallow");
+    act = actionCollection()->addAction("nav_shallow");
+    act->setText(i18n("&Higher Level"));
+    act->setIcon(KIcon("arrow-left"));
+    act->setShortcut(tr("Alt+Left"));
     act->setToolTip(i18n("Move packet to a higher (shallower) level "
         "in the tree"));
     act->setWhatsThis(i18n("Move the currently selected packet "
         "one level higher (shallower) in the packet tree.  The packet will "
         "abandon its current parent, and move one level closer to the root "
         "of the tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(moveShallow()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("&Lower Level"), "1rightarrow", CTRL+Key_Right,
-        this, SLOT(moveDeep()), actionCollection(), "nav_deep");
+    act = actionCollection()->addAction("nav_deep");
+    act->setText(i18n("&Lower Level"));
+    act->setIcon(KIcon("arrow-right"));
+    act->setShortcut(tr("Alt+Right"));
     act->setToolTip(i18n("Move packet to a lower (deeper) level in the tree"));
     act->setWhatsThis(i18n("Move the currently selected packet "
         "one level lower (deeper) in the packet tree.  The packet will "
         "abandon its current parent, and instead become a child of its "
         "next sibling."));
+    connect(act, SIGNAL(triggered()), this, SLOT(moveDeep()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("&Up"), "1uparrow", CTRL+Key_Up,
-        this, SLOT(moveUp()), actionCollection(), "nav_up");
+    act = actionCollection()->addAction("nav_up");
+    act->setText(i18n("&Up"));
+    act->setIcon(KIcon("arrow-up"));
+    act->setShortcut(tr("Alt+Up"));
     act->setToolTip(i18n("Move packet up through its siblings"));
     act->setWhatsThis(i18n("Move the currently selected packet "
         "one step up in the packet tree.  The packet will keep the "
         "same parent."));
+    connect(act, SIGNAL(triggered()), this, SLOT(moveUp()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("&Down"), "1downarrow", CTRL+Key_Down,
-        this, SLOT(moveDown()), actionCollection(), "nav_down");
+    act = actionCollection()->addAction("nav_down");
+    act->setText(i18n("&Down"));
+    act->setIcon(KIcon("arrow-down"));
+    act->setShortcut(tr("Alt+Down"));
     act->setToolTip(i18n("Move packet down through its siblings"));
     act->setWhatsThis(i18n("Move the currently selected packet "
         "one step down in the packet tree.  The packet will keep the "
         "same parent."));
+    connect(act, SIGNAL(triggered()), this, SLOT(moveDown()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("Jump U&p"), "2uparrow", CTRL+Key_PageUp,
-        this, SLOT(movePageUp()), actionCollection(), "nav_pageup");
+    act = actionCollection()->addAction("nav_pageup");
+    act->setText(i18n("Jump U&p"));
+    act->setIcon(KIcon("arrow-up-double"));
+    act->setShortcut(tr("Alt+Shift+Up"));
     act->setToolTip(i18n("Jump packet up through its siblings"));
     act->setWhatsThis(i18n("Move the currently selected packet "
         "several steps up in the packet tree.  The packet will keep the "
         "same parent."));
+    connect(act, SIGNAL(triggered()), this, SLOT(movePageUp()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("Jump Do&wn"), "2downarrow", CTRL+Key_PageDown,
-        this, SLOT(movePageDown()), actionCollection(), "nav_pagedown");
+    act = actionCollection()->addAction("nav_pagedown");
+    act->setText(i18n("Jump Do&wn"));
+    act->setIcon(KIcon("arrow-down-double"));
+    act->setShortcut(tr("Alt+Shift+Down"));
     act->setToolTip(i18n("Jump packet down through its siblings"));
     act->setWhatsThis(i18n("Move the currently selected packet "
         "several steps down in the packet tree.  The packet will keep the "
         "same parent."));
+    connect(act, SIGNAL(triggered()), this, SLOT(movePageDown()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("&Top"), "top", CTRL+Key_Home,
-        this, SLOT(moveTop()), actionCollection(), "nav_top");
+    act = actionCollection()->addAction("nav_top");
+    act->setText(i18n("&Top"));
+    act->setIcon(KIcon("go-top"));
+    act->setShortcut(tr("Alt+Home"));
     act->setToolTip(i18n("Move packet above all its siblings"));
     act->setWhatsThis(i18n("Move the currently selected packet "
         "up as far as possible amongst its siblings in the packet tree.  "
         "The packet will keep the same parent, but it will become the "
         "first child of this parent."));
+    connect(act, SIGNAL(triggered()), this, SLOT(moveTop()) );
     treePacketEditActions.append(act);
 
-    act = new KAction(i18n("&Bottom"), "bottom", CTRL+Key_End,
-        this, SLOT(moveBottom()), actionCollection(), "nav_bottom");
+    act = actionCollection()->addAction("nav_bottom");
+    act->setText(i18n("&Bottom"));
+    act->setIcon(KIcon("go-bottom"));
+    act->setShortcut(tr("Alt+End"));
     act->setToolTip(i18n("Move packet below all its siblings"));
     act->setWhatsThis(i18n("Move the currently selected packet "
         "down as far as possible amongst its siblings in the packet tree.  "
         "The packet will keep the same parent, but it will become the "
         "last child of this parent."));
+    connect(act, SIGNAL(triggered()), this, SLOT(moveBottom()) );
     treePacketEditActions.append(act);
 
     // New packets:
-    act = new KAction(i18n("New &Angle Structure Solutions"), "packet_angles",
-        ALT+Key_A, this, SLOT(newAngleStructures()), actionCollection(),
-        "tree_angles");
+    act = actionCollection()->addAction("tree_angles");
+    act->setText(i18n("New &Angle Structure Solutions"));
+    act->setIcon(KIcon("packet_angles"));
+    act->setShortcut(tr("Alt+a"));
     act->setToolTip(i18n("New angle structure solutions"));
     act->setWhatsThis(i18n("Create a new list of vertex angle structures "
         "for a triangulation."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newAngleStructures()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("New &Container"), "packet_container", ALT+Key_C,
-        this, SLOT(newContainer()), actionCollection(), "tree_container");
+    act = actionCollection()->addAction("tree_container");
+    act->setText(i18n("New &Container"));
+    act->setIcon(KIcon("packet_container"));
+    act->setShortcut(tr("Alt+c"));
     act->setToolTip(i18n("New container"));
     act->setWhatsThis(i18n("Create a new container packet.  Containers "
         "are used to help keep the packet tree organised &ndash; "
         "they serve no purpose other than to store child packets."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newContainer()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("New &Filter"), "packet_filter", ALT+Key_F,
-        this, SLOT(newFilter()), actionCollection(), "tree_filter");
+    act = actionCollection()->addAction("tree_filter");
+    act->setText(i18n("New &Filter"));
+    act->setIcon(KIcon("packet_filter"));
+    act->setShortcut(tr("Alt+f"));
     act->setToolTip(i18n("New surface filter"));
     act->setWhatsThis(i18n("Create a new normal surface filter.  Surface "
         "filters can be used to sort through normal surface lists and "
         "display only surfaces of particular interest."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newFilter()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("New &Normal Surface List"), "packet_surfaces",
-        ALT+Key_N, this, SLOT(newNormalSurfaces()), actionCollection(),
-        "tree_surfaces");
+    act = actionCollection()->addAction("tree_surfaces");
+    act->setText(i18n("New &Normal Surface List"));
+    act->setIcon(KIcon("packet_surfaces"));
+    act->setShortcut(tr("Alt+n"));
     act->setToolTip(i18n("New normal surface list"));
     act->setWhatsThis(i18n("Create a new list of vertex normal surfaces "
         "for a triangulation."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newNormalSurfaces()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("New &PDF Document"), "packet_pdf", ALT+Key_P,
-        this, SLOT(newPDF()), actionCollection(), "tree_pdf");
+    act = actionCollection()->addAction("tree_pdf");
+    act->setText(i18n("New &PDF Document"));
+    act->setIcon(KIcon("packet_pdf"));
+    act->setShortcut(tr("Alt+p"));
     act->setToolTip(i18n("New PDF document"));
     act->setWhatsThis(i18n("Create a new PDF packet containing a copy of "
         "an external PDF document."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newPDF()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("New &Script"), "packet_script", ALT+Key_S,
-        this, SLOT(newScript()), actionCollection(), "tree_script");
+    act = actionCollection()->addAction("tree_script");
+    act->setText(i18n("New &Script"));
+    act->setIcon(KIcon("packet_script"));
+    act->setShortcut(tr("Alt+s"));
     act->setToolTip(i18n("New script packet"));
     act->setWhatsThis(i18n("Create a new Python script that can work "
         "directly with this data file."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newScript()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("New Te&xt"), "packet_text", ALT+Key_X,
-        this, SLOT(newText()), actionCollection(), "tree_text");
+    act = actionCollection()->addAction("tree_text");
+    act->setText(i18n("New Te&xt"));
+    act->setIcon(KIcon("packet_text"));
+    act->setShortcut(tr("Alt+x"));
     act->setToolTip(i18n("New text packet"));
     act->setWhatsThis(i18n("Create a new piece of text to store within "
         "the packet tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newText()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("New &Triangulation"), "packet_triangulation",
-        ALT+Key_T, this, SLOT(newTriangulation()), actionCollection(),
-        "tree_triangulation");
+    act = new KAction(KIcon("packet_triangulation"), i18n("New &Triangulation"),
+        this);
+    act = actionCollection()->addAction("tree_triangulation");
+    act->setText(i18n("New &Triangulation"));
+    act->setIcon(KIcon("packet_triangulation"));
+    act->setShortcut(tr("Alt+t"));
     act->setToolTip(i18n("New triangulation"));
     act->setWhatsThis(i18n("Create a new 3-manifold triangulation."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newTriangulation()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("Form &Census"), "view_text", 0,
-        this, SLOT(newCensus()), actionCollection(), "tree_census");
+    act = actionCollection()->addAction("tree_census");
+    act->setText(i18n("Form &Census"));
+    act->setIcon(KIcon("view-list-text"));
     act->setToolTip(i18n("Form a new census of triangulations"));
     act->setWhatsThis(i18n("Create a new census of 3-manifold "
         "triangulations according to some set of census constraints."));
+    connect(act, SIGNAL(triggered()), this, SLOT(newCensus()) );
     treeGeneralEditActions.append(act);
 
     // Imports and exports:
-    act = new KAction(i18n("&Regina Data File"), "regina", 0,
-        this, SLOT(importRegina()), actionCollection(), "import_regina");
+    act = actionCollection()->addAction("import_regina");
+    act->setText(i18n("&Regina Data File"));
+    act->setIcon(KIcon("regina"));
     act->setToolTip(i18n("Import a Regina data file"));
     act->setWhatsThis(i18n("Import an external Regina data file.  The "
         "imported packet tree will be grafted into this packet tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(importRegina()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("&SnapPea Triangulation"), "snappea", 0,
-        this, SLOT(importSnapPea()), actionCollection(), "import_snappea");
+    act = actionCollection()->addAction("import_snappea");
+    act->setText(i18n("&SnapPea Triangulation"));
+    act->setIcon(KIcon("snappea"));
     act->setToolTip(i18n("Import a SnapPea triangulation"));
     act->setWhatsThis(i18n("Import an external SnapPea file as a new "
         "triangulation in this packet tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(importSnapPea()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("&Orb / Casson Triangulation"), "orb", 0,
-        this, SLOT(importOrb()), actionCollection(), "import_orb");
+    act= actionCollection()->addAction("import_orb");
+    act->setText(i18n("&Orb / Casson Triangulation"));
+    act->setIcon(KIcon("orb"));
     act->setToolTip(i18n("Import an Orb / Casson triangulation"));
     act->setWhatsThis(i18n("Import an external Orb / Casson file as a new "
         "triangulation in this packet tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(importOrb()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("&Dehydrated Triangulation List"), "dehydrated", 0,
-        this, SLOT(importDehydration()), actionCollection(),
-        "import_dehydrated");
+    act = actionCollection()->addAction("import_isosig3");
+    act->setText(i18n("&Isomorphism Signature List"));
+    act->setIcon(KIcon("document-sign"));
+    act->setToolTip(i18n("Import an isomorphism signature list "
+        "for 3-manifold triangulations"));
+    act->setWhatsThis(i18n("Import an external text file containing "
+        "isomorphism signatures for 3-manifold triangulations.  "
+        "For each isomorphism signature, "
+        "a new 3-manifold triangulation will be created in this packet tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(importIsoSig3()) );
+    treeGeneralEditActions.append(act);
+
+    act = actionCollection()->addAction("import_dehydrated");
+    act->setText(i18n("&Dehydrated Triangulation List"));
+    act->setIcon(KIcon("dehydrated"));
     act->setToolTip(i18n("Import a dehydrated triangulation list"));
     act->setWhatsThis(i18n("Import an external text file containing "
         "dehydrated triangulation strings.  For each dehydration string, "
         "a new triangulation will be created in this packet tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(importDehydration()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("&PDF Document"), "packet_pdf", 0,
-        this, SLOT(importPDF()), actionCollection(), "import_pdf");
+    act = actionCollection()->addAction("import_pdf");
+    act->setText(i18n("&PDF Document"));
+    act->setIcon(KIcon("packet_pdf"));
     act->setToolTip(i18n("Import a PDF document"));
     act->setWhatsThis(i18n("Import an external PDF document as a new PDF "
         "packet in this tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(importPDF()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("P&ython Script"), "packet_script", 0,
-        this, SLOT(importPython()), actionCollection(), "import_python");
+    act = actionCollection()->addAction("import_python");
+    act->setText(i18n("P&ython Script"));
+    act->setIcon(KIcon("packet_script"));
     act->setToolTip(i18n("Import a Python script"));
     act->setWhatsThis(i18n("Import an external Python file as a new script "
         "packet in this tree."));
+    connect(act, SIGNAL(triggered()), this, SLOT(importPython()) );
     treeGeneralEditActions.append(act);
 
-    act = new KAction(i18n("&Regina Data File"), "regina", 0,
-        this, SLOT(exportRegina()), actionCollection(), "export_regina");
+    act = actionCollection()->addAction("export_regina");
+    act->setText(i18n("&Regina Data File"));
+    act->setIcon(KIcon("regina"));
     act->setToolTip(i18n("Export a compressed Regina data file"));
     act->setWhatsThis(i18n("Export all or part of this packet tree "
         "to a separate Regina data file.  The separate data file will "
         "be saved as compressed XML (the default format)."));
+    connect(act, SIGNAL(triggered()), this, SLOT(exportRegina()) );
 
-    act = new KAction(i18n("Regina Data File (&Uncompressed)"), "regina", 0,
-        this, SLOT(exportReginaUncompressed()), actionCollection(),
-        "export_regina_uncompressed");
+    act = actionCollection()->addAction("export_regina_uncompressed");
+    act->setText(i18n("Regina Data File (&Uncompressed)"));
+    act->setIcon(KIcon("regina"));
     act->setToolTip(i18n("Export an uncompressed Regina data file"));
     act->setWhatsThis(i18n("Export all or part of this packet tree "
         "to a separate Regina data file.  The separate data file will "
         "be saved as uncompressed XML."));
+    connect(act, SIGNAL(triggered()), this, SLOT(exportReginaUncompressed()) );
 
-    act = new KAction(i18n("&SnapPea Triangulation"), "snappea", 0,
-        this, SLOT(exportSnapPea()), actionCollection(), "export_snappea");
+    act = actionCollection()->addAction("export_snappea");
+    act->setText(i18n("&SnapPea Triangulation"));
+    act->setIcon(KIcon("snappea"));
     act->setToolTip(i18n("Export a SnapPea triangulation"));
     act->setWhatsThis(i18n("Export a triangulation from this packet tree "
         "to a separate SnapPea file."));
+    connect(act, SIGNAL(triggered()), this, SLOT(exportSnapPea()) );
 
-    act = new KAction(i18n("&C++ Source"), "source_cpp", 0,
-        this, SLOT(exportSource()), actionCollection(), "export_source");
+    act = actionCollection()->addAction("export_source");
+    act->setText(i18n("&C++ Source"));
+    act->setIcon(KIcon("text-x-c++src"));
     act->setToolTip(i18n("Export a triangulation as C++ source"));
     act->setWhatsThis(i18n("Export a triangulation from this packet tree "
         "to a C++ source file.<p>"
         "The exported C++ code will reconstruct the original triangulation.  "
         "See the users' handbook for further information on using Regina "
         "in your own code."));
+    connect(act, SIGNAL(triggered()), this, SLOT(exportSource()) );
 
-    act = new KAction(i18n("CS&V Surface List"), "csvexport", 0,
-        this, SLOT(exportCSVSurfaceList()), actionCollection(),
-        "export_csv_surfaces");
+    act = actionCollection()->addAction("export_csv_surfaces");
+    act->setText(i18n("CS&V Surface List"));
+    act->setIcon(KIcon("csvexport"));
     act->setToolTip(i18n("Export a normal surface list as a "
         "text file with comma-separated values"));
     act->setWhatsThis(i18n("Export a normal surface list from this packet tree "
@@ -327,17 +427,22 @@ void ReginaPart::setupActions() {
         "Individual disc coordinates as well as various properties of the "
         "normal surfaces (such as orientability and Euler characteristic) "
         "will all be stored as separate fields in the CSV file."));
+    connect(act, SIGNAL(triggered()), this, SLOT(exportCSVSurfaceList()) );
 
-    act = new KAction(i18n("&PDF Document"), "packet_pdf", 0,
-        this, SLOT(exportPDF()), actionCollection(), "export_pdf");
+    act = actionCollection()->addAction("export_pdf");
+    act->setText(i18n("&PDF Document"));
+    act->setIcon(KIcon("packet_pdf"));
     act->setToolTip(i18n("Export a PDF document"));
     act->setWhatsThis(i18n("Export a PDF packet from this packet tree "
         "to a separate PDF document."));
+    connect(act, SIGNAL(triggered()), this, SLOT(exportPDF()) );
 
-    act = new KAction(i18n("P&ython Script"), "packet_script", 0,
-        this, SLOT(exportPython()), actionCollection(), "export_python");
+    act = actionCollection()->addAction("export_python");
+    act->setText(i18n("P&ython Script"));
+    act->setIcon(KIcon("packet_script"));
     act->setToolTip(i18n("Export a Python script"));
     act->setWhatsThis(i18n("Export a script packet from this packet tree "
         "to a separate Python file."));
+    connect(act, SIGNAL(triggered()), this, SLOT(exportPython()) );
 }
 
