@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -38,14 +38,14 @@
 
 const PDFHandler PDFHandler::instance;
 
-regina::NPacket* PDFHandler::import(const QString& fileName,
+regina::NPacket* PDFHandler::importData(const QString& fileName,
         QWidget* parentWidget) const {
     regina::NPacket* ans = regina::readPDF(
         static_cast<const char*>(QFile::encodeName(fileName)));
     if (! ans)
         KMessageBox::error(parentWidget, i18n(
             "The PDF document %1 could not be read.").arg(fileName));
-    ans->setPacketLabel(i18n("PDF document").ascii());
+    ans->setPacketLabel(i18n("PDF document").toAscii().constData());
     return ans;
 }
 
