@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -38,7 +38,7 @@
 
 const SnapPeaHandler SnapPeaHandler::instance;
 
-regina::NPacket* SnapPeaHandler::import(const QString& fileName,
+regina::NPacket* SnapPeaHandler::importData(const QString& fileName,
         QWidget* parentWidget) const {
     regina::NPacket* ans = regina::readSnapPea(
         static_cast<const char*>(QFile::encodeName(fileName)));
