@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -32,6 +32,7 @@
 #include "reginapart.h"
 #include "foreign/dehydrationhandler.h"
 #include "foreign/importdialog.h"
+#include "foreign/isosighandler.h"
 #include "foreign/orbhandler.h"
 #include "foreign/pdfhandler.h"
 #include "foreign/pythonhandler.h"
@@ -49,6 +50,11 @@ void ReginaPart::importDehydration() {
         i18n("Import Dehydrated Triangulation List"));
 }
 
+void ReginaPart::importIsoSig3() {
+    importFile(IsoSigHandler::instance3, 0, i18n(FILTER_ALL),
+        i18n("Import Isomorphism Signature List"));
+}
+
 void ReginaPart::importPDF() {
     importFile(PDFHandler::instance, 0, i18n(FILTER_PDF),
         i18n("Import PDF Document"));
@@ -89,14 +95,14 @@ void ReginaPart::importFile(const PacketImporter& importer,
             QString::null, fileFilter, widget(), dialogTitle);
         if (result.fileNames.empty() || result.fileNames.front().isEmpty())
             return;
-        newTree = importer.import(result.fileNames.front(),
-            QTextCodec::codecForName(result.encoding), widget());
+        newTree = importer.importData(result.fileNames.front(),
+            QTextCodec::codecForName(result.encoding.toAscii()), widget());
     } else {
-        QString file = KFileDialog::getOpenFileName(QString::null,
+        QString file = KFileDialog::getOpenFileName(KUrl(),
             fileFilter, widget(), dialogTitle);
         if (file.isEmpty())
             return;
-        newTree = importer.import(file, widget());
+        newTree = importer.importData(file, widget());
     }
 
     if (newTree) {
