@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -94,8 +94,10 @@ void ReginaPart::newPacket(PacketCreator* creator, PacketFilter* parentFilter,
         treeView->selectedPacket(), parentFilter, dialogTitle, suggestedLabel);
     if (dlg.validate() && dlg.exec() == QDialog::Accepted) {
         regina::NPacket* newPacket = dlg.createdPacket();
-        if (newPacket)
-            packetView(newPacket, true);
+        if (newPacket) {
+            // Open a UI for the new packet, and select it in the tree.
+            packetView(newPacket, true, true);
+        }
     }
 }
 
