@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -33,6 +33,8 @@
 #ifndef __PACKETCREATOR_H
 #define __PACKETCREATOR_H
 
+#include <qstring.h>
+
 class QWidget;
 
 namespace regina {
@@ -66,6 +68,17 @@ class PacketCreator {
          */
         virtual QWidget* getInterface();
 
+        /**
+         * If the parent packet plays a special role for this packet
+         * type, you may wish to customise the corresponding prompt
+         * and associated "what's this" text in the new packet dialog.
+         * If these routines return null strings, the default
+         * "Create beneath:" prompt and and corresponding help text will be
+         * used.
+         */
+        virtual QString parentPrompt();
+        virtual QString parentWhatsThis();
+
         /**
          * Create the packet according to the information entered by the
          * user in the main interface component.
@@ -109,4 +122,12 @@ inline QWidget* PacketCreator::getInterface() {
     return 0;
 }
 
+inline QString PacketCreator::parentPrompt() {
+    return QString();
+}
+
+inline QString PacketCreator::parentWhatsThis() {
+    return QString();
+}
+
 #endif
