@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -34,7 +34,7 @@
 #ifndef __ELTMOVEDIALOG_H
 #define __ELTMOVEDIALOG_H
 
-#include <kdialogbase.h>
+#include <kdialog.h>
 #include <vector>
 
 class QButtonGroup;
@@ -49,7 +49,7 @@ namespace regina {
  * A dialog used to select and perform an elementary move on a
  * triangulation.
  */
-class EltMoveDialog : public KDialogBase {
+class EltMoveDialog : public KDialog {
     Q_OBJECT
 
     private:
@@ -106,7 +106,7 @@ class EltMoveDialog : public KDialogBase {
 
     protected slots:
         /**
-         * KDialogBase overrides.
+         * Ok has been clicked.
          */
         virtual void slotOk();
 
