@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -38,7 +38,6 @@
 #include "../packetui.h"
 
 class QLabel;
-class QVBox;
 
 namespace regina {
     class NContainer;
@@ -60,7 +59,7 @@ class NContainerUI : public PacketReadOnlyUI, public regina::NPacketListener {
         /**
          * Internal components
          */
-        QVBox* interface;
+        QWidget* interface;
         QLabel* children;
         QLabel* descendants;
 
