@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -41,7 +41,6 @@
 
 #include <klocale.h>
 #include <qlabel.h>
-#include <qvbox.h>
 #include <qwhatsthis.h>
 
 using regina::NPacket;
@@ -84,7 +83,7 @@ NNormalSurfaceUI::NNormalSurfaceUI(regina::NNormalSurfaceList* packet,
     }
 }
 
-const QPtrList<KAction>& NNormalSurfaceUI::getPacketTypeActions() {
+const QLinkedList<KAction*>& NNormalSurfaceUI::getPacketTypeActions() {
     return coords->getPacketTypeActions();
 }
 
@@ -102,7 +101,7 @@ NSurfaceHeaderUI::NSurfaceHeaderUI(regina::NNormalSurfaceList* packet,
     header = new QLabel(0);
     header->setAlignment(Qt::AlignCenter);
     header->setMargin(10);
-    QWhatsThis::add(header, i18n("Displays the parameters of the "
+    header->setWhatsThis(i18n("Displays the parameters of the "
         "vertex enumeration that created this list of surfaces, including "
         "the specific coordinate system that was originally used.  Also "
         "displays the total number of surfaces in this list."));
@@ -135,4 +134,3 @@ void NSurfaceHeaderUI::refresh() {
         Coordinates::name(surfaces->getFlavour(), false)));
 }
 
-#include "nnormalsurfaceui.moc"
