@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -34,6 +34,7 @@
 #define __NNORMALSURFACEUI_H
 
 #include "../packettabui.h"
+#include <QLinkedList>
 
 class NSurfaceCompatibilityUI;
 class NSurfaceCoordinateUI;
@@ -67,7 +68,7 @@ class NNormalSurfaceUI : public PacketTabbedUI {
         /**
          * PacketUI overrides.
          */
-        const QPtrList<KAction>& getPacketTypeActions();
+        const QLinkedList<KAction*>& getPacketTypeActions();
         QString getPacketMenuText() const;
 
     public slots:
