@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -41,16 +41,16 @@
 #include <qlayout.h>
 #include <qmessagebox.h>
 #include <qpixmap.h>
-#include <qstylesheet.h>
-#include <qwidgetstack.h>
+#include <QTextDocument>
+#include <QStackedWidget>
 #include <kiconloader.h>
 #include <klocale.h>
-#include <kparts/componentfactory.h>
+#include <kmimetypetrader.h>
 #include <kparts/partmanager.h>
 #include <kprocess.h>
 #include <krun.h>
+#include <kshell.h>
 #include <kstandarddirs.h>
-#include <ktempfile.h>
 #include <kurl.h>
 
 #define PDF_MIMETYPE "application/pdf"
@@ -60,29 +60,24 @@ using regina::NPDF;
 
 NPDFUI::NPDFUI(NPDF* packet, PacketPane* enclosingPane) :
         PacketReadOnlyUI(enclosingPane), pdf(packet),
-        temp(locateLocal("tmp", "pdf-"), ".pdf"),
         viewer(0), proc(0), runPid(0) {
-    temp.setAutoDelete(true);
-    temp.close();
+    temp.setSuffix(".pdf");
 
     ReginaPart* part = enclosingPane->getPart();
     const ReginaPrefSet& prefs = part->getPreferences();
     autoClose = prefs.pdfAutoClose;
     embed = prefs.pdfEmbed;
-    externalViewer = prefs.pdfExternalViewer.stripWhiteSpace();
+    externalViewer = prefs.pdfExternalViewer.trimmed();
 
-    ui = new QWidget();
-    QBoxLayout* baseLayout = new QVBoxLayout(ui);
-    stack = new QWidgetStack(ui);
+    stack = new QStackedWidget();
 
     // Information layer.
-    layerInfo = messageLayer(msgInfo, "messagebox_info");
+    layerInfo = messageLayer(msgInfo, "dialog-information");
 
     // Error layer.
-    layerError = messageLayer(msgError, "messagebox_critical");
+    layerError = messageLayer(msgError, "dialog-error");
 
     // Finish off.
-    baseLayout->addWidget(stack);
     refresh();
 
     connect(part, SIGNAL(preferencesChanged(const ReginaPrefSet&)),
@@ -99,7 +94,7 @@ NPacket* NPDFUI::getPacket() {
 }
 
 QWidget* NPDFUI::getInterface() {
-    return ui;
+    return stack;
 }
 
 QString NPDFUI::getPacketMenuText() const {
@@ -114,10 +109,18 @@ void NPDFUI::refresh() {
         setDirty(false);
         return;
     }
-    if (! regina::writePDF(
-            static_cast<const char*>(QFile::encodeName(temp.name())), *pdf)) {
+    if (! temp.open()) {
+        showError(i18n("<qt>The temporary PDF file <i>%1</i> could not be "
+            "created.</qt>").arg(temp.fileName()));
+        setDirty(false);
+        return;
+    }
+    temp.close();
+
+    if (! regina::writePDF(static_cast<const char*>(
+            QFile::encodeName(temp.fileName())), *pdf)) {
         showError(i18n("An error occurred whilst writing the PDF "
-            "data to the temporary file %1.").arg(temp.name()));
+            "data to the temporary file <i>%1</i>.").arg(temp.fileName()));
         setDirty(false);
         return;
     }
@@ -129,9 +132,9 @@ void NPDFUI::refresh() {
     if (embed) {
         if (! viewer) {
             // We don't yet have an embedded PDF viewer.
-            viewer = KParts::ComponentFactory::
+            viewer = KMimeTypeTrader::
                 createPartInstanceFromQuery<KParts::ReadOnlyPart>(
-                PDF_MIMETYPE, QString::null, stack, 0, stack);
+                PDF_MIMETYPE, 0, this);
 
             if (viewer) {
                 viewer->setProgressInfoEnabled(false);
@@ -147,11 +150,13 @@ void NPDFUI::refresh() {
 
         // If we actually found ourselves a viewer, load the PDF.
         if (viewer) {
-            if (viewer->openURL(KURL::fromPathOrURL(temp.name())))
-                stack->raiseWidget(viewer->widget());
+            stack->addWidget(viewer->widget());
+            if (viewer->openUrl(KUrl(temp.fileName())))
+                stack->setCurrentWidget((viewer->widget()));
             else
                 showError(i18n("An error occurred whilst re-reading the PDF "
-                    "data from the temporary file %1.").arg(temp.name()));
+                    "data from the temporary file <i>%1</i>.")
+                    .arg(temp.fileName()));
 
             setDirty(false);
             return;
@@ -164,9 +169,9 @@ void NPDFUI::refresh() {
         showInfo(i18n("<qt>No embedded PDF viewer could be found on "
             "your system.  Falling back to an external viewer instead.<p>"
             "If you would like PDFs to appear directly inside "
-            "Regina's main window, you could try installing either "
-            "<i>kpdf</i> or <i>kghostview</i> (both part of the "
-            "<i>kdegraphics</i> package shipped with KDE 3.x).</qt>"));
+            "Regina's main window, you could try installing "
+            "<i>okular</i> (which is part of the "
+            "<i>kdegraphics</i> package shipped with KDE 4.x).</qt>"));
     else
         showInfo(i18n("<qt>Using an external PDF viewer as "
             "requested.<p>To change this preference, you can "
@@ -177,12 +182,12 @@ void NPDFUI::refresh() {
         viewer = 0;
 
         // Just to be sure...
-        stack->raiseWidget(layerInfo);
+        stack->setCurrentWidget(layerInfo);
     }
 
     if (externalViewer.isEmpty()) {
         // Fall back to the KDE default for PDFs.
-        runPid = KRun::runURL(KURL::fromPathOrURL(temp.name()), PDF_MIMETYPE,
+        runPid = KRun::runUrl(KUrl(temp.fileName()), PDF_MIMETYPE,
                 false /* delete temp file on application exit */,
                 false /* do not allow KRun to "open" executables */);
         if (! runPid)
@@ -191,23 +196,37 @@ void NPDFUI::refresh() {
                 "Please specify your preferred PDF viewer under the "
                 "PDF options in Regina's settings.</qt>"));
     } else {
-        QString filename = temp.name();
-        KRun::shellQuote(filename);
-        cmd = externalViewer + ' ' + filename;
+        cmd = externalViewer + ' ' + KShell::quoteArg(temp.fileName());
 
         proc = new KProcess(this);
-        proc->setUseShell(true);
-        (*proc) << cmd;
+        proc->setShellCommand(cmd);
 
-        connect(proc, SIGNAL(processExited(KProcess*)),
-            this, SLOT(processExited(KProcess*)));
-
-        if (! proc->start())
-            showError(i18n("<qt>Regina was unable to open an external "
-                "PDF viewer.  The failed command was:<p>"
-                "<tt>%1</tt><p>"
-                "You can fix this by editing the PDF options in "
-                "Regina's settings.</qt>").arg(QStyleSheet::escape(cmd)));
+        connect(proc, SIGNAL(finished(int, QProcess::ExitStatus)),
+            this, SLOT(processExited(int, QProcess::ExitStatus)));
+        
+        if (autoClose) {
+            proc->start();
+            if (! proc->waitForStarted(10000 /* milliseconds */)) {
+                showError(i18n("<qt>Regina was unable to open an external "
+                    "PDF viewer.  The failed command was:<p>"
+                    "<tt>%1</tt><p>"
+                    "You can fix this by editing the PDF options in "
+                    "Regina's settings.</qt>").arg(Qt::escape(cmd)));
+                proc->kill();
+                delete proc;
+                proc = 0;
+            }
+        } else {
+            if (! proc->startDetached()) {
+                showError(i18n("<qt>Regina was unable to open an external "
+                    "PDF viewer.  The failed command was:<p>"
+                    "<tt>%1</tt><p>"
+                    "You can fix this by editing the PDF options in "
+                    "Regina's settings.</qt>").arg(Qt::escape(cmd)));
+                delete proc;
+                proc = 0;
+            }
+        }
     }
 
     setDirty(false);
@@ -215,7 +234,7 @@ void NPDFUI::refresh() {
 
 void NPDFUI::updatePreferences(const ReginaPrefSet& newPrefs) {
     // Whitespace should already have been stripped by now, but just in case...
-    QString newExternalViewer = newPrefs.pdfExternalViewer.stripWhiteSpace();
+    QString newExternalViewer = newPrefs.pdfExternalViewer.trimmed();
 
     // Do we need to refresh afterwards?
     bool needRefresh = ((embed != newPrefs.pdfEmbed) ||
@@ -230,46 +249,48 @@ void NPDFUI::updatePreferences(const ReginaPrefSet& newPrefs) {
 }
 
 QWidget* NPDFUI::messageLayer(QLabel*& text, const char* iconName) {
-    QWidget* layer = new QWidget(stack);
-    QBoxLayout* layout = new QHBoxLayout(layer, 5 /* margin */,
-        5 /* spacing */);
+    QWidget* layer = new QWidget();
+    QBoxLayout* layout = new QHBoxLayout(layer);
 
     layout->addStretch(1);
 
-    QPixmap iconPic = enclosingPane->getPart()->instance()->iconLoader()->
-        loadIcon(iconName, KIcon::NoGroup, KIcon::SizeMedium,
-        KIcon::DefaultState, 0, true /* may be null */);
+    QPixmap iconPic = KIconLoader::global()->loadIcon(iconName, 
+        KIconLoader::NoGroup, KIconLoader::SizeMedium,
+        KIconLoader::DefaultState, QStringList(), 0L, true /* may be null */);
     if (iconPic.isNull())
         iconPic = QMessageBox::standardIcon(QMessageBox::Critical);
 
     QLabel* icon = new QLabel(layer);
     icon->setPixmap(iconPic);
-    layout->addWidget(icon);
-    layout->setStretchFactor(icon, 0);
+    layout->addWidget(icon, 0);
 
     layout->addSpacing(10);
 
     text = new QLabel(i18n("<qt>Initialising...</qt>"), layer);
-    layout->addWidget(text);
-    layout->setStretchFactor(text, 4);
+    text->setWordWrap(true);
+    layout->addWidget(text, 4);
 
     layout->addStretch(1);
+    stack->addWidget(layer);
 
     return layer;
 }
 
 void NPDFUI::showInfo(const QString& msg) {
     msgInfo->setText(msg);
-    stack->raiseWidget(layerInfo);
+    stack->setCurrentWidget(layerInfo);
 }
 
 void NPDFUI::showError(const QString& msg) {
     msgError->setText(msg);
-    stack->raiseWidget(layerError);
+    stack->setCurrentWidget(layerError);
 }
 
 void NPDFUI::abandonProcess() {
     if (proc) {
+        // Don't flag an error when we kill then process.
+        disconnect(proc, 0, this, 0);
+
         if (autoClose) {
             // Set proc = 0 *before* we kill the process, so that the
             // exit signal is ignored.
@@ -278,9 +299,8 @@ void NPDFUI::abandonProcess() {
             tmpProc->kill();
             delete tmpProc;
         } else {
-            // Cut the process free so it is not killed when the
-            // KProcess is eventually destroyed.
-            proc->detach();
+            // The process was already cut free when we called
+            // startDetached(); there is nothing more for us to do here.
             delete proc;
             proc = 0;
         }
@@ -291,17 +311,15 @@ void NPDFUI::abandonProcess() {
     }
 }
 
-void NPDFUI::processExited(KProcess* oldProc) {
+void NPDFUI::processExited(int exitCode, QProcess::ExitStatus exitStatus) {
     // Did we try to start a viewer but couldn't?
-    if (oldProc == proc) {
-        if (! (proc->normalExit() && proc->exitStatus() == 0))
-            showError(i18n("<qt>Regina tried to open an external "
-                "PDF viewer but could not.  The failed command was:<p>"
-                "<tt>%1</tt><p>"
-                "You can fix this by editing the PDF options in "
-                "Regina's settings.</qt>").arg(QStyleSheet::escape(cmd)));
-        proc = 0;
-    }
+    if (! (exitStatus == QProcess::NormalExit && exitCode == 0))
+        showError(i18n("<qt>Regina tried to open an external "
+            "PDF viewer but could not.  The failed command was:<p>"
+            "<tt>%1</tt><p>"
+            "You can fix this by editing the PDF options in "
+            "Regina's settings.</qt>").arg(Qt::escape(cmd)));
+    delete proc;
+    proc = 0;
 }
 
-#include "npdfui.moc"
