@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -35,7 +35,9 @@
 
 #include "../packetui.h"
 
-#include <ktempfile.h>
+#include <KTemporaryFile>
+#include <QProcess>
+#include <QStackedWidget>
 
 class QWidgetStack;
 class KProcess;
@@ -66,13 +68,12 @@ class NPDFUI : public QObject, public PacketReadOnlyUI {
         /**
          * Temporary PDF storage
          */
-        KTempFile temp;
+        KTemporaryFile temp;
 
         /**
          * Internal components
          */
-        QWidget* ui;
-        QWidgetStack* stack;
+        QStackedWidget* stack;
         QWidget* layerInfo;
         QWidget* layerError;
         QLabel* msgInfo;
@@ -139,7 +140,7 @@ class NPDFUI : public QObject, public PacketReadOnlyUI {
         /**
          * Process control for external PDF viewers.
          */
-        void processExited(KProcess* oldProc);
+        void processExited(int exitCode, QProcess::ExitStatus exitStatus);
 };
 
 #endif
