@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -37,13 +37,13 @@
 #include "ntriskeleton.h"
 #include "ntrisnappea.h"
 #include "ntrisurfaces.h"
+#include "../packeteditiface.h"
 #include "../reginapart.h"
 
 #include <klocale.h>
 #include <ktoolbar.h>
 #include <qlabel.h>
-#include <qvbox.h>
-#include <qwhatsthis.h>
+#include <QVBoxLayout>
 
 using regina::NPacket;
 using regina::NTriangulation;
@@ -93,9 +93,15 @@ NTriangulationUI::NTriangulationUI(regina::NTriangulation* packet,
         case ReginaPrefSet::SnapPea:
             setCurrentTab(5); break;
     }
+
+    editIface = new PacketEditTabbedUI(this);
 }
 
-const QPtrList<KAction>& NTriangulationUI::getPacketTypeActions() {
+NTriangulationUI::~NTriangulationUI() {
+    delete editIface;
+}
+
+const QLinkedList<KAction*>& NTriangulationUI::getPacketTypeActions() {
     return gluings->getPacketTypeActions();
 }
 
@@ -114,17 +120,21 @@ void NTriangulationUI::updatePreferences(const ReginaPrefSet& newPrefs) {
 NTriHeaderUI::NTriHeaderUI(regina::NTriangulation* packet,
         PacketTabbedUI* useParentUI) : PacketViewerTab(useParentUI),
         tri(packet) {
-    ui = new QVBox();
+    ui = new QWidget();
+    QBoxLayout* uiLayout = new QVBoxLayout();
+    uiLayout->setContentsMargins(0, 0, 0, 0);
+    ui->setLayout(uiLayout);
 
-    bar = new KToolBar(ui, "triangulationActionBar", false, false);
-    bar->setFullSize(true);
-    bar->setIconText(KToolBar::IconTextRight);
+    bar = new KToolBar(ui, false, true);
+    bar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+    uiLayout->addWidget(bar);
 
-    header = new QLabel(ui);
+    header = new QLabel();
     header->setAlignment(Qt::AlignCenter);
     header->setMargin(10);
-    QWhatsThis::add(header, i18n("Displays a few basic properties of the "
+    header->setWhatsThis(i18n("Displays a few basic properties of the "
         "triangulation, such as boundary and orientability."));
+    uiLayout->addWidget(header);
 }
 
 regina::NPacket* NTriHeaderUI::getPacket() {
@@ -151,15 +161,23 @@ void NTriHeaderUI::refresh() {
     if (tri->isClosed())
         msg += i18n("Closed, ");
     else {
-        if (tri->isIdeal())
-            msg += i18n("Ideal, ");
-        if (tri->hasBoundaryFaces())
-            msg += i18n("Real Bdry, ");
+        if (tri->isIdeal() && tri->hasBoundaryFaces())
+            msg += i18n("Ideal & real bdry, ");
+        else if (tri->isIdeal())
+            msg += i18n("Ideal bdry, ");
+        else if (tri->hasBoundaryFaces())
+            msg += i18n("Real bdry, ");
     }
 
-    msg += (tri->isOrientable() ? i18n("Orientable, ") :
-        i18n("Non-orientable, "));
-    msg += (tri->isConnected() ? i18n("Connected") : i18n("Disconnected"));
+    if (tri->isOrientable()) {
+        if (tri->isOriented())
+            msg += i18n("orientable and oriented, ");
+        else
+            msg += i18n("orientable, ");
+    } else
+        msg += i18n("non-orientable, ");
+
+    msg += (tri->isConnected() ? i18n("connected") : i18n("disconnected"));
 
     header->setText(msg);
 }
@@ -168,4 +186,3 @@ void NTriHeaderUI::editingElsewhere() {
     header->setText(i18n("Editing..."));
 }
 
-#include "ntriangulationui.moc"
