@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -56,26 +56,25 @@
 // UI includes:
 #include "ntricomposition.h"
 #include "../packetchooser.h"
+#include "../packeteditiface.h"
 #include "../packetfilter.h"
 
 #include <kiconloader.h>
-#include <klistview.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <memory>
 #include <qapplication.h>
 #include <qclipboard.h>
-#include <qheader.h>
+#include <QHeaderView>
 #include <qlabel.h>
 #include <qlayout.h>
-#include <qpopupmenu.h>
+#include <QMenu>
 #include <qpushbutton.h>
-#include <qtooltip.h>
-#include <qwhatsthis.h>
+#include <QTreeWidgetItem>
 
 using regina::NEdge;
 using regina::NPacket;
-using regina::NPerm;
+using regina::NPerm4;
 using regina::NSatRegion;
 using regina::NTriangulation;
 
@@ -86,14 +85,13 @@ NTriCompositionUI::NTriCompositionUI(regina::NTriangulation* packet,
 
     ui = new QWidget();
     QBoxLayout* layout = new QVBoxLayout(ui);
-    layout->addSpacing(5);
 
     // Set up the isomorphism tester.
-    QBoxLayout* wideIsoArea = new QHBoxLayout(layout, 5);
-    wideIsoArea->addSpacing(5);
+    QBoxLayout* wideIsoArea = new QHBoxLayout();
+    layout->addLayout(wideIsoArea);
 
-    QBoxLayout* leftIsoArea = new QVBoxLayout(wideIsoArea, 0);
-    wideIsoArea->setStretchFactor(leftIsoArea, 1);
+    QBoxLayout* leftIsoArea = new QVBoxLayout();
+    wideIsoArea->addLayout(leftIsoArea, 1);
 
     QString msg = i18n("<qt>Compare this with another triangulation to "
         "see whether the triangulations are isomorphic, or whether one is "
@@ -105,50 +103,48 @@ NTriCompositionUI::NTriCompositionUI(regina::NTriangulation* packet,
         "examined through the <i>Details</i> button.");
 
     QLabel* label = new QLabel(i18n("Isomorphism / subcomplex test:"), ui);
-    QWhatsThis::add(label, msg);
+    label->setWhatsThis(msg);
     leftIsoArea->addWidget(label);
 
-    QBoxLayout* isoSelectArea = new QHBoxLayout(leftIsoArea, 5);
+    QBoxLayout* isoSelectArea = new QHBoxLayout();
+    leftIsoArea->addLayout(isoSelectArea);
     label = new QLabel(i18n("Compare with T ="), ui);
-    QWhatsThis::add(label, msg);
+    label->setWhatsThis(msg);
     isoSelectArea->addWidget(label);
     isoTest = new PacketChooser(tri->getTreeMatriarch(),
         new SingleTypeFilter<NTriangulation>(), true, 0, ui);
     isoTest->setAutoUpdate(true);
-    QWhatsThis::add(isoTest, msg);
+    isoTest->setWhatsThis(msg);
     connect(isoTest, SIGNAL(activated(int)), this, SLOT(updateIsoPanel()));
     isoSelectArea->addWidget(isoTest);
     isoSelectArea->addStretch(1);
 
     isoResult = new QLabel(i18n("Result:"), ui);
-    QWhatsThis::add(isoResult, msg);
+    isoResult->setWhatsThis(msg);
     leftIsoArea->addWidget(isoResult);
 
-    isoView = new QPushButton(SmallIconSet("viewmag"), i18n("Details..."), ui);
+    isoView = new QPushButton(KIcon("packet_view"), i18n("Details..."), ui);
     // isoView->setFlat(true);
-    QToolTip::add(isoView, i18n("View details of isomorphism"));
-    QWhatsThis::add(isoView, i18n("View the details of the isomorphism "
+    isoView->setToolTip(i18n("View details of isomorphism"));
+    isoView->setWhatsThis(i18n("View the details of the isomorphism "
         "(if any) between this and the selected triangulation.  The precise "
         "mapping between tetrahedra and tetrahedron vertices will be "
         "displayed in a separate window."));
     connect(isoView, SIGNAL(clicked()), this, SLOT(viewIsomorphism()));
     wideIsoArea->addWidget(isoView);
-    wideIsoArea->addSpacing(5);
 
     // Add a central divider.
-    layout->addSpacing(5);
-
     QFrame* divider = new QFrame(ui);
     divider->setFrameStyle(QFrame::HLine | QFrame::Sunken);
     layout->addWidget(divider);
 
-    layout->addSpacing(5);
-
     // Set up the composition viewer.
     msg = i18n("<qt>Displays (i) the precise name of the triangulation "
         "and/or underlying 3-manifold if these can be recognised "
-        "immediately, (ii) the Callahan-Hildebrand-Weeks dehydration "
-        "string if the triangulation supports it, and (iii) the details "
+        "immediately, (ii) the isomorphism signature from which the "
+        "triangulation can be reconstructed, "
+        "(iii) the Callahan-Hildebrand-Weeks dehydration "
+        "string if the triangulation supports it, and (iv) the details "
         "of any standard combinatorial structures found within the "
         "triangulation.<p>"
         "You can right-click on any line of text to copy it to the "
@@ -157,24 +153,21 @@ NTriCompositionUI::NTriCompositionUI(regina::NTriangulation* packet,
         "listed here.</qt>");
 
     label = new QLabel(i18n("Triangulation composition:"), ui);
-    QWhatsThis::add(label, msg);
+    label->setWhatsThis(msg);
     layout->addWidget(label);
 
-    details = new KListView(ui);
-    details->header()->hide();
-    details->addColumn(QString::null);
-    details->setSorting(-1);
-    details->setSelectionMode(QListView::Single);
-    QWhatsThis::add(details, msg);
+    details = new QTreeWidget(ui);
+    details->setHeaderHidden(true);
+    details->setAlternatingRowColors(true);
+    details->setSelectionMode(QAbstractItemView::SingleSelection);
+    details->setWhatsThis(msg);
     layout->addWidget(details, 1);
 
-    // Set up context menus.
-    detailsMenu = new QPopupMenu(details);
-    detailsMenu->insertItem(i18n("&Copy to Clipboard"),
-        this, SLOT(detailsCopy()));
-    connect(details,
-        SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
-        this, SLOT(detailsPopup(QListViewItem*, const QPoint&, int)));
+    editIface = new PacketEditTreeWidgetSingleLine(details);
+}
+
+NTriCompositionUI::~NTriCompositionUI() {
+    delete editIface;
 }
 
 regina::NPacket* NTriCompositionUI::getPacket() {
@@ -207,6 +200,9 @@ void NTriCompositionUI::refresh() {
     } else
         addTopLevelSection(i18n("Triangulation not recognised"));
 
+    // Add the isomorphism signature.
+    addTopLevelSection(i18n("Isomorphism signature: ") + tri->isoSig().c_str());
+
     // Offer a dehydration string if we have one.
     std::string dehydration = tri->dehydrate();
     if (! dehydration.empty())
@@ -232,19 +228,21 @@ void NTriCompositionUI::refresh() {
 
     // Expand so that two levels of children are visible.
     bool foundInnerChildren = false;
-    for (QListViewItem* topChild = details->firstChild(); topChild;
-            topChild = topChild->nextSibling())
-        if (topChild->firstChild()) {
-            topChild->setOpen(true);
+    QTreeWidgetItem* rootItem = details->invisibleRootItem();
+    for (int i=0; i < rootItem->childCount(); i++ ) {
+
+        QTreeWidgetItem* topChild = rootItem->child(i);
+        if (topChild->childCount() > 0) {
+            topChild->setExpanded(true);
             foundInnerChildren = true;
         }
-
+    }
     details->setRootIsDecorated(foundInnerChildren);
 }
 
 void NTriCompositionUI::editingElsewhere() {
     details->clear();
-    new KListViewItem(details, i18n("Editing..."));
+    (new QTreeWidgetItem(details))->setText(0,i18n("Editing..."));
     details->setRootIsDecorated(false);
 }
 
@@ -350,21 +348,27 @@ void NTriCompositionUI::viewIsomorphism() {
     KMessageBox::informationList(ui, msg, details, title);
 }
 
-QListViewItem* NTriCompositionUI::addTopLevelSection(const QString& text) {
-    if (details->lastItem())
-        return new KListViewItem(details, details->lastItem(), text);
-    else
-        return new KListViewItem(details, text);
+QTreeWidgetItem* NTriCompositionUI::addTopLevelSection(const QString& text) {
+    QTreeWidgetItem* newItem = new QTreeWidgetItem(details);
+    newItem->setText(0,text);
+    return newItem;
+    //if (details->lastItem())
+    //    return new QTreeWidgetItem(details, details->lastItem(), text);
+    //else
+    //    return new QTreeWidgetItem(details, text);
 }
 
-QListViewItem* NTriCompositionUI::addComponentSection(const QString& text) {
+QTreeWidgetItem* NTriCompositionUI::addComponentSection(const QString& text) {
     if (! components)
         components = addTopLevelSection(i18n("Components"));
 
-    if (lastComponent)
-        lastComponent = new KListViewItem(components, lastComponent, text);
-    else
-        lastComponent = new KListViewItem(components, text);
+    //if (lastComponent)
+    //    lastComponent = new QTreeWidgetItem(components, lastComponent, text);
+    //else
+    //    lastComponent = new QTreeWidgetItem(components, text);
+    
+    lastComponent = new QTreeWidgetItem(components);
+    lastComponent->setText(0,text);
 
     return lastComponent;
 }
@@ -372,8 +376,8 @@ QListViewItem* NTriCompositionUI::addComponentSection(const QString& text) {
 void NTriCompositionUI::findAugTriSolidTori() {
     unsigned long nComps = tri->getNumberOfComponents();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NAugTriSolidTorus* aug;
     for (unsigned long i = 0; i < nComps; i++) {
@@ -383,11 +387,12 @@ void NTriCompositionUI::findAugTriSolidTori() {
             id = addComponentSection(i18n(
                 "Augmented triangular solid torus ") + aug->getName().c_str());
 
-            details = new KListViewItem(id, i18n("Component %1").arg(i));
+            details = new QTreeWidgetItem(id);
+            details->setText(0,i18n("Component %1").arg(i));
 
             const regina::NTriSolidTorus& core = aug->getCore();
-            details = new KListViewItem(id, details,
-                i18n("Core: tets %1, %2, %3").
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0,i18n("Core: tets %1, %2, %3").
                 arg(tri->tetrahedronIndex(core.getTetrahedron(0))).
                 arg(tri->tetrahedronIndex(core.getTetrahedron(1))).
                 arg(tri->tetrahedronIndex(core.getTetrahedron(2))));
@@ -403,12 +408,13 @@ void NTriCompositionUI::findAugTriSolidTori() {
                 else
                     chainType = i18n("unknown");
 
-                details = new KListViewItem(id, details,
-                    i18n("Attached: layered chain (%1) + layered solid torus").
+                details = new QTreeWidgetItem(id, details);
+                details->setText(0,i18n("Attached: layered chain (%1) + "
+                    "layered solid torus").
                     arg(chainType));
             } else
-                details = new KListViewItem(id, details,
-                    i18n("Attached: 3 layered solid tori"));
+                details = new QTreeWidgetItem(id, details);
+                details->setText(0,i18n("Attached: 3 layered solid tori"));
 
             delete aug;
         }
@@ -416,9 +422,9 @@ void NTriCompositionUI::findAugTriSolidTori() {
 }
 
 void NTriCompositionUI::describeSatRegion(const NSatRegion& region,
-        QListViewItem* parent) {
-    QListViewItem* details;
-    QListViewItem* annuli;
+        QTreeWidgetItem* parent) {
+    QTreeWidgetItem* details;
+    QTreeWidgetItem* annuli;
 
     regina::NSatBlockSpec spec;
     regina::NSatAnnulus ann;
@@ -428,18 +434,20 @@ void NTriCompositionUI::describeSatRegion(const NSatRegion& region,
     QString thisAnnulus, adjAnnulus;
     for (b = region.numberOfBlocks() - 1; b >= 0; b--) {
         spec = region.block(b);
-        details = new KListViewItem(parent, i18n("Block %1: %2").
+        details = new QTreeWidgetItem(parent);
+        details->setText(0,i18n("Block %1: %2").
             arg(b).arg(spec.block->getAbbr().c_str()));
 
         nAnnuli = spec.block->nAnnuli();
 
-        annuli = new KListViewItem(details, i18n("Adjacencies:"));
+        annuli = new QTreeWidgetItem(details);
+        annuli->setText(0,i18n("Adjacencies:"));
 
         for (a = nAnnuli - 1; a >= 0; a--) {
             thisAnnulus = i18n("Annulus %1/%2").arg(b).arg(a);
             if (! spec.block->hasAdjacentBlock(a))
-                new KListViewItem(annuli, i18n("%1 --> boundary").
-                    arg(thisAnnulus));
+                (new QTreeWidgetItem(annuli))->setText(0,
+                    i18n("%1 --> boundary").arg(thisAnnulus));
             else {
                 adjAnnulus = i18n("Annulus %1/%2").
                     arg(region.blockIndex(spec.block->adjacentBlock(a))).
@@ -448,31 +456,36 @@ void NTriCompositionUI::describeSatRegion(const NSatRegion& region,
                 back = spec.block->adjacentBackwards(a);
 
                 if (ref && back)
-                    new KListViewItem(annuli,
+                    (new QTreeWidgetItem(annuli))->setText(0,
                         i18n("%1 --> %2 (reflected, backwards)").
                         arg(thisAnnulus).arg(adjAnnulus));
                 else if (ref)
-                    new KListViewItem(annuli, i18n("%1 --> %2 (reflected)").
+                    (new QTreeWidgetItem(annuli))->setText(0,
+                        i18n("%1 --> %2 (reflected)").
                         arg(thisAnnulus).arg(adjAnnulus));
                 else if (back)
-                    new KListViewItem(annuli, i18n("%1 --> %2 (backwards)").
+                    (new QTreeWidgetItem(annuli))->setText(0,
+                        i18n("%1 --> %2 (backwards)").
                         arg(thisAnnulus).arg(adjAnnulus));
                 else
-                    new KListViewItem(annuli, i18n("%1 --> %2").
+                    (new QTreeWidgetItem(annuli))->setText(0,
+                        i18n("%1 --> %2").
                         arg(thisAnnulus).arg(adjAnnulus));
             }
         }
 
-        if (nAnnuli == 1)
-            annuli = new KListViewItem(details, i18n("1 annulus"));
-        else
-            annuli = new KListViewItem(details, i18n("%1 annuli").arg(nAnnuli));
-
+        if (nAnnuli == 1) {
+            annuli = new QTreeWidgetItem(details);
+            annuli->setText(0,i18n("1 annulus"));
+        } else {
+            annuli = new QTreeWidgetItem(details);
+            annuli->setText(0,i18n("%1 annuli").arg(nAnnuli));
+        }
         for (a = nAnnuli - 1; a >= 0; a--) {
             thisAnnulus = i18n("Annulus %1/%2").arg(b).arg(a);
             ann = spec.block->annulus(a);
 
-            new KListViewItem(annuli,
+            (new QTreeWidgetItem(annuli))->setText(0,
                 i18n("%1 : Tet %2 (%3%4%5), Tet %6 (%7%8%9)").
                 arg(thisAnnulus).
                 arg(tri->tetrahedronIndex(ann.tet[0])).
@@ -486,22 +499,26 @@ void NTriCompositionUI::describeSatRegion(const NSatRegion& region,
         }
 
         if (spec.refVert && spec.refHoriz)
-            new KListViewItem(details,
+            (new QTreeWidgetItem(details))->setText(0,
                 i18n("Reflected vertically and horizontally"));
         else if (spec.refVert)
-            new KListViewItem(details, i18n("Reflected vertically"));
+            (new QTreeWidgetItem(details))->setText(0,
+                i18n("Reflected vertically"));
         else if (spec.refHoriz)
-            new KListViewItem(details, i18n("Reflected horizontally"));
+            (new QTreeWidgetItem(details))->setText(0,
+                i18n("Reflected horizontally"));
         else
-            new KListViewItem(details, i18n("No reflections"));
+            (new QTreeWidgetItem(details))->setText(0,
+                i18n("No reflections"));
 
-        new KListViewItem(details, spec.block->toString().c_str());
+        (new QTreeWidgetItem(details))->setText(0,
+            spec.block->toString().c_str());
     }
 }
 
 void NTriCompositionUI::findBlockedTriangulations() {
-    QListViewItem* id;
-    QListViewItem* details;
+    QTreeWidgetItem* id;
+    QTreeWidgetItem* details;
 
     regina::NBlockedSFS* sfs = regina::NBlockedSFS::isBlockedSFS(tri);
     if (sfs) {
@@ -515,10 +532,11 @@ void NTriCompositionUI::findBlockedTriangulations() {
     if (loop) {
         id = addComponentSection(i18n("Blocked SFS Loop"));
 
-        details = new KListViewItem(id, i18n("Internal region:"));
+        details = new QTreeWidgetItem(id);
+        details->setText(0,i18n("Internal region:"));
         describeSatRegion(loop->region(), details);
 
-        new KListViewItem(id, i18n("Matching relation: %1").
+        (new QTreeWidgetItem(id))->setText(0, i18n("Matching relation: %1").
             arg(matrixString(loop->matchingReln())));
 
         delete loop;
@@ -529,13 +547,15 @@ void NTriCompositionUI::findBlockedTriangulations() {
     if (pair) {
         id = addComponentSection(i18n("Blocked SFS Pair"));
 
-        details = new KListViewItem(id, i18n("Second region:"));
+        details = new QTreeWidgetItem(id);
+        details->setText(0, i18n("Second region:"));
         describeSatRegion(pair->region(1), details);
 
-        details = new KListViewItem(id, i18n("First region:"));
+        details = new QTreeWidgetItem(id);
+        details->setText(0, i18n("First region:"));
         describeSatRegion(pair->region(0), details);
 
-        new KListViewItem(id, i18n("Matching relation (first --> second): %1").
+        (new QTreeWidgetItem(id))->setText(0, i18n("Matching relation (first --> second): %1").
             arg(matrixString(pair->matchingReln())));
 
         delete pair;
@@ -546,20 +566,23 @@ void NTriCompositionUI::findBlockedTriangulations() {
     if (triple) {
         id = addComponentSection(i18n("Blocked SFS Triple"));
 
-        details = new KListViewItem(id, i18n("Second end region:"));
+        details = new QTreeWidgetItem(id);
+        details->setText(0, i18n("Second end region:"));
         describeSatRegion(triple->end(1), details);
 
-        details = new KListViewItem(id, i18n("First end region:"));
+        details = new QTreeWidgetItem(id);
+        details->setText(0, i18n("First end region:"));
         describeSatRegion(triple->end(0), details);
 
-        details = new KListViewItem(id, i18n("Central region:"));
+        details = new QTreeWidgetItem(id);
+        details->setText(0, i18n("Central region:"));
         describeSatRegion(triple->centre(), details);
 
-        new KListViewItem(id,
+        (new QTreeWidgetItem(id))->setText(0,
             i18n("Matching relation (centre --> second end): %1").
             arg(matrixString(triple->matchingReln(1))));
 
-        new KListViewItem(id,
+        (new QTreeWidgetItem(id))->setText(0,
             i18n("Matching relation (centre --> first end): %1").
             arg(matrixString(triple->matchingReln(0))));
 
@@ -571,17 +594,17 @@ void NTriCompositionUI::findBlockedTriangulations() {
     if (bundle) {
         id = addComponentSection(i18n("Layered Torus Bundle"));
 
-        new KListViewItem(id,
+        (new QTreeWidgetItem(id))->setText(0,
             i18n("Layering relation (lower a/b --> upper a/b): %1").
             arg(matrixString(bundle->layeringReln())));
 
-        new KListViewItem(id,
+        (new QTreeWidgetItem(id))->setText(0,
             i18n("Core relation (upper a/b --> lower a/b): %1").
             arg(matrixString(bundle->core().parallelReln())));
 
-        new KListViewItem(id,
+        (new QTreeWidgetItem(id))->setText(0,
             i18n("Core T x I triangulation: %1").
-            arg(bundle->core().getName()));
+            arg(bundle->core().getName().c_str()));
 
         delete bundle;
     }
@@ -591,16 +614,17 @@ void NTriCompositionUI::findBlockedTriangulations() {
     if (pBundle) {
         id = addComponentSection(i18n("Plugged Torus Bundle"));
 
-        details = new KListViewItem(id, i18n("Saturated region:"));
+        details = new QTreeWidgetItem(id);
+        details->setText(0, i18n("Saturated region:"));
         describeSatRegion(pBundle->region(), details);
 
-        new KListViewItem(id,
+        (new QTreeWidgetItem(id))->setText(0,
             i18n("Matching relation (joining region boundaries): %1").
             arg(matrixString(pBundle->matchingReln())));
 
-        new KListViewItem(id,
+        (new QTreeWidgetItem(id))->setText(0,
             i18n("Thin I-bundle (T x I): %1").
-            arg(pBundle->bundle().getName()));
+            arg(pBundle->bundle().getName().c_str()));
 
         delete pBundle;
     }
@@ -609,8 +633,8 @@ void NTriCompositionUI::findBlockedTriangulations() {
 void NTriCompositionUI::findL31Pillows() {
     unsigned long nComps = tri->getNumberOfComponents();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NL31Pillow* pillow;
     for (unsigned long i = 0; i < nComps; i++) {
@@ -619,9 +643,11 @@ void NTriCompositionUI::findL31Pillows() {
             id = addComponentSection(i18n("L(3,1) pillow ") +
                 pillow->getName().c_str());
 
-            details = new KListViewItem(id, i18n("Component %1").arg(i));
+            details = new QTreeWidgetItem(id);
+            details->setText(0, i18n("Component %1").arg(i));
 
-            details = new KListViewItem(id, details,
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, 
                 i18n("Pillow interior vertex: %1").
                 arg(tri->vertexIndex(
                     pillow->getTetrahedron(0)->getVertex(
@@ -635,8 +661,8 @@ void NTriCompositionUI::findL31Pillows() {
 void NTriCompositionUI::findLayeredChainPairs() {
     unsigned long nComps = tri->getNumberOfComponents();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NLayeredChainPair* pair;
     for (unsigned long i = 0; i < nComps; i++) {
@@ -646,9 +672,11 @@ void NTriCompositionUI::findLayeredChainPairs() {
             id = addComponentSection(i18n("Layered chain pair ") +
                 pair->getName().c_str());
 
-            details = new KListViewItem(id, i18n("Component %1").arg(i));
+            details = new QTreeWidgetItem(id);
+            details->setText(0, i18n("Component %1").arg(i));
 
-            details = new KListViewItem(id, details,
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0,
                 i18n("Chain lengths: %1, %2").
                 arg(pair->getChain(0)->getIndex()).
                 arg(pair->getChain(1)->getIndex()));
@@ -661,8 +689,8 @@ void NTriCompositionUI::findLayeredChainPairs() {
 void NTriCompositionUI::findLayeredLensSpaces() {
     unsigned long nComps = tri->getNumberOfComponents();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NLayeredLensSpace* lens;
     for (unsigned long i = 0; i < nComps; i++) {
@@ -672,10 +700,12 @@ void NTriCompositionUI::findLayeredLensSpaces() {
             id = addComponentSection(i18n("Layered lens space ") +
                 lens->getName().c_str());
 
-            details = new KListViewItem(id, i18n("Component %1").arg(i));
+            details = new QTreeWidgetItem(id);
+            details->setText(0, i18n("Component %1").arg(i));
 
             const regina::NLayeredSolidTorus& torus(lens->getTorus());
-            details = new KListViewItem(id, details, i18n(
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n(
                 "Layered %1-%2-%3 solid torus %4").
                 arg(torus.getMeridinalCuts(0)).
                 arg(torus.getMeridinalCuts(1)).
@@ -691,8 +721,8 @@ void NTriCompositionUI::findLayeredLensSpaces() {
 void NTriCompositionUI::findLayeredLoops() {
     unsigned long nComps = tri->getNumberOfComponents();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NLayeredLoop* loop;
     for (unsigned long i = 0; i < nComps; i++) {
@@ -701,18 +731,23 @@ void NTriCompositionUI::findLayeredLoops() {
             id = addComponentSection(i18n("Layered loop ") +
                 loop->getName().c_str());
 
-            details = new KListViewItem(id, i18n("Component %1").arg(i));
+            details = new QTreeWidgetItem(id);
+            details->setText(0, i18n("Component %1").arg(i));
 
             if (loop->isTwisted()) {
-                details = new KListViewItem(id, details, i18n(
+                details = new QTreeWidgetItem(id, details);
+                details->setText(0, i18n(
                     "Length %1, twisted").arg(loop->getLength()));
-                details = new KListViewItem(id, details, i18n(
+                details = new QTreeWidgetItem(id, details);
+                details->setText(0, i18n(
                     "Hinge: edge %1").
                     arg(tri->edgeIndex(loop->getHinge(0))));
             } else {
-                details = new KListViewItem(id, details, i18n(
+                details = new QTreeWidgetItem(id, details);
+                details->setText(0, i18n(
                     "Length %1, not twisted").arg(loop->getLength()));
-                details = new KListViewItem(id, details, i18n(
+                details = new QTreeWidgetItem(id);
+                details->setText(0, i18n(
                     "Hinges: edge %1, %2").
                     arg(tri->edgeIndex(loop->getHinge(0))).
                     arg(tri->edgeIndex(loop->getHinge(1))));
@@ -726,8 +761,8 @@ void NTriCompositionUI::findLayeredLoops() {
 void NTriCompositionUI::findLayeredSolidTori() {
     unsigned long nTets = tri->getNumberOfTetrahedra();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NLayeredSolidTorus* torus;
     unsigned long topIndex;
@@ -738,21 +773,26 @@ void NTriCompositionUI::findLayeredSolidTori() {
             id = addComponentSection(i18n("Layered solid torus ") +
                 torus->getName().c_str());
 
-            details = new KListViewItem(id, i18n("Base: tet %1").arg(
+            details = new QTreeWidgetItem(id);
+            details->setText(0, i18n("Base: tet %1").arg(
                 tri->tetrahedronIndex(torus->getBase())));
             topIndex = tri->tetrahedronIndex(torus->getTopLevel());
-            details = new KListViewItem(id, details, i18n("Top level: tet %1").
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n("Top level: tet %1").
                 arg(topIndex));
 
-            details = new KListViewItem(id, details, i18n(
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n(
                 "Weight %1 edge: %2").arg(torus->getMeridinalCuts(0)).
                 arg(edgeString(topIndex, torus->getTopEdge(0, 0),
                     torus->getTopEdge(0, 1))));
-            details = new KListViewItem(id, details, i18n(
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n(
                 "Weight %1 edge: %2").arg(torus->getMeridinalCuts(1)).
                 arg(edgeString(topIndex, torus->getTopEdge(1, 0),
                     torus->getTopEdge(1, 1))));
-            details = new KListViewItem(id, details, i18n(
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n(
                 "Weight %1 edge: %2").arg(torus->getMeridinalCuts(2)).
                 arg(edgeString(topIndex, torus->getTopEdge(2, 0),
                     torus->getTopEdge(2, 1))));
@@ -765,8 +805,8 @@ void NTriCompositionUI::findLayeredSolidTori() {
 void NTriCompositionUI::findPillowSpheres() {
     unsigned long nFaces = tri->getNumberOfFaces();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     unsigned long i, j;
     regina::NFace* f1;
@@ -780,10 +820,12 @@ void NTriCompositionUI::findPillowSpheres() {
             if (pillow) {
                 id = addComponentSection(i18n("Pillow 2-sphere"));
 
-                details = new KListViewItem(id, i18n("Faces: %1, %2").
+                details = new QTreeWidgetItem(id);
+                details->setText(0, i18n("Faces: %1, %2").
                     arg(i).arg(j));
 
-                details = new KListViewItem(id, details, i18n(
+                details = new QTreeWidgetItem(id, details);
+                details->setText(0, i18n(
                     "Equator: edges %1, %2, %3").
                      arg(tri->edgeIndex(f1->getEdge(0))).
                      arg(tri->edgeIndex(f1->getEdge(1))).
@@ -798,8 +840,8 @@ void NTriCompositionUI::findPillowSpheres() {
 void NTriCompositionUI::findPlugTriSolidTori() {
     unsigned long nComps = tri->getNumberOfComponents();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NPlugTriSolidTorus* plug;
     const regina::NLayeredChain* chain;
@@ -810,10 +852,12 @@ void NTriCompositionUI::findPlugTriSolidTori() {
             id = addComponentSection(i18n("Plugged triangular solid torus ") +
                 plug->getName().c_str());
 
-            details = new KListViewItem(id, i18n("Component %1").arg(i));
+            details = new QTreeWidgetItem(id);
+            details->setText(0, i18n("Component %1").arg(i));
 
             const regina::NTriSolidTorus& core(plug->getCore());
-            details = new KListViewItem(id, details,
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0,
                 i18n("Core: tets %1, %2, %3").
                 arg(tri->tetrahedronIndex(core.getTetrahedron(0))).
                 arg(tri->tetrahedronIndex(core.getTetrahedron(1))).
@@ -832,10 +876,11 @@ void NTriCompositionUI::findPlugTriSolidTori() {
                 if (j < 2)
                     lengths += ", ";
             }
-            details = new KListViewItem(id, details, lengths);
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, lengths);
 
-            details = new KListViewItem(id, details,
-                i18n("Equator type: ") +
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n("Equator type: ") +
                 (plug->getEquatorType() ==
                 regina::NPlugTriSolidTorus::EQUATOR_MAJOR ?
                 i18n("major") : i18n("minor")));
@@ -848,8 +893,8 @@ void NTriCompositionUI::findPlugTriSolidTori() {
 void NTriCompositionUI::findSnappedBalls() {
     unsigned long nTets = tri->getNumberOfTetrahedra();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NSnappedBall* ball;
     for (unsigned long i = 0; i < nTets; i++) {
@@ -858,9 +903,11 @@ void NTriCompositionUI::findSnappedBalls() {
         if (ball) {
             id = addComponentSection(i18n("Snapped 3-ball"));
 
-            details = new KListViewItem(id, i18n("Tetrahedron %1").arg(i));
+            details = new QTreeWidgetItem(id);
+            details->setText(0, i18n("Tetrahedron %1").arg(i));
 
-            details = new KListViewItem(id, details, i18n("Equator: edge %1%2").
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n("Equator: edge %1%2").
                 arg(ball->getInternalFace(0)).arg(ball->getInternalFace(1)));
 
             delete ball;
@@ -871,8 +918,8 @@ void NTriCompositionUI::findSnappedBalls() {
 void NTriCompositionUI::findSnappedSpheres() {
     unsigned long nTets = tri->getNumberOfTetrahedra();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     unsigned long i, j;
     regina::NTetrahedron* t1;
@@ -886,11 +933,13 @@ void NTriCompositionUI::findSnappedSpheres() {
             if (sphere) {
                 id = addComponentSection(i18n("Snapped 2-sphere"));
 
-                details = new KListViewItem(id, i18n("Tetrahedra: %1, %2").
+                details = new QTreeWidgetItem(id);
+                details->setText(0, i18n("Tetrahedra: %1, %2").
                     arg(i).arg(j));
 
                 const regina::NSnappedBall* ball = sphere->getSnappedBall(0);
-                details = new KListViewItem(id, details, i18n(
+                details = new QTreeWidgetItem(id, details);
+                details->setText(0, i18n(
                     "Equator: edge %1").arg(tri->edgeIndex(
                     ball->getTetrahedron()->getEdge(ball->getEquatorEdge()))));
 
@@ -903,8 +952,8 @@ void NTriCompositionUI::findSnappedSpheres() {
 void NTriCompositionUI::findSpiralSolidTori() {
     unsigned long nTets = tri->getNumberOfTetrahedra();
 
-    QListViewItem* id = 0;
-    QListViewItem* details = 0;
+    QTreeWidgetItem* id = 0;
+    QTreeWidgetItem* details = 0;
 
     regina::NSpiralSolidTorus* spiral;
     regina::NTetrahedron* tet;
@@ -913,11 +962,11 @@ void NTriCompositionUI::findSpiralSolidTori() {
     for (i = 0; i < nTets; i++) {
         tet = tri->getTetrahedron(i);
         for (whichPerm = 0; whichPerm < 24 /* size of S4 */; ++whichPerm) {
-            if (NPerm::S4[whichPerm][0] > NPerm::S4[whichPerm][3])
+            if (NPerm4::S4[whichPerm][0] > NPerm4::S4[whichPerm][3])
                 continue;
 
             spiral = regina::NSpiralSolidTorus::formsSpiralSolidTorus(tet,
-                NPerm::S4[whichPerm]);
+                NPerm4::S4[whichPerm]);
             if (! spiral)
                 continue;
             if (! spiral->isCanonical(tri)) {
@@ -941,11 +990,13 @@ void NTriCompositionUI::findSpiralSolidTori() {
                     tetSet += ", ";
                 tetSet += QString::number(tetIndex[j]);
             }
-            details = new KListViewItem(id, tetSet);
+            details = new QTreeWidgetItem(id);
+            details->setText(0, tetSet);
 
             QString data;
-            QListViewItem* edge;
-            details = new KListViewItem(id, details, i18n("Major edges:"));
+            QTreeWidgetItem* edge;
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n("Major edges:"));
             edge = 0;
             for (j = 0; j < spiralTets; j++) {
                 data =
@@ -958,12 +1009,15 @@ void NTriCompositionUI::findSpiralSolidTori() {
                     edgeString(tetIndex[(j + 1) % spiralTets],
                         spiral->getVertexRoles((j + 1) % spiralTets), 0, 1);
                 if (edge)
-                    edge = new KListViewItem(details, edge, data);
+                    edge = new QTreeWidgetItem(details, edge);
                 else
-                    edge = new KListViewItem(details, data);
+                    edge = new QTreeWidgetItem(details);
+
+                edge->setText(0, data);
             }
 
-            details = new KListViewItem(id, details, i18n("Minor edges:"));
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n("Minor edges:"));
             edge = 0;
             for (j = 0; j < spiralTets; j++) {
                 data =
@@ -972,20 +1026,25 @@ void NTriCompositionUI::findSpiralSolidTori() {
                     edgeString(tetIndex[(j + 1) % spiralTets],
                         spiral->getVertexRoles((j + 1) % spiralTets), 0, 2);
                 if (edge)
-                    edge = new KListViewItem(details, edge, data);
+                    edge = new QTreeWidgetItem(details, edge);
                 else
-                    edge = new KListViewItem(details, data);
+                    edge = new QTreeWidgetItem(details);
+
+                edge->setText(0, data);
             }
 
-            details = new KListViewItem(id, details, i18n("Axis edges:"));
+            details = new QTreeWidgetItem(id, details);
+            details->setText(0, i18n("Axis edges:"));
             edge = 0;
             for (j = 0; j < spiralTets; j++) {
                 data = edgeString(tetIndex[j], spiral->getVertexRoles(j),
                     0, 3);
                 if (edge)
-                    edge = new KListViewItem(details, edge, data);
+                    edge = new QTreeWidgetItem(details, edge);
                 else
-                    edge = new KListViewItem(details, data);
+                    edge = new QTreeWidgetItem(details);
+
+                edge->setText(0, data);
             }
 
             delete spiral;
@@ -1008,7 +1067,7 @@ QString NTriCompositionUI::edgeString(unsigned long tetIndex,
 }
 
 QString NTriCompositionUI::edgeString(unsigned long tetIndex,
-        const regina::NPerm& roles, int startPreimage, int endPreimage) {
+        const regina::NPerm4& roles, int startPreimage, int endPreimage) {
     return QString("%1 (%2%3)").arg(tetIndex).arg(roles[startPreimage]).
         arg(roles[endPreimage]);
 }
@@ -1018,17 +1077,3 @@ QString NTriCompositionUI::matrixString(const regina::NMatrix2& matrix) {
         arg(matrix[0][0]).arg(matrix[0][1]).arg(matrix[1][0]).arg(matrix[1][1]);
 }
 
-void NTriCompositionUI::detailsPopup(QListViewItem* item, const QPoint& pos,
-        int) {
-    if (item) {
-        detailsLastSelection = item->text(0);
-        detailsMenu->popup(pos);
-    }
-}
-
-void NTriCompositionUI::detailsCopy() {
-    QApplication::clipboard()->setText(detailsLastSelection,
-        QClipboard::Clipboard);
-}
-
-#include "ntricomposition.moc"
