@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -40,17 +40,15 @@
 #include <klocale.h>
 #include <kprocess.h>
 #include <kstandarddirs.h>
-#include <ktempfile.h>
+#include <KTemporaryFile>
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qmessagebox.h>
 #include <qpushbutton.h>
-#include <qscrollview.h>
-#include <qtooltip.h>
-#include <qwhatsthis.h>
-#include <qwidgetstack.h>
+#include <QScrollArea>
+#include <QStackedWidget>
 
 using regina::NPacket;
 using regina::NTriangulation;
@@ -78,14 +76,15 @@ NTriSkelCompUI::NTriSkelCompUI(regina::NTriangulation* packet,
     QBoxLayout* layout = new QVBoxLayout(ui);
     layout->addStretch(1);
 
-    QGridLayout* grid = new QGridLayout(layout, 3, 13, 5 /* spacing */);
-    grid->setColStretch(0, 1);
-    grid->setColSpacing(2, 5);
-    grid->setColSpacing(4, 10);
-    grid->setColSpacing(6, 10);
-    grid->setColSpacing(8, 5);
-    grid->setColSpacing(10, 10);
-    grid->setColStretch(12, 1);
+    QGridLayout* grid = new QGridLayout();
+    layout->addLayout(grid);
+    grid->setColumnStretch(0, 1);
+    grid->setColumnMinimumWidth(2, 5);
+    grid->setColumnMinimumWidth(4, 10);
+    grid->setColumnMinimumWidth(6, 10);
+    grid->setColumnMinimumWidth(8, 5);
+    grid->setColumnMinimumWidth(10, 10);
+    grid->setColumnStretch(12, 1);
 
     QLabel* label;
     QPushButton* btn;
@@ -97,8 +96,8 @@ NTriSkelCompUI::NTriSkelCompUI(regina::NTriangulation* packet,
     nVertices->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     grid->addWidget(nVertices, 0, 3);
     msg = i18n("The total number of vertices in this triangulation.");
-    QWhatsThis::add(label, msg);
-    QWhatsThis::add(nVertices, msg);
+    label->setWhatsThis(msg);
+    nVertices->setWhatsThis(msg);
 
     label = new QLabel(i18n("Edges:"), ui);
     grid->addWidget(label, 1, 1);
@@ -106,8 +105,8 @@ NTriSkelCompUI::NTriSkelCompUI(regina::NTriangulation* packet,
     nEdges ->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     grid->addWidget(nEdges, 1, 3);
     msg = i18n("The total number of edges in this triangulation.");
-    QWhatsThis::add(label, msg);
-    QWhatsThis::add(nEdges, msg);
+    label->setWhatsThis(msg);
+    nEdges->setWhatsThis(msg);
 
     label = new QLabel(i18n("Faces:"), ui);
     grid->addWidget(label, 2, 1);
@@ -115,8 +114,8 @@ NTriSkelCompUI::NTriSkelCompUI(regina::NTriangulation* packet,
     nFaces->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     grid->addWidget(nFaces, 2, 3);
     msg = i18n("The total number of faces in this triangulation.");
-    QWhatsThis::add(label, msg);
-    QWhatsThis::add(nFaces, msg);
+    label->setWhatsThis(msg);
+    nFaces->setWhatsThis(msg);
 
     label = new QLabel(i18n("Components:"), ui);
     grid->addWidget(label, 0, 7);
@@ -125,8 +124,8 @@ NTriSkelCompUI::NTriSkelCompUI(regina::NTriangulation* packet,
     grid->addWidget(nComps, 0, 9);
     msg = i18n("The total number of connected components in this "
         "triangulation.");
-    QWhatsThis::add(label, msg);
-    QWhatsThis::add(nComps, msg);
+    label->setWhatsThis(msg);
+    nComps->setWhatsThis(msg);
 
     label = new QLabel(i18n("Bdry Components:"), ui);
     grid->addWidget(label, 1, 7);
@@ -136,8 +135,8 @@ NTriSkelCompUI::NTriSkelCompUI(regina::NTriangulation* packet,
     msg = i18n("The total number of boundary components in this "
         "triangulation.  Boundary components can either be ideal vertices "
         "or collections of adjacent boundary faces.");
-    QWhatsThis::add(label, msg);
-    QWhatsThis::add(nBdryComps, msg);
+    label->setWhatsThis(msg);
+    nBdryComps->setWhatsThis(msg);
 
     label = new QLabel(i18n("Tetrahedra:"), ui);
     grid->addWidget(label, 2, 7);
@@ -145,44 +144,44 @@ NTriSkelCompUI::NTriSkelCompUI(regina::NTriangulation* packet,
     nTets->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     grid->addWidget(nTets, 2, 9);
     msg = i18n("The total number of tetrahedra in this triangulation.");
-    QWhatsThis::add(label, msg);
-    QWhatsThis::add(nTets, msg);
+    label->setWhatsThis(msg);
+    nTets->setWhatsThis(msg);
 
-    btn = new QPushButton(SmallIconSet("viewmag"), i18n("View..."), ui);
+    btn = new QPushButton(KIcon("packet_view"), i18n("View..."), ui);
     // btn->setFlat(true);
-    QToolTip::add(btn, i18n("View details of individual vertices"));
-    QWhatsThis::add(btn, i18n("View details of this triangulation's "
+    btn->setToolTip(i18n("View details of individual vertices"));
+    btn->setWhatsThis(i18n("View details of this triangulation's "
         "individual vertices in a separate window."));
     connect(btn, SIGNAL(clicked()), this, SLOT(viewVertices()));
     grid->addWidget(btn, 0, 5);
 
-    btn = new QPushButton(SmallIconSet("viewmag"), i18n("View..."), ui);
-    QToolTip::add(btn, i18n("View details of individual edges"));
-    QWhatsThis::add(btn, i18n("View details of this triangulation's "
+    btn = new QPushButton(KIcon("packet_view"), i18n("View..."), ui);
+    btn->setToolTip(i18n("View details of individual edges"));
+    btn->setWhatsThis(i18n("View details of this triangulation's "
         "individual edges in a separate window."));
     // btn->setFlat(true);
     connect(btn, SIGNAL(clicked()), this, SLOT(viewEdges()));
     grid->addWidget(btn, 1, 5);
 
-    btn = new QPushButton(SmallIconSet("viewmag"), i18n("View..."), ui);
-    QToolTip::add(btn, i18n("View details of individual faces"));
-    QWhatsThis::add(btn, i18n("View details of this triangulation's "
+    btn = new QPushButton(KIcon("packet_view"), i18n("View..."), ui);
+    btn->setToolTip(i18n("View details of individual faces"));
+    btn->setWhatsThis(i18n("View details of this triangulation's "
         "individual faces in a separate window."));
     // btn->setFlat(true);
     connect(btn, SIGNAL(clicked()), this, SLOT(viewFaces()));
     grid->addWidget(btn, 2, 5);
 
-    btn = new QPushButton(SmallIconSet("viewmag"), i18n("View..."), ui);
-    QToolTip::add(btn, i18n("View details of individual components"));
-    QWhatsThis::add(btn, i18n("View details of this triangulation's "
+    btn = new QPushButton(KIcon("packet_view"), i18n("View..."), ui);
+    btn->setToolTip(i18n("View details of individual components"));
+    btn->setWhatsThis(i18n("View details of this triangulation's "
         "individual connected components in a separate window."));
     // btn->setFlat(true);
     connect(btn, SIGNAL(clicked()), this, SLOT(viewComponents()));
     grid->addWidget(btn, 0, 11);
 
-    btn = new QPushButton(SmallIconSet("viewmag"), i18n("View..."), ui);
-    QToolTip::add(btn, i18n("View details of individual boundary components"));
-    QWhatsThis::add(btn, i18n("View details of this triangulation's "
+    btn = new QPushButton(KIcon("packet_view"), i18n("View..."), ui);
+    btn->setToolTip(i18n("View details of individual boundary components"));
+    btn->setWhatsThis(i18n("View details of this triangulation's "
         "individual boundary components in a separate window.  Note that "
         "boundary components can either be ideal vertices of collections "
         "of adjacent boundary faces."));
@@ -191,8 +190,6 @@ NTriSkelCompUI::NTriSkelCompUI(regina::NTriangulation* packet,
     grid->addWidget(btn, 1, 11);
 
     layout->addStretch(1);
-
-    viewers.setAutoDelete(true);
 }
 
 regina::NPacket* NTriSkelCompUI::getPacket() {
@@ -210,9 +207,10 @@ void NTriSkelCompUI::refresh() {
     nTets->setText(QString::number(tri->getNumberOfTetrahedra()));
     nComps->setText(QString::number(tri->getNumberOfComponents()));
     nBdryComps->setText(QString::number(tri->getNumberOfBoundaryComponents()));
-
-    for (SkeletonWindow* win = viewers.first(); win; win = viewers.next())
-        win->refresh();
+    
+    QLinkedListIterator<SkeletonWindow*> it(viewers);
+    while( it.hasNext())
+        (it.next())->refresh();
 }
 
 void NTriSkelCompUI::editingElsewhere() {
@@ -223,11 +221,16 @@ void NTriSkelCompUI::editingElsewhere() {
     nComps->setText(i18n("Editing..."));
     nBdryComps->setText(i18n("Editing..."));
 
-    for (SkeletonWindow* win = viewers.first(); win; win = viewers.next())
-        win->editingElsewhere();
+    QLinkedListIterator<SkeletonWindow*> it(viewers);
+    while( it.hasNext())
+        (it.next())->editingElsewhere();
 }
 
 void NTriSkelCompUI::viewVertices() {
+    // Because we pass this as parent to the new window, we are
+    // guaranteed that the window will be closed and deleted
+    // automatically if the packet pane is closed.
+    // Similarly for edges, faces, etc.
     SkeletonWindow* win = new SkeletonWindow(this, SkeletonWindow::Vertices);
     win->show();
     viewers.append(win);
@@ -265,27 +268,32 @@ NTriFaceGraphUI::NTriFaceGraphUI(regina::NTriangulation* packet,
         graphvizExec(useGraphvizExec) {
     ui = new QWidget();
     QBoxLayout* baseLayout = new QVBoxLayout(ui);
-    stack = new QWidgetStack(ui);
+    stack = new QStackedWidget(ui);
 
     // Information layer.
-    layerInfo = messageLayer(msgInfo, "messagebox_info");
+    layerInfo = messageLayer(msgInfo, "dialog-information");
     msgInfo->setText(i18n("<qt>Initialising...</qt>"));
 
     // Error layer.
-    layerError = messageLayer(msgError, "messagebox_critical");
+    layerError = messageLayer(msgError, "dialog-error");
     msgError->setText(i18n("<qt>Initialising...</qt>"));
 
     // Graph layer.
-    layerGraph = new QScrollView(stack);
+    layerGraph = new QScrollArea();
+    // Don't set transparency: a border and lighter background looks
+    // kind of nice here.
+    // layerGraph->setStyleSheet("QScrollArea, .QWidget { "
+    //                             "background-color:transparent; "
+    //                         "}");
     graph = new QLabel(layerGraph);
-    graph->setAlignment(AlignCenter);
-    layerGraph->addChild(graph);
-
-    QWhatsThis::add(layerGraph, i18n("<qt>The <i>face pairing graph</i> "
+    graph->setAlignment(Qt::AlignCenter);
+    layerGraph->setWidget(graph);
+    layerGraph->setWhatsThis(i18n("<qt>The <i>face pairing graph</i> "
         "of a triangulation describes which tetrahedron faces are "
         "identified with which.<p>Each vertex of the graph represents "
         "a tetrahedron, and each edge represents a pair of tetrahedron "
         "faces that are joined together.</qt>"));
+    stack->addWidget(layerGraph);
 
     // Finish off.
     baseLayout->addWidget(stack);
@@ -390,15 +398,20 @@ void NTriFaceGraphUI::refresh() {
         return;
     }
 
-    KTempFile tmpDot(locateLocal("tmp", "fpg-"), ".dot");
+    KTemporaryFile tmpDot;
+    tmpDot.setSuffix(".dot");
+    if (! tmpDot.open()) {
+        showError(i18n("<qt>The temporary DOT file <i>%1</i> "
+            "could not be created.</qt>").arg(tmpDot.fileName()));
+        return;
+    }
     tmpDot.close();
 
     std::ofstream outDot(
-        static_cast<const char*>(QFile::encodeName(tmpDot.name())));
+        static_cast<const char*>(QFile::encodeName(tmpDot.fileName())));
     if (! outDot) {
         showError(i18n("<qt>The temporary DOT file <i>%1</i> "
-            "could not be opened for writing.</qt>").arg(tmpDot.name()));
-        tmpDot.unlink();
+            "could not be opened for writing.</qt>").arg(tmpDot.fileName()));
         return;
     }
 
@@ -407,55 +420,47 @@ void NTriFaceGraphUI::refresh() {
     outDot.close();
     delete pairing;
 
-    KTempFile tmpPng(locateLocal("tmp", "fpg-"), ".png");
+    KTemporaryFile tmpPng;
+    tmpPng.setSuffix(".png");
+    if (! tmpPng.open()) {
+        showError(i18n("<qt>The temporary PNG file <i>%1</i> "
+            "could not be created.</qt>").arg(tmpPng.fileName()));
+        return;
+    }
     tmpPng.close();
 
     KProcess graphviz;
     graphviz << useExec << "-Tpng" << "-Gsize=2.5,4"
-        << "-o" << tmpPng.name() << tmpDot.name();
-    if (! graphviz.start(KProcess::Block)) {
+        << "-o" << tmpPng.fileName() << tmpDot.fileName();
+    graphviz.start();
+    graphviz.waitForFinished();
+    if ( graphviz.exitStatus() != QProcess::NormalExit) {
+        if ( graphviz.error() == QProcess::FailedToStart ) {
+            showError(i18n("<qt>The Graphviz executable <i>%1</i> "
+                "could not be started.</qt>").arg(useExec));
+            return;
+        }
         showError(i18n("<qt>The Graphviz executable <i>%1</i> "
-            "could not be started.</qt>").arg(useExec));
-        tmpDot.unlink();
-        tmpPng.unlink();
-        return;
-    }
-    if (graphviz.signalled()) {
-        showError(i18n("<qt>The Graphviz executable <i>%1</i> "
-            "did not exit normally.  It was killed with signal %2.</qt>").
-            arg(useExec).arg(graphviz.exitSignal()));
-        tmpDot.unlink();
-        tmpPng.unlink();
-        return;
-    }
-    if ((! graphviz.normalExit()) || graphviz.exitStatus()) {
-        showError(i18n("<qt>The Graphviz executable <i>%1</i> "
-            "appears to have encountered an internal error.  "
-            "It finished with exit status %2.</qt>").
-            arg(useExec).arg(graphviz.exitStatus()));
-        tmpDot.unlink();
-        tmpPng.unlink();
+            "did not exit normally, and may have encountered an "
+            "internal error.  It finished with exit status %2.</qt>")
+            .arg(useExec).arg(graphviz.exitCode()));
         return;
     }
 
-    QPixmap png(tmpPng.name());
+    QPixmap png(tmpPng.fileName());
     if (png.isNull()) {
         showError(i18n("<qt>The PNG graphic created by Graphviz "
             "could not be loaded.<p>The Graphviz executable used "
             "was <i>%1</i>.  If this is not correct, please change it "
             "in the Regina configuration (Triangulation section).</qt>").
             arg(useExec));
-        tmpDot.unlink();
-        tmpPng.unlink();
         return;
     }
 
     graph->setPixmap(png);
+    graph->resize(graph->sizeHint());
 
-    tmpDot.unlink();
-    tmpPng.unlink();
-
-    stack->raiseWidget(layerGraph);
+    stack->setCurrentWidget(layerGraph);
 }
 
 void NTriFaceGraphUI::editingElsewhere() {
@@ -464,42 +469,40 @@ void NTriFaceGraphUI::editingElsewhere() {
 
 QWidget* NTriFaceGraphUI::messageLayer(QLabel*& text,
         const char* iconName) {
-    QWidget* layer = new QWidget(stack);
-    QBoxLayout* layout = new QHBoxLayout(layer, 5 /* margin */,
-        5 /* spacing */);
+    QWidget* layer = new QWidget();
+    QBoxLayout* layout = new QHBoxLayout(layer);
 
     layout->addStretch(1);
 
-    QPixmap iconPic = enclosingPane->getPart()->instance()->iconLoader()->
-        loadIcon(iconName, KIcon::NoGroup, KIcon::SizeMedium,
-        KIcon::DefaultState, 0, true /* may be null */);
+    QPixmap iconPic = KIconLoader::global()->
+        loadIcon(iconName, KIconLoader::NoGroup, KIconLoader::SizeMedium,
+        KIconLoader::DefaultState, QStringList(), 0, true /* may be null */);
     if (iconPic.isNull())
         iconPic = QMessageBox::standardIcon(QMessageBox::Critical);
 
     QLabel* icon = new QLabel(layer);
     icon->setPixmap(iconPic);
-    layout->addWidget(icon);
-    layout->setStretchFactor(icon, 0);
+    layout->addWidget(icon, 0);
 
     layout->addSpacing(10);
 
     text = new QLabel(i18n("<qt>Initialising...</qt>"), layer);
-    layout->addWidget(text);
-    layout->setStretchFactor(text, 4);
+    text->setWordWrap(true);
+    layout->addWidget(text, 4);
 
     layout->addStretch(1);
+    stack->addWidget(layer);
 
     return layer;
 }
 
 void NTriFaceGraphUI::showInfo(const QString& msg) {
     msgInfo->setText(msg);
-    stack->raiseWidget(layerInfo);
+    stack->setCurrentWidget(layerInfo);
 }
 
 void NTriFaceGraphUI::showError(const QString& msg) {
     msgError->setText(msg);
-    stack->raiseWidget(layerError);
+    stack->setCurrentWidget(layerError);
 }
 
-#include "ntriskeleton.moc"
