@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -39,8 +39,7 @@
 #include <klocale.h>
 #include <qlabel.h>
 #include <qlayout.h>
-#include <qwhatsthis.h>
-#include <qwidgetstack.h>
+#include <QStackedWidget>
 
 using regina::NPacket;
 using regina::NSnapPeaTriangulation;
@@ -62,50 +61,51 @@ NTriSnapPeaUI::NTriSnapPeaUI(regina::NTriangulation* packet,
 
     layout->addStretch(1);
 
-    data = new QWidgetStack(ui);
+    data = new QStackedWidget(ui);
 
     // Data for a null SnapPea triangulation:
 
-    dataNull = new QWidget(data);
-    QBoxLayout* nullLayout = new QVBoxLayout(dataNull, 5 /* margin */,
-        0 /* spacing */);
+    dataNull = new QWidget();
+    QBoxLayout* nullLayout = new QVBoxLayout(dataNull);
 
-    unavailable = new NoSnapPea(reginaTri, allowClosed, dataNull, 0, true);
+    unavailable = new NoSnapPea(reginaTri, allowClosed, dataNull, true);
     unavailable->setAlignment(Qt::AlignCenter);
     nullLayout->addWidget(unavailable);
+    data->addWidget(dataNull);
 
     // Data for a non-null SnapPea triangulation:
 
-    dataValid = new QWidget(data);
-    QGridLayout* validGrid = new QGridLayout(dataValid, 2 /* rows */, 5, 5);
-    validGrid->setColStretch(0, 1);
-    validGrid->setColSpacing(2, 5); // Horizontal gap
-    validGrid->setColStretch(4, 1);
+    dataValid = new QWidget();
+    QGridLayout* validGrid = new QGridLayout(dataValid);
+    validGrid->setColumnStretch(0, 1);
+    validGrid->setColumnMinimumWidth(2, 5); // Horizontal gap
+    validGrid->setColumnStretch(4, 1);
 
     QString msg;
-    int align = Qt::AlignTop | Qt::AlignAuto | Qt::ExpandTabs;
 
     solutionTypeLabel = new QLabel(i18n("Solution type:"), dataValid);
-    solutionTypeLabel->setAlignment(align);
+    solutionTypeLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     validGrid->addWidget(solutionTypeLabel, 0, 1);
     solutionType = new QLabel(dataValid);
-    solutionType->setAlignment(align);
+    solutionType->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     validGrid->addWidget(solutionType, 0, 3);
     solutionTypeExplnBase = i18n("The type of solution that was found "
         "when solving for a complete hyperbolic structure.");
-    QWhatsThis::add(solutionTypeLabel, solutionTypeExplnBase);
-    QWhatsThis::add(solutionType, solutionTypeExplnBase);
+    solutionTypeLabel->setWhatsThis(solutionTypeExplnBase);
+    solutionType->setWhatsThis(solutionTypeExplnBase);
 
     label = new QLabel(i18n("Volume:"), dataValid);
-    label->setAlignment(align);
+    label->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     validGrid->addWidget(label, 1, 1);
     volume = new QLabel(dataValid);
-    volume->setAlignment(align);
+    volume->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     validGrid->addWidget(volume, 1, 3);
     msg = i18n("The volume of the underlying 3-manifold.  The estimated "
         "number of decimal places of accuracy is also shown.");
-    QWhatsThis::add(label, msg);
-    QWhatsThis::add(volume, msg);
+    label->setWhatsThis(msg);
+    volume->setWhatsThis(msg);
+
+    data->addWidget(dataValid);
 
     // Finish off.
 
@@ -133,18 +133,18 @@ void NTriSnapPeaUI::refresh() {
     snappeaTri = new NSnapPeaTriangulation(*reginaTri, allowClosed);
 
     if (snappeaTri->isNull()) {
-        data->raiseWidget(dataNull);
+        data->setCurrentWidget(dataNull);
         unavailable->refresh(allowClosed);
     } else {
-        data->raiseWidget(dataValid);
+        data->setCurrentWidget(dataValid);
 
         solutionType->setText(solutionTypeString(snappeaTri->solutionType()));
         solutionType->setEnabled(true);
 
         QString expln = i18n("%1  %2").arg(solutionTypeExplnBase)
             .arg(solutionTypeExplanation(snappeaTri->solutionType()));
-        QWhatsThis::add(solutionTypeLabel, expln);
-        QWhatsThis::add(solutionType, expln);
+        solutionTypeLabel->setWhatsThis(expln);
+        solutionType->setWhatsThis(expln);
 
         int places;
         double ans = snappeaTri->volume(places);
@@ -181,14 +181,14 @@ void NTriSnapPeaUI::refresh() {
 }
 
 void NTriSnapPeaUI::editingElsewhere() {
-    data->raiseWidget(dataValid);
+    data->setCurrentWidget(dataValid);
 
     QString msg(i18n("Editing..."));
 
     solutionType->setText(msg);
     solutionType->setEnabled(false);
-    QWhatsThis::add(solutionTypeLabel, solutionTypeExplnBase);
-    QWhatsThis::add(solutionType, solutionTypeExplnBase);
+    solutionTypeLabel->setWhatsThis(solutionTypeExplnBase);
+    solutionType->setWhatsThis(solutionTypeExplnBase);
 
     volume->setText(msg);
     volume->setEnabled(false);
@@ -249,4 +249,3 @@ QString NTriSnapPeaUI::solutionTypeExplanation(int solnType) {
     }
 }
 
-#include "ntrisnappea.moc"
