@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -35,22 +35,21 @@
 
 #include "packet/npacketlistener.h"
 
-#include <qptrlist.h>
-#include <qvbox.h>
+#include <QFrame>
+#include <QLinkedList>
+#include <kactionmenu.h>
 
 class KAction;
 class KActionMenu;
 class KMainWindow;
+class PacketEditIface;
 class PacketPane;
+class PacketWindow;
 class QLabel;
 class QToolButton;
+class QTreeWidget;
 class ReginaPart;
 
-namespace KTextEditor {
-    class Document;
-    class View;
-}
-
 namespace regina {
     class NPacket;
 };
@@ -58,7 +57,7 @@ namespace regina {
 /**
  * A packet header, containing an appropriate icon and text title.
  */
-class PacketHeader : public QHBox {
+class PacketHeader : public QFrame {
     Q_OBJECT
 
     private:
@@ -77,8 +76,7 @@ class PacketHeader : public QHBox {
         /**
          * Constructor.
          */
-        PacketHeader(regina::NPacket* pkt, QWidget* parent = 0,
-            const char* name = 0);
+        PacketHeader(regina::NPacket* pkt, QWidget* parent = 0);
 
     public slots:
         /**
@@ -139,14 +137,13 @@ class PacketUI {
         virtual QWidget* getInterface() = 0;
 
         /**
-         * Return the primary text component associated with this
-         * interface, or 0 if there is no primary text component.
-         * This routine should always return the same pointer throughout
-         * the life of this object.
+         * Return details of the interface's interaction with standard
+         * edit and clipboard operations.  This may be 0 if there is no
+         * such interaction.
          *
-         * The default implementation of this routine simply returns 0.
+         * The default implementation simply returns 0.
          */
-        virtual KTextEditor::Document* getTextComponent();
+        virtual PacketEditIface* getEditIface();
 
         /**
          * Return a list of actions specific to the particular type of
@@ -157,7 +154,7 @@ class PacketUI {
          * The default implementation of this routine simply returns an
          * empty list.
          */
-        virtual const QPtrList<KAction>& getPacketTypeActions();
+        virtual const QLinkedList<KAction*>& getPacketTypeActions();
 
         /**
          * Return the label of the menu that should contain the actions
@@ -230,7 +227,7 @@ class PacketUI {
         /**
          * An empty action list.
          */
-        static QPtrList<KAction> noActions;
+        static QLinkedList<KAction*> noActions;
 };
 
 /**
@@ -304,23 +301,15 @@ class DefaultPacketUI : public ErrorPacketUI {
  * Packet panes may be either docked within the main ReginaPart widget
  * or may be floating freely in their own frames.
  */
-class PacketPane : public QVBox, public regina::NPacketListener {
+class PacketPane : public QWidget, public regina::NPacketListener {
     Q_OBJECT
 
-    public:
-        /**
-         * The list of edit operations for which actions can be
-         * registered and deregistered.
-         */
-        enum EditOperation { editCut, editCopy, editPaste,
-            editUndo, editRedo };
-
     private:
         /**
          * External components
          */
         ReginaPart* part;
-        KMainWindow* frame;
+        PacketWindow* frame;
 
         /**
          * Internal components
@@ -346,17 +335,14 @@ class PacketPane : public QVBox, public regina::NPacketListener {
         KAction* actRefresh;
         KAction* actDockUndock;
         KAction* actClose;
-        KAction* actSeparator;
         KActionMenu* packetTypeMenu;
 
         /**
-         * Externally registered actions
+         * Externally registered edit actions and their sources
          */
-        KAction* extCut;
-        KAction* extCopy;
-        KAction* extPaste;
-        KAction* extUndo;
-        KAction* extRedo;
+        KAction* editCut;
+        KAction* editCopy;
+        KAction* editPaste;
 
     public:
         /**
@@ -367,14 +353,13 @@ class PacketPane : public QVBox, public regina::NPacketListener {
          * by way of the PacketManager class.
          */
         PacketPane(ReginaPart* newPart, regina::NPacket* newPacket,
-            QWidget* parent = 0, const char* name = 0);
+            QWidget* parent = 0);
         ~PacketPane();
 
         /**
          * Query components and actions.
          */
         regina::NPacket* getPacket();
-        bool hasTextComponent();
         KActionMenu* getPacketTypeMenu();
         ReginaPart* getPart();
 
@@ -432,44 +417,29 @@ class PacketPane : public QVBox, public regina::NPacketListener {
         bool queryClose();
 
         /**
-         * Registers or deregisters standard editor actions with the
-         * primary text component of this interface.  If this interface
-         * has no primary text component then these routines do nothing.
+         * Registers or deregisters standard editor actions to operate
+         * on this packet interface.
          *
          * Registered actions will be connected to appropriate edit
-         * operations in the text component, and will be enabled and disabled
-         * over time according to the current status of the packet pane
-         * and its primary text component.
+         * operations in this interface, and will be enabled and disabled
+         * over time according to the current status of the internal UI
+         * components.
          *
-         * When an action is deregistered, these relationships will be
-         * broken and the action will be left in a disabled state.
+         * When the actions are deregistered, these relationships will be
+         * broken and the actions will be left in a disabled state.
          *
-         * Only one action for each operation may be registered at a
-         * time.  Each registered action for an operation should be
-         * deregistered before a new action is registered in its place.
-         *
-         * The only exception to this rule is if a currently registered
-         * action is to be destroyed; in this case a new action (which
-         * may be a null pointer) may simply be registered over top of
-         * it, though this must be done before the action is destroyed
-         * or else a crash will occur.
+         * Only one set of editor actions may be registered at a time.
+         * Any attempt to register a new set of actions will
+         * automatically deregister any previously registered actions.
          *
          * When a packet pane is destroyed, it is not a requirement that
          * currently registered actions be deregistered beforehand,
          * though the final enabled/disabled status of any remaining
          * actions that are not deregistered is not guaranteed.
-         *
-         * Passing a null pointer to a registration routine will not
-         * register any new actions.  The only situation in which doing
-         * this might be useful is when an action is to be destroyed but is
-         * not being deregistered (see the earlier paragraph that
-         * describes this case).
-         *
-         * Passing a null pointer to a deregistration routine will have
-         * no effect whatsoever.
          */
-        void registerEditOperation(KAction* act, EditOperation op);
-        void deregisterEditOperation(KAction* act, EditOperation op);
+        void registerEditOperations(KAction* actCut, KAction* actCopy,
+            KAction* actPaste);
+        void deregisterEditOperations();
 
         /**
          * NPacketListener overrides.
@@ -613,13 +583,12 @@ class PacketPane : public QVBox, public regina::NPacketListener {
          * actions.  These slots are for internal use.
          */
         void updateClipboardActions();
-        void updateUndoActions();
 
     protected:
         /**
          * Allow GUI updates from within a non-GUI thread.
          */
-        void customEvent(QCustomEvent* evt);
+        void customEvent(QEvent* evt);
 };
 
 inline PacketUI::PacketUI(PacketPane* newEnclosingPane) :
@@ -629,11 +598,11 @@ inline PacketUI::PacketUI(PacketPane* newEnclosingPane) :
 inline PacketUI::~PacketUI() {
 }
 
-inline KTextEditor::Document* PacketUI::getTextComponent() {
+inline PacketEditIface* PacketUI::getEditIface() {
     return 0;
 }
 
-inline const QPtrList<KAction>& PacketUI::getPacketTypeActions() {
+inline const QLinkedList<KAction*>& PacketUI::getPacketTypeActions() {
     return noActions;
 }
 
@@ -660,10 +629,6 @@ inline ReginaPart* PacketPane::getPart() {
     return part;
 }
 
-inline bool PacketPane::hasTextComponent() {
-    return mainUI->getTextComponent();
-}
-
 inline bool PacketPane::isDirty() {
     return dirty;
 }
