@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -33,9 +33,10 @@
 #ifndef __EXAMPLESACTION_H
 #define __EXAMPLESACTION_H
 
-#include <kactionclasses.h>
-#include <qstringlist.h>
-#include <qvaluevector.h>
+#include <kselectaction.h>
+#include <qmap.h>
+
+class KUrl;
 
 /**
  * An action offering a selection of sample data files that can be
@@ -49,24 +50,16 @@ class ExamplesAction : public KSelectAction {
     Q_OBJECT
 
     private:
-        /**
-         * GUI components
-         */
-        KPopupMenu* popup_;
-
         /**
          * Sample data files
          */
-        QValueVector<QString> urls_;
-        QStringList descs_;
+        QMap<QAction*, KUrl> urls_;
 
     public:
         /**
-         * Constructor and destructor.  The given slot is to be connected
-         * to urlSelected(const KURL&).
+         * Constructor and destructor.
          */
-        ExamplesAction(const QObject* receiver, const char* slot,
-            QObject* parent, const char* name = 0);
+        ExamplesAction(QObject* parent);
         virtual ~ExamplesAction();
 
         /**
@@ -74,32 +67,19 @@ class ExamplesAction : public KSelectAction {
          *
          * The filename should be relative to the Regina examples directory.
          */
-        void addURL(const QString& fileName, const QString& description);
-
-        /**
-         * KAction overrides.
-         */
-        virtual int plug(QWidget *widget, int index = -1);
+        void addUrl(const QString& fileName, const QString& description);
 
     signals:
         /**
          * Emitted when a sample data file is selected for opening.
          */
-        void urlSelected(const KURL& url);
+        void urlSelected(const KUrl& url);
 
     protected slots:
         /**
          * All activation events lead here.
          */
-        void exampleActivated(int index);
-
-        /**
-         * Internal slots for keeping the various GUI containers up-to-date
-         * and following their individual events.
-         */
-        void menuAboutToShow();
-        void slotClicked();
-        virtual void slotActivated();
+        void exampleActivated(QAction*);
 };
 
 #endif
