@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -35,14 +35,14 @@
 
 #include "reginaprefset.h"
 
-#include <qvbox.h>
-#include <kdialogbase.h>
+#include <kpagedialog.h>
 
 class CoordinateChooser;
-class KListView;
 class QCheckBox;
 class QComboBox;
+class QLabel;
 class QLineEdit;
+class QListWidget;
 class QPushButton;
 class ReginaMain;
 class ReginaPrefCensus;
@@ -60,7 +60,7 @@ namespace KTextEditor {
 /**
  * The Regina configuration dialog.
  */
-class ReginaPreferences : public KDialogBase {
+class ReginaPreferences : public KPageDialog {
     Q_OBJECT
 
     private:
@@ -94,16 +94,16 @@ class ReginaPreferences : public KDialogBase {
 /**
  * The page of the Regina configuration dialog for general preferences.
  */
-class ReginaPrefGeneral : public QVBox {
+class ReginaPrefGeneral : public QWidget {
     Q_OBJECT
 
     private:
         QCheckBox* cbAutoDock;
         QCheckBox* cbAutoFileExtension;
-        QCheckBox* cbDisplayIcon;
-        QCheckBox* cbDisplayTagsInTree;
+        // QCheckBox* cbDisplayTagsInTree;
         QLineEdit* editTreeJumpSize;
         QCheckBox* cbTipOfDay;
+        QCheckBox* cbHandbookInKHelpCenter;
 
     public:
         ReginaPrefGeneral(QWidget* parent = 0);
@@ -112,14 +112,13 @@ class ReginaPrefGeneral : public QVBox {
 };
 
 /**
- * The page of the Regina configuration dialog for triangulation
+ * The page of the Regina configuration dialog for 3-manifold triangulation
  * preferences.
  */
-class ReginaPrefTri : public QVBox {
+class ReginaPrefTri : public QWidget {
     Q_OBJECT
 
     private:
-        QComboBox* comboEditMode;
         QComboBox* comboInitialTab;
         QComboBox* comboInitialSkeletonTab;
         QComboBox* comboInitialAlgebraTab;
@@ -137,7 +136,7 @@ class ReginaPrefTri : public QVBox {
  * The page of the Regina configuration dialog for normal surface
  * preferences.
  */
-class ReginaPrefSurfaces : public QVBox {
+class ReginaPrefSurfaces : public QWidget {
     Q_OBJECT
 
     private:
@@ -157,7 +156,7 @@ class ReginaPrefSurfaces : public QVBox {
  * The page of the Regina configuration dialog for PDF viewing
  * preferences.
  */
-class ReginaPrefPDF : public QVBox {
+class ReginaPrefPDF : public QWidget {
     Q_OBJECT
 
     private:
@@ -174,12 +173,12 @@ class ReginaPrefPDF : public QVBox {
 /**
  * The page of the Regina configuration dialog for census preferences.
  */
-class ReginaPrefCensus : public QVBox {
+class ReginaPrefCensus : public QWidget {
     Q_OBJECT
 
     private:
         QLabel* activeCount;
-        KListView* listFiles;
+        QListWidget* listFiles;
         QPushButton* btnRemove;
         QPushButton* btnActivate;
         QPushButton* btnDeactivate;
@@ -203,7 +202,7 @@ class ReginaPrefCensus : public QVBox {
 /**
  * The page of the Regina configuration dialog for Python preferences.
  */
-class ReginaPrefPython : public QVBox {
+class ReginaPrefPython : public QWidget {
     Q_OBJECT
 
     private:
@@ -212,7 +211,7 @@ class ReginaPrefPython : public QVBox {
         QLineEdit* editSpacesPerTab;
 
         QLabel* activeCount;
-        KListView* listFiles;
+        QListWidget* listFiles;
         QPushButton* btnRemove;
         QPushButton* btnActivate;
         QPushButton* btnDeactivate;
@@ -236,11 +235,12 @@ class ReginaPrefPython : public QVBox {
  * The page of the Regina configuration dialog that controls interaction
  * with the SnapPea kernel.
  */
-class ReginaPrefSnapPea : public QVBox {
+class ReginaPrefSnapPea : public QWidget {
     Q_OBJECT
 
     private:
         QCheckBox* cbClosed;
+        QCheckBox* cbMessages;
 
     public:
         ReginaPrefSnapPea(QWidget* parent = 0);
@@ -253,7 +253,7 @@ class ReginaPrefSnapPea : public QVBox {
  *
  * Adapted from the KWrite sources, which are released under the LGPL.
  */
-class ReginaEditorChooser : public KDialogBase {
+class ReginaEditorChooser : public KDialog {
     Q_OBJECT
 
     private:
