@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,8 +26,12 @@
 
 /* end stub */
 
-#include "file/nglobaldirs.h"
+// We need to see Python.h first to avoid a "portability fix" in pyport.h
+// that breaks boost.python on MacOSX.
+#include "Python.h"
+
 #include <boost/python.hpp>
+#include "file/nglobaldirs.h"
 
 using namespace boost::python;
 using regina::NGlobalDirs;
@@ -39,11 +43,13 @@ void addNGlobalDirs() {
         .def("pythonLibs", &NGlobalDirs::pythonLibs)
         .def("examples", &NGlobalDirs::examples)
         .def("engineDocs", &NGlobalDirs::engineDocs)
+        .def("data", &NGlobalDirs::data)
         .staticmethod("home")
         .staticmethod("pythonModule")
         .staticmethod("pythonLibs")
         .staticmethod("examples")
         .staticmethod("engineDocs")
+        .staticmethod("data")
     ;
 }
 
