@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -28,6 +28,7 @@
 
 void addForeignCSVSurfaceList();
 void addForeignDehydration();
+void addForeignIsoSig();
 void addForeignOrb();
 void addForeignPDF();
 void addForeignSnapPea();
@@ -35,6 +36,7 @@ void addForeignSnapPea();
 void addForeign() {
     addForeignCSVSurfaceList();
     addForeignDehydration();
+    addForeignIsoSig();
     addForeignOrb();
     addForeignPDF();
     addForeignSnapPea();
