@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,7 +26,12 @@
 
 /* end stub */
 
-#include "triangulation/nperm.h"
+// We need to see Python.h first to avoid a "portability fix" in pyport.h
+// that breaks boost.python on MacOSX.
+#include "Python.h"
+
+#include "maths/nperm3.h"
+#include "maths/nperm4.h"
 #include "globalarray.h"
 
 void addGlobalArray() {
@@ -35,7 +40,8 @@ void addGlobalArray() {
     regina::python::GlobalArray3D<int>::wrapClass("GlobalArray3D_int");
     regina::python::GlobalArray<unsigned>::wrapClass("GlobalArray_unsigned");
     regina::python::GlobalArray<const char*>::wrapClass("GlobalArray_char_string");
-    regina::python::GlobalArray<regina::NPerm>::wrapClass("GlobalArray_NPerm");
-    regina::python::GlobalArray2D<regina::NPerm>::wrapClass("GlobalArray2D_NPerm");
+    regina::python::GlobalArray<regina::NPerm3>::wrapClass("GlobalArray_NPerm3");
+    regina::python::GlobalArray<regina::NPerm4>::wrapClass("GlobalArray_NPerm4");
+    regina::python::GlobalArray2D<regina::NPerm4>::wrapClass("GlobalArray2D_NPerm4");
 }
 
