@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,14 +26,9 @@
 
 /* end stub */
 
-#include "regina-config.h"
+#include <boost/python.hpp>
 #include "manifold/ngraphtriple.h"
 #include "manifold/nsfs.h"
-#include <boost/python.hpp>
-
-#ifdef BOOST_MAKE_CONSTRUCTOR_HACK
-#include <boost/python/make_constructor.hpp>
-#endif
 
 using namespace boost::python;
 using regina::NGraphTriple;
