@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -30,6 +30,8 @@ void addMatrixOps();
 void addNLargeInteger();
 void addNMatrix2();
 void addNMatrixInt();
+void addNPerm3();
+void addNPerm4();
 void addNPrimes();
 void addNRational();
 void addNumberTheory();
@@ -39,6 +41,8 @@ void addMaths() {
     addNLargeInteger();
     addNMatrix2();
     addNMatrixInt();
+    addNPerm3();
+    addNPerm4();
     addNPrimes();
     addNRational();
     addNumberTheory();
