@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -56,8 +56,8 @@ using regina::ShareableObject;
 namespace {
     std::string welcome() {
         return std::string(PACKAGE_STRING) +
-            "\nA Normal Surface Theory Calculator" +
-            "\nCopyright (c) 1999-2009, Ben Burton";
+            "\nSoftware for 3-manifold topology and normal surface theory" +
+            "\nCopyright (c) 1999-2011, The Regina development team";
     }
 
     void shareableWriteTextShort(const ShareableObject& obj) {
