@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -26,13 +26,14 @@
 
 /* end stub */
 
+#include <boost/python.hpp>
 #include "triangulation/ncomponent.h"
 #include "triangulation/nedge.h"
 #include "triangulation/nface.h"
 #include "triangulation/ntetrahedron.h"
+#include "triangulation/ntriangulation.h"
 #include "triangulation/nvertex.h"
 #include "../globalarray.h"
-#include <boost/python.hpp>
 
 using namespace boost::python;
 using regina::NTetrahedron;
@@ -59,6 +60,8 @@ void addNTetrahedron() {
         .def("unjoin", &NTetrahedron::unjoin,
             return_value_policy<reference_existing_object>())
         .def("isolate", &NTetrahedron::isolate)
+        .def("getTriangulation", &NTetrahedron::getTriangulation,
+            return_value_policy<reference_existing_object>())
         .def("getComponent", &NTetrahedron::getComponent,
             return_value_policy<reference_existing_object>())
         .def("getVertex", &NTetrahedron::getVertex,
@@ -67,6 +70,7 @@ void addNTetrahedron() {
             return_value_policy<reference_existing_object>())
         .def("getFace", &NTetrahedron::getFace,
             return_value_policy<reference_existing_object>())
+        .def("getVertexMapping", &NTetrahedron::getVertexMapping)
         .def("getEdgeMapping", &NTetrahedron::getEdgeMapping)
         .def("getFaceMapping", &NTetrahedron::getFaceMapping)
         .def("orientation", &NTetrahedron::orientation)
