@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Test Suite                                                            *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -31,6 +31,7 @@
 #include <cppunit/Test.h>
 #include <cppunit/TestResult.h>
 #include <cppunit/TextTestProgressListener.h>
+#include "testsuite/testparams.h"
 #include "testsuite/angle/testangle.h"
 #include "testsuite/census/testcensus.h"
 #include "testsuite/maths/testmaths.h"
@@ -100,6 +101,7 @@ class ReginaProgress : public CppUnit::TextTestProgressListener {
  */
 bool runAllTests() {
     std::cout << "Regina calculation engine test suite\n\n" << std::flush;
+    checkTestParams();
 
     CppUnit::TextUi::TestRunner runner;
     ReginaProgress progress;
@@ -112,17 +114,19 @@ bool runAllTests() {
      */
 
     // Utilities:
+    addNBitmask(runner);
     addNIndexedArray(runner);
     addNLargeInteger(runner);
     addNRational(runner);
 
     // Maths:
+    addNPerm3(runner);
+    addNPerm4(runner);
     addNPrimes(runner);
     addNumberTheory(runner);
     addMatrixOps(runner);
 
     // Triangulation:
-    addNPerm(runner);
     addNTriangulation(runner);
     addElementaryMoves(runner);
     addConnectedSumDecomp(runner);
