@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Test Suite                                                            *
  *                                                                        *
- *  Copyright (c) 1999-2009, Ben Burton                                   *
+ *  Copyright (c) 1999-2011, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -31,14 +31,13 @@
 #include <cppunit/extensions/HelperMacros.h>
 #include "manifold/nsimplesurfacebundle.h"
 #include "triangulation/nedge.h"
-#include "triangulation/nperm.h"
 #include "triangulation/ntetrahedron.h"
 #include "triangulation/ntriangulation.h"
 #include "triangulation/nexampletriangulation.h"
 #include "testsuite/triangulation/testtriangulation.h"
 
 using regina::NEdge;
-using regina::NPerm;
+using regina::NPerm4;
 using regina::NSimpleSurfaceBundle;
 using regina::NTetrahedron;
 using regina::NTriangulation;
@@ -64,18 +63,14 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
 
     public:
         void setUp() {
-            NTetrahedron* r = new NTetrahedron();
-            NTetrahedron* s = new NTetrahedron();
-            r->joinTo(0, s, NPerm());
-            r->joinTo(1, s, NPerm());
-            base.addTetrahedron(r);
-            base.addTetrahedron(s);
+            NTetrahedron* r = base.newTetrahedron();
+            NTetrahedron* s = base.newTetrahedron();
+            r->joinTo(0, s, NPerm4());
+            r->joinTo(1, s, NPerm4());
 
             baseKB.insertTriangulation(base);
             baseKB.getTetrahedron(0)->joinTo(2,
-                baseKB.getTetrahedron(1), NPerm(1, 2, 3, 0));
-
-            baseKB.gluingsHaveChanged();
+                baseKB.getTetrahedron(1), NPerm4(1, 2, 3, 0));
         }
 
         void tearDown() {
@@ -154,17 +149,9 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 NTriangulation t;
                 t.insertTriangulation(base);
                 NTetrahedron* tet = t.getTetrahedron(0);
-                tet->joinTo(2, tet, NPerm(2, 3));
-                NTetrahedron* tet2 = new NTetrahedron();
-                t.getTetrahedron(1)->joinTo(2, tet2, NPerm());
-                t.addTetrahedron(tet2);
-
-                // Force a recalculation of the skeleton, since
-                // NTetrahedron routines such as getEdge() are oblivious
-                // to the encompassing triangulation.  There has got to
-                // be a better way (TODO).  We do this several times
-                // here in this file.
-                t.getNumberOfEdges();
+                tet->joinTo(2, tet, NPerm4(2, 3));
+                NTetrahedron* tet2 = t.newTetrahedron();
+                t.getTetrahedron(1)->joinTo(2, tet2, NPerm4());
 
                 NEdge* e = tet->getEdge(NEdge::edgeNumber[0][1]);
                 CPPUNIT_ASSERT_MESSAGE(
@@ -173,7 +160,7 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                         t.isOrientable());
 
                 NTriangulation ball;
-                ball.addTetrahedron(new NTetrahedron());
+                ball.newTetrahedron();
 
                 verify20Edge(&t, 3, &ball, "boundary-loop-tet");
             }
@@ -183,10 +170,9 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 // an LST.
                 NTriangulation orig;
                 orig.insertLayeredSolidTorus(4, 7);
-                NTetrahedron* top = new NTetrahedron();
-                orig.getTetrahedron(0)->joinTo(2, top, NPerm(2, 3, 0, 1));
-                orig.getTetrahedron(0)->joinTo(3, top, NPerm(2, 3, 0, 1));
-                orig.addTetrahedron(top);
+                NTetrahedron* top = orig.newTetrahedron();
+                orig.getTetrahedron(0)->joinTo(2, top, NPerm4(2, 3, 0, 1));
+                orig.getTetrahedron(0)->joinTo(3, top, NPerm4(2, 3, 0, 1));
 
                 NTriangulation lst;
                 lst.insertLayeredSolidTorus(3, 4);
@@ -308,8 +294,7 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 NTriangulation t;
                 t.insertTriangulation(baseKB);
                 t.insertTriangulation(baseKB);
-                t.getTetrahedron(0)->joinTo(3, t.getTetrahedron(2), NPerm());
-                t.gluingsHaveChanged();
+                t.getTetrahedron(0)->joinTo(3, t.getTetrahedron(2), NPerm4());
 
                 verify20Edge(&t, 3, &baseKB, "bdry-cross-tet");
             }
@@ -339,18 +324,14 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 // both boundary.
                 NTriangulation t;
                 t.insertTriangulation(base);
-                NTetrahedron* p = new NTetrahedron();
-                NTetrahedron* q = new NTetrahedron();
-                NTetrahedron* r = new NTetrahedron();
-                NTetrahedron* s = new NTetrahedron();
-                t.getTetrahedron(0)->joinTo(2, p, NPerm());
-                t.getTetrahedron(0)->joinTo(3, q, NPerm());
-                t.getTetrahedron(1)->joinTo(2, r, NPerm());
-                t.getTetrahedron(1)->joinTo(3, s, NPerm());
-                t.addTetrahedron(p);
-                t.addTetrahedron(q);
-                t.addTetrahedron(r);
-                t.addTetrahedron(s);
+                NTetrahedron* p = t.newTetrahedron();
+                NTetrahedron* q = t.newTetrahedron();
+                NTetrahedron* r = t.newTetrahedron();
+                NTetrahedron* s = t.newTetrahedron();
+                t.getTetrahedron(0)->joinTo(2, p, NPerm4());
+                t.getTetrahedron(0)->joinTo(3, q, NPerm4());
+                t.getTetrahedron(1)->joinTo(2, r, NPerm4());
+                t.getTetrahedron(1)->joinTo(3, s, NPerm4());
 
                 verify20EdgeInvalid(&t, "boundary-edges");
             }
@@ -360,23 +341,16 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 // internal but identified (sphere).
                 NTriangulation t;
                 t.insertTriangulation(base);
-                NTetrahedron* p = new NTetrahedron();
-                NTetrahedron* q = new NTetrahedron();
-                NTetrahedron* r = new NTetrahedron();
-                NTetrahedron* s = new NTetrahedron();
-                t.getTetrahedron(0)->joinTo(2, p, NPerm());
-                t.getTetrahedron(0)->joinTo(3, q, NPerm());
-                t.getTetrahedron(1)->joinTo(2, r, NPerm());
-                t.getTetrahedron(1)->joinTo(3, s, NPerm());
-                p->joinTo(3, r, NPerm());
-                q->joinTo(2, s, NPerm());
-                t.addTetrahedron(p);
-                t.addTetrahedron(q);
-                t.addTetrahedron(r);
-                t.addTetrahedron(s);
-
-                // Ugly hack to regenerate skeleton, see notes above.
-                t.getNumberOfEdges();
+                NTetrahedron* p = t.newTetrahedron();
+                NTetrahedron* q = t.newTetrahedron();
+                NTetrahedron* r = t.newTetrahedron();
+                NTetrahedron* s = t.newTetrahedron();
+                t.getTetrahedron(0)->joinTo(2, p, NPerm4());
+                t.getTetrahedron(0)->joinTo(3, q, NPerm4());
+                t.getTetrahedron(1)->joinTo(2, r, NPerm4());
+                t.getTetrahedron(1)->joinTo(3, s, NPerm4());
+                p->joinTo(3, r, NPerm4());
+                q->joinTo(2, s, NPerm4());
 
                 NEdge* e = t.getTetrahedron(0)->getEdge(
                     NEdge::edgeNumber[0][1]);
@@ -393,23 +367,16 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 // internal but identified (RP2).
                 NTriangulation t;
                 t.insertTriangulation(base);
-                NTetrahedron* p = new NTetrahedron();
-                NTetrahedron* q = new NTetrahedron();
-                NTetrahedron* r = new NTetrahedron();
-                NTetrahedron* s = new NTetrahedron();
-                t.getTetrahedron(0)->joinTo(2, p, NPerm());
-                t.getTetrahedron(0)->joinTo(3, q, NPerm());
-                t.getTetrahedron(1)->joinTo(2, r, NPerm());
-                t.getTetrahedron(1)->joinTo(3, s, NPerm());
-                p->joinTo(3, r, NPerm(0, 1));
-                q->joinTo(2, s, NPerm(0, 1));
-                t.addTetrahedron(p);
-                t.addTetrahedron(q);
-                t.addTetrahedron(r);
-                t.addTetrahedron(s);
-
-                // Ugly hack to regenerate skeleton, see notes above.
-                t.getNumberOfEdges();
+                NTetrahedron* p = t.newTetrahedron();
+                NTetrahedron* q = t.newTetrahedron();
+                NTetrahedron* r = t.newTetrahedron();
+                NTetrahedron* s = t.newTetrahedron();
+                t.getTetrahedron(0)->joinTo(2, p, NPerm4());
+                t.getTetrahedron(0)->joinTo(3, q, NPerm4());
+                t.getTetrahedron(1)->joinTo(2, r, NPerm4());
+                t.getTetrahedron(1)->joinTo(3, s, NPerm4());
+                p->joinTo(3, r, NPerm4(0, 1));
+                q->joinTo(2, s, NPerm4(0, 1));
 
                 NEdge* e = t.getTetrahedron(0)->getEdge(
                     NEdge::edgeNumber[0][1]);
@@ -426,11 +393,7 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 NTriangulation t;
                 t.insertTriangulation(base);
                 NTetrahedron* tet = t.getTetrahedron(0);
-                tet->joinTo(2, tet, NPerm(2, 3));
-                t.gluingsHaveChanged();
-
-                // Ugly hack to regenerate skeleton, see notes above.
-                t.getNumberOfEdges();
+                tet->joinTo(2, tet, NPerm4(2, 3));
 
                 NEdge* e = tet->getEdge(NEdge::edgeNumber[0][1]);
                 CPPUNIT_ASSERT_MESSAGE(
@@ -446,9 +409,6 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 NTriangulation t;
                 t.insertTriangulation(baseKB);
 
-                // Ugly hack to regenerate skeleton, see notes above.
-                t.getNumberOfEdges();
-
                 NEdge* e = t.getTetrahedron(0)->
                     getEdge(NEdge::edgeNumber[0][1]);
                 CPPUNIT_ASSERT_MESSAGE(
@@ -491,9 +451,8 @@ class ElementaryMovesTest : public CppUnit::TestFixture {
                 orig.insertTriangulation(base);
 
                 NTetrahedron* top = orig.getTetrahedron(0);
-                orig.getTetrahedron(3)->joinTo(3, top, NPerm(1,2,0,3));
-                orig.getTetrahedron(4)->joinTo(2, top, NPerm(0,3,2,1));
-                orig.gluingsHaveChanged();
+                orig.getTetrahedron(3)->joinTo(3, top, NPerm4(1,2,0,3));
+                orig.getTetrahedron(4)->joinTo(2, top, NPerm4(0,3,2,1));
 
                 verify20EdgeInvalid(&orig, "boundary-diag");
             }
